; ****************************************************************************************
; **** LUCIDSCIENCE.COM - VGA DUAL BUFFER DEMO BY RADBRAD
;
; VGAGEN.asm
;
; erweitert und an geaenderte Hardware angepasst von Ronald Daleske 08/2012 - 10/2012
; Version 1.0
; ****************************************************************************************

; COMPILER SETTINGS
.INCLUDE "M644pDEF.INC" 



;Werte
.equ Fuellzeichen	= ' '

.equ Baudrate		=9600				; Baudrate
.equ F_CPU			=22000000			; CPU Frequenz in Hz
.equ UBRR_VAL		= ((F_CPU+Baudrate*8)/(Baudrate*16)-1)

.equ Max_X_Pixel	= 255			; Teilbarkeit beachten fuer CLEARSCREEN
.equ Max_Y_Pixel	= 240

.equ Max_X_Zeichen	= 42

.equ Max_X_Zeichen2	= 80

.equ Max_X_Show		= Max_X_Zeichen 

.equ Max_Y_Zeichen	= 30


.equ F_KHz_CPU		= F_CPU/1000		; CPU Frequenz in KHz
.equ CTC_WERT		= 31750*F_KHz_CPU/1000000


; Verwaltung der Ports
.equ 	port_adr_low	= porta
.equ 	port_adr_hig	= portb
.equ 	port_dat		= portc
.equ 	port_control	= portd

.equ 	ddr_adr_low		= ddra
.equ 	ddr_adr_hig		= ddrb
.equ 	ddr_dat			= ddrc
.equ 	ddr_control		= ddrd

.equ 	pin_dat			= pinc

; *** port_control
.equ 	HOR			= 2			; Horizontale Synchronimpulse Pin 13
.equ 	VER			= 3			; Vertikale Synchronimpulse Pin 14
.equ 	G			= 4			; /CE fuer 74HC244
.equ 	WE			= 5			; /WE fuer SRAM
.equ 	OE			= 6			; /OE fuer SRAM
.equ 	RAMPAGE		= 7			;  RAMPAGE fuer SRAM





; VERTICAL LINE CONTROL 
.equ 	VL_RESET	= 0			; 
.equ 	VL_VER_IMP	= 1			;
.equ 	VL_OUT_VA	= 2			;
.equ 	VL_INC_VA	= 3			;
.equ 	VL_MEMREADY	= 4			;



; Farben
.equ 	cl_White	= 0xFF			; %11111111
.equ 	cl_Black	= 0x00			; %00000000
.equ 	cl_Red		= 0x43			; %11000011
.equ 	cl_Green	= 0x4C			; %11001100
.equ 	cl_Blue		= 0xB0			; %11110000
.equ 	cl_Yellow	= 0x4F			; %11001100 !!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!! 
.equ 	cl_Purple	= 0x73			; %11001100 !!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!! 
.equ 	cl_Aqua		= 0x7C			; %11001100 !!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!


.equ 	cl_Gray		= 0xBF			; %10111111

.equ 	XON			= 0x11
.equ 	XOFF		= 0x13


.equ	START_CUR	= 0xFF			; 0xFF = Kursor ON; 0x00 = Kursor OFF;

; ****************************************************************************************
; **** Definieren der Register der Hauptschleife
; ****************************************************************************************


; *** Register R02 bis R15 ***

; 1. Register in der Hauptroutine
.def XPOS = r2			; 
.def YPOS = r3			; 

.def X_KUR_POS = r4		; 
.def Y_KUR_POS = r5		; 

.def X_PIX_POS = r6		; 
.def Y_PIX_POS = r7		; 

.def FCOLOR = r8		; Vordergrundfarbe 
.def BCOLOR = r9		; Hintergrundfarbe

.def RX_STOP = r10


; 2. Register in der Interruptroutine
.def POS_RXP_WR = r12
.def WERT_RXP = r13



; *** Register R16 bis R25 ***

; 1. Register in der Hauptroutine

.def par01 = r16		; 1. Unterprogramm-Parameter

.def temp1 = r17		; TEMP REGISTER 1 nur fuer Hauptroutine
.def temp2 = r18		; TEMP REGISTER 2
.def temp3 = r19		; TEMP REGISTER 3
.def temp4 = r20		; TEMP REGISTER 4

.def memready = r21		; MEMREADY FLAG

; 2. Register in der Interruptroutine

.def temp1i = r22		; TEMP REGISTER 1 nur fuer Interruptroutine
.def VER_ADR = r23
.def REG_YL = r24
.def REG_YH = r25





; TermMode -> 0=Normal_TM, 1=IBM_ESC_TM, 2=IBM_CSI_TM,
;             3=IBM_CSI_TM_Par, 4=IBM_CSI_TM_Par_Zeile 
.equ Normal_TM				= 0
.equ IBM_ESC_TM				= 1
.equ IBM_CSI_TM				= 2
.equ IBM_CSI_TM_Par			= 3
.equ IBM_CSI_TM_Par_Zeile	= 4
.equ IBM_CSI_TM_Par_Xterm	= 5


.equ 	CUR_CNT	= 40	; Zeit fuer ein Cursorblinken

.equ 	Laenge_RXP	= 200	; Laenge des UART-Puffers

.equ 	Laenge_RX_STOP	= 140	; etwa 3/4 der Laenge des UART-Puffers

.equ 	Laenge_ZEIP	= 8	; Laenge des Zeichen-Puffers

.equ 	Laenge_SRP	= Max_X_Pixel+1	; Laenge des SCREENROLL-Puffers


;****************************************************************************
;	RAM = Datensegment
;****************************************************************************

.dseg


CSI_P1:
		.db		0
CSI_P2:
		.db		0

; schaltet das Kursorblinken ein und aus, 00=aus, FF=ein
CUR_ON_OFF:
		.db		0

; Kursorzustand 00 oder FF -> Zeichen oder '_'
CUR_ST:
		.db		0

; Kursorzustand sichtbares Zeichen oder Steuerzeichen 
;      (bei Steuerzeichen darf der CUR_PUF nicht angezeigt werden) 
CUR_PUF_INHALT:
		.db		0

; Zaehler fuer das Kursorblinken
CUR_CNT_LO:
		.dw		0

CUR_CNT_HI:
		.dw		0

; Terminal-Mode
; 0=Normal_TM, 1=IBM_ESC_TM, 2=IBM_CSI_TM, 
; 3=IBM_CSI_TM_Par, 4=IBM_CSI_TM_Par_Zeile
TermMode:
		.db		0


; Position im Zeichen-Puffer
POS_ZEP:
		.db		0
		

; Position der Lesedaten im UART-Puffer 
POS_RXP_RD:
		.db		0


; Zeichen-Puffer fr die auszugebenden Zeichen
ZEI_PUF:
		.byte	Laenge_ZEIP		


; UART-Puffer fr die empfangenen Daten (RX)
RX_PUF:
		.byte	Laenge_RXP


; CUR_PUF - Abspeicherung des grafischen Zeichens an der Kursorposition
CUR_PUF:
		.byte	6*8





TP_CNT_ZL:
		.db		0	

TP_CNT_ZH:
		.db		0





;****************************************************************************
;	Interrupt Vektoren
;****************************************************************************


.cseg


.org 0
	rjmp	RESET
.org oc1aaddr
	rjmp	VIDEO		;2



RESET:






; STACK POINTER
	ldi		temp1,low(ramend)
	out		spl,temp1
	ldi		temp1,high(ramend)
	out		sph,temp1




; ****************************************************************************************
; **** IO PORT A SETUP
; ****************************************************************************************

	sbi		ddr_adr_low,0		; ADDRESS BIT 00
	sbi		ddr_adr_low,1		; ADDRESS BIT 01
	sbi		ddr_adr_low,2		; ADDRESS BIT 02
	sbi		ddr_adr_low,3		; ADDRESS BIT 03
	sbi		ddr_adr_low,4		; ADDRESS BIT 04
	sbi		ddr_adr_low,5		; ADDRESS BIT 05
	sbi		ddr_adr_low,6		; ADDRESS BIT 06
	sbi		ddr_adr_low,7		; ADDRESS BIT 07


 

; ****************************************************************************************
; **** IO PORT C SETUP
; ****************************************************************************************

	sbi		ddr_adr_hig,0		; ADDRESS BIT 08
	sbi		ddr_adr_hig,1		; ADDRESS BIT 09
	sbi		ddr_adr_hig,2		; ADDRESS BIT 12
	sbi		ddr_adr_hig,3		; ADDRESS BIT 11
	sbi		ddr_adr_hig,4		; ADDRESS BIT 12
	sbi		ddr_adr_hig,5		; ADDRESS BIT 13
	sbi		ddr_adr_hig,6		; ADDRESS BIT 14
	sbi		ddr_adr_hig,7		; ADDRESS BIT 15


 

; ****************************************************************************************
; **** IO PORT D SETUP
; ****************************************************************************************

	sbi		ddr_dat,0			; RED BIT 0
	sbi		ddr_dat,1			; RED BIT 1
	sbi		ddr_dat,2			; GRN BIT 0
	sbi		ddr_dat,3			; GRN BIT 1
	sbi		ddr_dat,4			; BLU BIT 0
	sbi		ddr_dat,5			; BLU BIT 1
	sbi		ddr_dat,6			; INT BIT 0
	sbi		ddr_dat,7			; INT BIT 1


 

; ****************************************************************************************
; **** IO PORT B SETUP
; ****************************************************************************************

	sbi		ddr_control,HOR	 	; HORIZONTAL SYNC PULSE
	sbi		ddr_control,VER	 	; VERTICAL SYNC PULSE
	sbi		ddr_control,G		; VIDEO DAC ENABLE
	sbi		ddr_control,WE		; SRAM WRITE ENABLE
	sbi		ddr_control,OE		; SRAM OUTPUT ENABLE
	sbi		ddr_control,RAMPAGE	; SRAM PAGE CONTROL



; Lschen der lokalen Datenregister

	clr		xl
	clr		xh
	clr		yl
	clr		yh
	clr		zl
	clr		zh
 
	clr		VER_ADR
	clr		REG_YL
	clr		REG_YH

	clr		POS_RXP_WR
	clr		RX_STOP



; Lschen der RAM-Register

	clr		temp1
	sts		POS_RXP_RD, temp1
	sts		POS_ZEP, temp1
	sts		CSI_P1, temp1
	sts		CSI_P2, temp1
	sts		CUR_PUF_INHALT, temp1
	sts		CUR_ST, temp1
	sts		CUR_CNT_LO, temp1


; Setzen Cursor-Zeit	
	ldi		temp1, CUR_CNT
	sts		CUR_CNT_HI, temp1


; Einschalten des Kursorblinkens
	ldi		temp1, START_CUR
	sts		CUR_ON_OFF, temp1


; Setzen des Terminal-Mode
	ldi		temp1, Normal_TM
	sts		TermMode,temp1



; Initialisieren der HintergrundDatenRegister
;
	clr		X_KUR_POS
	clr		Y_KUR_POS

	clr		X_PIX_POS
	clr		Y_PIX_POS

	ldi		temp1,cl_Black
	mov		FCOLOR,temp1
	ldi		temp1,cl_White
	mov		BCOLOR,temp1




; ****************************************************************************************
; **** STARTUP SEQUENCE
; ****************************************************************************************


; CTC Modus (Clear Timer on Compare Match mode) 
; ---------------------------------------------

; genutzt wird der 16-Bit Timer1

; TCCR1B = Timer/Counter control Register B des Timers1
; | ICNC1 | ICES1 | leer | WGM13 | WGM12 | CS12 | CS11 | CS10 |
; CS12  CS11  CS10  
;   0     0     1  = CPU-Takt ohne Teiler
;
; WGM12 = // CTC-Modus, TOP = OCR1A
;
; Timer 1, ein 16-Bit-Timer, wird mit einem Vorteiler von 1 betrieben. 
; Dadurch wird erreicht, dass der Timer mit hchster Zeitauflsung arbeiten kann. 
; Bei jedem Ticken des Systemtaktes von 4 MHz wird auch der Timer um 1 erhht. 
; Zustzlich wird noch das WGM12-Bit bei der Konfiguration gesetzt. 
; Dadurch wird der Timer in den CTC-Modus gesetzt. 
; Dabei wird der Inhalt des Timers hardwaremig mit dem Inhalt des OCR1A-Registers verglichen. 
; Stimmen beide berein, so wird der Timer auf 0 zurckgesetzt und im nchsten Taktzyklus ein OCIE1A-Interrupt ausgelst.
; Dadurch ist es mglich, exakt die Anzahl an Taktzyklen festzulegen, die von einem Interrupt zum nchsten vergehen sollen.


	ldi		temp1,(1<<CS10 | 1<<WGM12)
	sts		tccr1b,temp1




; Einstellen der Interrupt-Zeit
; alle 31,75 us oder 31750 ns 

	ldi		temp1,high(CTC_WERT)
	sts		ocr1ah,temp1
	ldi		temp1,low(CTC_WERT)
	sts		ocr1al,temp1



; TIMSK 
; | TICIE1 | OCIE1A | OCIE1B | TOIE1 |  
;
; TICIE1 - Timer 1 Input Capture Interrupt Enable
; OCIE1A - Timer 1 Output Compare A Match Interrupt Enable
; OCIE1B - Timer 1 Output Compare B Match Interrupt Enable
; TOIE1  - Timer 1 Overflow Interrupt Enable
;
; OCIE1A - Timer/Counter 1 Vergleichszahl A Interrupt - 1: Int bei Erreichen Stand in A
;
; SET TIMER1 MATCH INTERRUPT
	ldi		temp1,(1<<OCIE1A)
	sts		timsk1,temp1




; Initialisieren der seriellen Schnittstelle
; ---------------------------------------------


; festlegen der Baudrate fuer die serielle Schnittstelle
	ldi		temp1, HIGH(UBRR_VAL)
	sts		UBRR0H, temp1

	ldi		temp1, LOW(UBRR_VAL)
	sts		UBRR0L, temp1


; RX aktivieren
	lds		temp1, UCSR0B
    sbr		temp1, (1<<RXEN0)|(1<<TXEN0)		
    sts		UCSR0B, temp1	


; Set comms parameters ---> BITS = 8, PARITY = none, STOP = 1 
	ldi		temp1, (1<<UCSZ01) | (1<<UCSZ00) 
	sts		UCSR0C, temp1 



; aktivieren des SRAMs
	sbi		port_control,WE
	sbi		port_control,OE




; Lschen des Bildschirms auf Seite 0
	cbi		port_control,RAMPAGE
	call	CLEARSCREEN
	ldi		zl,low(2*PALETTE)
	ldi		zh,high(2*PALETTE)
	call	DRAWSPRITE



; Lschen des Bildschirms auf Seite 1
	sbi		port_control,RAMPAGE
	call	CLEARSCREEN
	ldi		zl,low(2*PALETTE)
	ldi		zh,high(2*PALETTE)
	call	DRAWSPRITE


	ldi		temp1, 8
	mov		Y_KUR_POS, temp1

	ldi		temp1, 64
	mov		Y_PIX_POS, temp1








; !!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!

; Einschalten des Interrupts 
; damit beginnt sofort (nach dem naechsten Befehl) die ISR mit der Anzeige des Bildes
	sei



; ****************************************************************************************
; **** MAIN PROGRAM LOOP
; ****************************************************************************************


MainLoop:



; Abschnitt 0 -> Kursor Blinken
; ---------------------------------------------------------
;
; Ist das Kursorblinken eingeschaltet (dann ist CUR_ON_OFF=FF)?
	ser		temp1
	lds		temp2, CUR_ON_OFF
	cp		temp1, temp2
	brne	ML_09			; wenn nicht, dann Sprung zu ML_09

; Herunterzaehlen vom Kursor-Zaehler CUR_CNT_LO
	lds		temp1, CUR_CNT_LO
	dec		temp1
	; und abspeichern
	sts		CUR_CNT_LO, temp1
	; Ist CUR_CNT_LO=0?
	brne	ML_09
	
; Herunterzaehlen vom Kursor-Zaehler CUR_CNT_HI
	lds		temp1, CUR_CNT_HI
	dec		temp1
	; und abspeichern
	sts		CUR_CNT_HI, temp1
	; Ist CUR_CNT_HI=0?
	brne	ML_09

; Je nach Kursorstatus CUR_ST verzweigen
	lds		temp1, CUR_ST
	cpi		temp1, 0
	brne	ML_01

	call	SET_CUR_ON
	call	PAGEFLIP
	call	SET_CUR_ON
	call	PAGEFLIP
	rjmp	ML_09

ML_01:

	call	SET_CUR_OFF
	call	PAGEFLIP
	call	SET_CUR_OFF
	call	PAGEFLIP

ML_09:






; Abschnitt 1 -> Laden des Grundzustandes der Register
; ---------------------------------------------------------
; temp2=0
	clr		temp2

; Laden der Adresse des RX-Puffers plus der Position des LeseZeigers (POS_RXP_RD)
	ldi		zl, low(RX_PUF)	
	ldi		zh, high(RX_PUF)

	lds		temp1, POS_RXP_RD
	add		zl, temp1
	adc		zh, temp2

; Laden der Adresse des Zeichen-Puffers
	ldi		xl, low(ZEI_PUF)
	ldi		xh, high(ZEI_PUF)

	lds		temp3, POS_ZEP


; Abschnitt 2 -> Lesen eines Zeichens aus dem RX-Puffer
; ---------------------------------------------------------
ML_20:

; Vergleich der RX-Position des LeseZeigers (POS_RXP_RD) der Hauptroutine 
; mit den SchreibZeiger (POS_RXP_WR) aus der Interrupt-Routine
	cp		POS_RXP_WR, temp1

; sind beide gleich, so ist kein weiteres Zeichen mehr vorhanden
	brne	ML_29
	rjmp	ML_50

ML_29:

; ist POS_RXP_RD ungleich POS_RXP_WR so liegt mindestens ein Datenbyte im RX-Puffer

; Lesen des Datenbyte vom RX-Puffer in par01 
	ld		par01,z+

; Inkrementieren von POS_RXP_RD
	inc		temp1

; Ist die maximale Groesse des RX-Puffers erreicht?
	cpi		temp1,Laenge_RXP

; Wenn nicht Sprung nach ML_02
	brne	ML_21

; Ist die maximale Groesse des RX-Puffers erreicht,
; dann wird der zeiger wieder auf 0 gesetzt und
	clr		temp1
; der Zeiger auf die Puffferadesse in z auf den Anfang eingestellt
	ldi		zl, low(RX_PUF)	
	ldi		zh, high(RX_PUF)


ML_21:


; Abtesten des TerminalModes
	lds		temp4, TermMode 
	cpi		temp4, Normal_TM

	breq	ML_28






; Abspeichern des temporaeren Wertes temp1 in POS_RXP_RD
	sts		POS_RXP_RD,temp1


; *** TermMode = IBM_ESC_TM *** 
	cpi		temp4, IBM_ESC_TM
	brne	ML_22

	rcall	IETM
	rjmp	MainLoop

ML_22:


; *** TermMode = IBM_CSI_TM *** 
	cpi		temp4, IBM_CSI_TM
	brne	ML_23

	rcall	ICTM
	rjmp	MainLoop


ML_23:

	; TermMode = IBM_CSI_TM_Par
	cpi		temp4, IBM_CSI_TM_Par
	brne	ML_24

	rcall	ICTMP
	rjmp	MainLoop


ML_24:

	; TermMode = IBM_CSI_TM_Par_Zeile
	cpi		temp4, IBM_CSI_TM_Par_Zeile
	brne	ML_25

	rcall	ICTMPZ
	rjmp	MainLoop


ML_25:

	; TermMode = IBM_CSI_TM_Par_Xterm
	cpi		temp4, IBM_CSI_TM_Par_Xterm
	brne	ML_25

	rcall	ICTMPX
	rjmp	MainLoop


ML_26:

	call	Meldung_TM
	rjmp	MainLoop



ML_28:



; Abschnitt 3 -> Auswerten des Zeichens
; ---------------------------------------------------------

; Vergleich des Zeichens mit dem Leerzeichen
	cpi		par01, ' '

; Ist das Zeichen kleiner als das Leerzeichen, 
; so handelt es sich um ein Steuerzeichen -
; Ansonsten wird nach ML_40 gesprungen
	brcc	ML_40



; Abschnitt 3a -> Bearbeiten des Steuerzeichen
; ............................................

; Abspeichern des temporaeren Wertes temp1 in POS_RXP_RD
	sts		POS_RXP_RD,temp1
; Abspeichern des temporaeren Wertes temp3 in POS_ZEP
	sts		POS_ZEP,temp3

; Ist im Zeichen-Puffer ein Zeichen zur Anzeige vorhanden?
	cp		temp3, temp2
; wenn nicht, dann Ueberspringen der Anzeige
	breq	ML_30

; Retten des Zeichens
	push	par01
; Anzeige des Zeichen-Puffers
	call	ANZCHR2PAGES
; Zurueckholen des Zeichens
	pop		par01

ML_30:




	rcall	AKT_CUR_OFF

; Ausschalten des Kursorblinkens
	sts		CUR_PUF_INHALT, temp2





; Auswertung des Steuerzeichens
	call	NTM_COMM

; Zurueck zum Anfang
	rjmp	MainLoop



; Abschnitt 4 -> Abspeichern des Zeichens in par01 in den Zeichen-Puffer
; ---------------------------------------------------------
;
ML_40:

	st		x+,par01
	
; Inkrementieren des POS_ZEP
	inc		temp3

; Ist die maximale Groesse des Zeichen-Puffers erreicht 
	cpi		temp3, Laenge_ZEIP
	
; wenn noch nicht, dann Ruecksprung nach ML_02
	breq	ML_41	
	rjmp	ML_20

ML_41:

; Abspeichern des temporaeren Wertes temp1 in POS_RXP_RD
	sts		POS_RXP_RD,temp1
; Abspeichern des temporaeren Wertes temp3 in POS_ZEP
	sts		POS_ZEP,temp3


	call	ANZCHR2PAGES


; Zurueck zum Anfang
	rjmp	MainLoop






; Abschnitt 5 -> RX-Puffer ist leer 
;				-> Vergleich, ob RX_STOP = 00
;				-> Test, ob Zeichenpuffer leer ist 
; ---------------------------------------------------------

ML_50:


; -------------------------------------
	; 1. Vergleich, ob RX_STOP = 00
	;     
	clr		temp4					; 1
	cp		temp4, RX_STOP			; 1
	breq	ML_52	 				; 1/2

	; Ist RX_STOP gesetzt, dann kann nun XON gesendet werden
	;
ML_51:
	lds		temp4, UCSR0A 		    ; Laden von UCSR0A in temp4
	sbrs	temp4, UDRE0			; Ueberspringen wenn UDRE0 gesetzt ist 
	brne	ML_51					; Ruecksprung

	ldi		temp4, XON	
	sts		UDR0, temp4	

	; und RX_STOP auf 00 gesetzt werden
	;
	clr		temp4					; 1
	mov		RX_STOP, temp4			; 1

ML_52:
; -------------------------------------

; Abspeichern des temporaeren Wertes temp1 in POS_RXP_RD
	sts		POS_RXP_RD,temp1
; Abspeichern des temporaeren Wertes temp3 in POS_ZEP
	sts		POS_ZEP,temp3



; Ist im Zeichen-Puffer ein Zeichen zur Anzeige vorhanden?
	cp		temp3, temp2



; wenn nicht, dann Ruecksprung nach ML_20
	brne	ML_53

	rjmp	MainLoop	;ML_20					; (Sprungdistance groesser breq)

ML_53:

	rcall	ANZCHR2PAGES

; Zurueck zum Anfang
	rjmp	MainLoop











; ****************************************************************************************
; Unterprogramm: NTM_COMM
; Funktion: Auswertung des Steuerzeichens im Terminal Modus

; Uebergaberegister: par01 = Steuerzeichen
; ****************************************************************************************
;
NTM_COMM:

	; 01H = Cursor Home
	cpi		par01, 0x01
	brne	NTM_CurRight

; Ruecksetzen der Hintergrundregister
	clr		X_PIX_POS
	clr		X_KUR_POS
	clr		Y_PIX_POS
	clr		Y_KUR_POS

; Ruecksprung
;
	ret
	;


NTM_CurRight:
	; 04 = CURSOR_RIGHT
	cpi		par01, 0x04
	brne	NTM_CurUp

NTM_CurRight_A:
	mov		temp1, X_KUR_POS
	cpi		temp1, Max_X_Zeichen-1	; Zeilenende ?

	breq	NTM_CurRight_01

	; X_PIX_POS + 6
	ldi		temp1,6
	add		X_PIX_POS,temp1

	; X_KUR_POS + 1
	inc		X_KUR_POS

NTM_CurRight_01:

; Ruecksprung
;
	ret
	;


NTM_CurUp:
	; 05 = CURSOR_UP
	cpi		par01, 0x05
	brne	NTM_Bell

NTM_CurUp_A:
	mov		temp1, Y_KUR_POS
	cpi		temp1, 0			; 1. Zeile erreicht?

	breq	NTM_CurUp_01

	; Y_PIX_POS - 8
	ldi		temp1,8
	sub		Y_PIX_POS,temp1

	; Y_KUR_POS - 1
	dec		Y_KUR_POS

NTM_CurUp_01:

; Ruecksprung
;
	ret
	;


NTM_Bell:
	; 07 = BELL
	cpi		par01, 0x07
	brne	NTM_Tab

;	rcall	RUN_BELL

; Ruecksprung
;
	ret
	;


NTM_Tab:
	; 09 = TAB
	cpi		par01, 0x09
	brne	NTM_LF

;	rcall	RUN_TAB

; Ruecksprung
;
	ret
	;


NTM_LF:
	; 0AH = Line Feed (Zeilenschaltung)
	cpi		par01, 0x0A			;Line Feed
	brne	NTM_CLS

	mov		temp1, Y_KUR_POS
	cpi		temp1, Max_Y_Zeichen-1	; Seitenende ?

	breq	NTM_LF_01


	; Y_PIX_POS + 8
	ldi		temp1,8
	add		Y_PIX_POS,temp1

	; Y_KUR_POS + 1
	inc		Y_KUR_POS

; Ruecksprung
;
	ret
	;


NTM_LF_01:
; Ansonsten ScreenRoll des Bildschirmes auf beiden Seiten
	call	SCREENROLL
	call	PAGEFLIP

	call	SCREENROLL
	call	PAGEFLIP

; Ruecksetzen der Hintergrundregister
; die X-Register werden bei CR gesetzt

	ldi		temp1, Max_Y_Zeichen-1
	mov		Y_KUR_POS, temp1

	ldi		temp1, Max_Y_Pixel-8
	mov		Y_PIX_POS, temp1


; Ruecksprung
;
	ret
	;


NTM_CLS:

	; 0CH = CLS - Clear Screen
	cpi		par01,0x0C				;Clear all
	brne	NTM_CR

NTM_CLS_01:

; Lschen des Bildschirmes auf beiden Seiten
	call	CLEARSCREEN
	call	PAGEFLIP

	call	CLEARSCREEN
	call	PAGEFLIP

; Ruecksetzen der Hintergrundregister
	clr		X_PIX_POS
	clr		X_KUR_POS
	clr		Y_PIX_POS
	clr		Y_KUR_POS

; Ruecksprung
;
	ret
	;


NTM_CR:

	; 0DH = Carridge Return (Wagenruecklauf)
	cpi		par01, 0x0D		; CR
	brne	NTM_Cur_Left 

; Ruecksetzen der Hintergrundregister
	clr		X_PIX_POS
	clr		X_KUR_POS

; Ruecksprung
;
	ret
	;


NTM_Cur_Left:

	; 13H = CURSOR_Left
	cpi		par01, 0x13
	brne	NTM_DEL_EOL

NTM_Cur_Left_A:
	mov		temp1, X_KUR_POS
	cpi		temp1, 0			; 1. Spalte erreicht?

	breq	NTM_Cur_Left_01

	; X_PIX_POS - 6
	ldi		temp1,6
	sub		X_PIX_POS,temp1

	; X_KUR_POS - 1
	dec		X_KUR_POS

NTM_Cur_Left_01:

; Ruecksprung
;
	ret
	;


NTM_DEL_EOL:

	; 16H = DELETE_EOL
	cpi		par01, 0x16
	brne	NTM_Cur_Down

	; DELETE_EOL
	call	DELETE_EOL
	call	PAGEFLIP

	call	DELETE_EOL
	call	PAGEFLIP

; Ruecksprung
;
	ret
	;



NTM_Cur_Down:

	; 18H = CURSOR_DOWN
	cpi		par01, 0x18
	brne	NTM_ESC

NTM_Cur_Down_A:
	mov		temp1, Y_KUR_POS
	cpi		temp1, Max_Y_Zeichen-1	; Seitenende ?

	breq	NTM_Cur_Down_01

	; Y_PIX_POS + 8
	ldi		temp1,8
	add		Y_PIX_POS,temp1

	; Y_KUR_POS + 1
	inc		Y_KUR_POS

NTM_Cur_Down_01:

; Ruecksprung
;
	ret
	;


NTM_ESC:

	; 1BH = Escape
	cpi		par01, 0x1B
	brne	NTM_End


	ldi		temp1, IBM_ESC_TM
	sts		TermMode,temp1

; Ruecksprung
;
	ret
	;



NTM_End:

; Ruecksprung
;
	ret
	;










; ****************************************************************************************
; Unterprogramm: IETM
; Funktion: TermMode = IBM_ESC_TM
;
; Uebergaberegister: par01 = Steuerzeichen
; ****************************************************************************************
;
IETM:

	; 1BH = Escape
	cpi		par01, '['
	brne	IETM_A


	ldi		temp1, IBM_CSI_TM
	sts		TermMode,temp1

	rjmp	IETM_X


IETM_A:

	; Meldung ESC auf den Bildschirm
	push	par01
	rcall	Meldung_ESC

	pop		par01
	rcall	PRINTCHAR

	ldi		par01, Fuellzeichen
	rcall	PRINTCHAR

; Setzen des Terminal-Mode
	ldi		temp1, Normal_TM
	sts		TermMode,temp1


IETM_X:

	; Ruecksprung
	ret
; .....






; ****************************************************************************************
; Unterprogramm: ICTM
; Funktion: TermMode = IBM_CSI_TM / Control Sequence Introducer (CSI) 
;
; Uebergaberegister: par01 = Steuerzeichen
; ****************************************************************************************
;
ICTM:

	; 30..39H -> Parameter 1
	cpi		par01, 0x30
	brlo	ICTM_A

	cpi		par01, 0x3A
	brcc	ICTM_A

	rcall	AB_to_CSI_P1

; Setzen des Terminal-Mode
	ldi		temp1, IBM_CSI_TM_Par
	sts		TermMode,temp1

	rjmp	ICTM_X

ICTM_A:

	; Lschen bis Zeilenende / ESC [ K
	cpi		par01, 'K'
	brne	ICTM_B


	; Lschen bis Zeilenende
	call	DELETE_EOL
	call	PAGEFLIP

	call	DELETE_EOL
	call	PAGEFLIP


	rjmp	ICTM_Y


ICTM_B:
 
	; Schirm lschen / ESC [ f
	cpi		par01, 'f'
	brne	ICTM_D

	rcall	NTM_CLS_01

	rjmp	ICTM_Y

ICTM_D:

	; Meldung ESC auf den Bildschirm
	push	par01
	rcall	Meldung_ESC

	ldi		par01, '['
	rcall	PRINTCHAR

	pop		par01
	rcall	PRINTCHAR

	ldi		par01, Fuellzeichen
	rcall	PRINTCHAR


ICTM_Y:

	; CSI_P1 Loeschen
	clr		temp1
	sts		CSI_P1,temp1

; Setzen des Terminal-Mode
	ldi		temp1, Normal_TM
	sts		TermMode,temp1

ICTM_X:

	; Ruecksprung
	ret
; .....












; ****************************************************************************************
; Unterprogramm: ICTMP
; Funktion: TermMode = IBM_CSI_TM_Par 
;
; Uebergaberegister: par01 = Steuerzeichen
; ****************************************************************************************
;
ICTMP:

	; 30..39H -> Parameter 1
	cpi		par01, 0x30
	brlo	ICTMP_A

	cpi		par01, 0x3A
	brcc	ICTMP_A

	rcall	AB_to_CSI_P1

; Setzen des Terminal-Mode
	ldi		temp1, IBM_CSI_TM_Par
	sts		TermMode,temp1

	rjmp	ICTMP_X


ICTMP_A:

	; ";" 
	; ESC [ Pn1 ; PN2 H oder 
	; ESC [ Pn1 ; PN2 f
	; Xterm 256 colors - ESC[48;5;BCOLOR m oder
	; Xterm 256 colors - ESC[38;5;FCOLOR m
	cpi		par01, ';'
	brne	ICTMP_B


	; TermMode:=IBM_CSI_TM_Par_Zeile; 
	ldi		temp1, IBM_CSI_TM_Par_Zeile
	sts		TermMode,temp1



	rjmp	ICTMP_X


ICTMP_B:

	; CUU -- Cursor Up = ESC [ Pn A
	cpi		par01, 'A'
	brne	ICTMP_C

	; CSI_P1 in AusgabeByte laden
	lds		temp1, CSI_P1


ICTMP_B1:
	; einmal Cursor Up
	rcall	NTM_CurUp_A

	; Zaehler dekrementieren
	dec		temp1

	; wiederholen solange wie CSI_P1
	brne	ICTMP_B1


	rjmp	ICTMP_Y


ICTMP_C:

	; CUD -- Cursor Down = ESC [ Pn B
	cpi		par01, 'B'
	brne	ICTMP_D

	; CSI_P1 in AusgabeByte laden
	lds		temp1, CSI_P1


ICTMP_C1:
	; einmal Cursor Down
	rcall	NTM_Cur_Down_A

	; Zaehler dekrementieren
	dec		temp1

	; wiederholen solange wie CSI_P1
	brne	ICTMP_C1

	rjmp	ICTMP_Y


ICTMP_D:

	; CUD -- CUF -- Cursor Forward  = ESC [ Pn C
	cpi		par01, 'C'
	brne	ICTMP_E

	; CSI_P1 in AusgabeByte laden
	lds		temp1, CSI_P1


ICTMP_D1:

	; einmal Cursor Right
	rcall	NTM_CurRight_A

	; Zaehler dekrementieren
	dec		temp1

	; wiederholen solange wie CSI_P1
	brne	ICTMP_D1

	rjmp	ICTMP_Y


ICTMP_E:

	; CUB -- Cursor Backward = ESC [ Pn D
	cpi		par01, 'D'
	brne	ICTMP_F

	; CSI_P1 in AusgabeByte laden
	lds		temp1, CSI_P1


ICTMP_E1:

	; einmal Cursor Right
	rcall	NTM_Cur_Left_A

	; Zaehler dekrementieren
	dec		temp1

	; wiederholen solange wie CSI_P1
	brne	ICTMP_E1

	rjmp	ICTMP_Y


ICTMP_F:

	; ERASE LINE = ESC [ Pn K
	cpi		par01, 'K'
	brne	ICTMP_G

	; CSI_P1 in par01 laden
	lds		temp1, CSI_P1


	; par01 = 0 -> DELETE_EOL
	; Von Cursor bis Ende
	cpi		temp1, 0
	brne	ICTMP_F1

	; DELETE_EOL
	call	DELETE_EOL
	call	PAGEFLIP

	call	DELETE_EOL
	call	PAGEFLIP

	rjmp	ICTMP_Y


ICTMP_F1:

	; par01 = 1 -> DELETE_BOL
	; Von Anfang bis Cursor
	cpi		temp1, 1
	brne	ICTMP_F2

	; DELETE_BOL
	call	DELETE_BOL
	call	PAGEFLIP

	call	DELETE_BOL
	call	PAGEFLIP

	rjmp	ICTMP_Y


ICTMP_F2:

	; par01 = 2 -> DELETE_LINE
	; Ganze Zeile loeschen
	cpi		par01, 1
	brne	ICTMP_F3

	; DELETE_LINE
	call	DELETE_LINE
	call	PAGEFLIP

	call	DELETE_LINE
	call	PAGEFLIP

	rjmp	ICTMP_Y


ICTMP_F3:

	; Meldung ESC auf den Bildschirm
	push	par01
	rcall	Meldung_ESC

	ldi		par01, '['
	rcall	PRINTCHAR

	pop		par01
	rcall	PRINTCHAR

	ldi		par01, 'K'
	rcall	PRINTCHAR

	rjmp	ICTMP_Z


ICTMP_G:

	; Bildschirm loeschen = ESC [ Pn J
	cpi		par01, 'J'
	brne	ICTMP_H

	; CSI_P1 in AusgabeByte laden
	lds		temp1, CSI_P1


	; Ausgabebyte = 2 -> Cursor Home
	cpi		temp1, 2
	brne	ICTMP_G3

; Lschen des Bildschirmes auf beiden Seiten
	call	CLEARSCREEN
	call	PAGEFLIP

	call	CLEARSCREEN
	call	PAGEFLIP

; Ruecksetzen der Hintergrundregister
	clr		X_PIX_POS
	clr		X_KUR_POS
	clr		Y_PIX_POS
	clr		Y_KUR_POS

	rjmp	ICTMP_Y


ICTMP_G3:

	; Meldung ESC auf den Bildschirm
	push	par01
	rcall	Meldung_ESC

	ldi		par01, '['
	rcall	PRINTCHAR

	pop		par01
	ldi		temp1, '0'
	add		par01, temp1
	rcall	PRINTCHAR

	ldi		par01, 'J'
	rcall	PRINTCHAR

	rjmp	ICTMP_Z


ICTMP_H:

	; Zeile einfgen = ESC [ 1 L
	cpi		par01, 'L'
	brne	ICTMP_I

	; CSI_P1 in AusgabeByte laden
	lds		par01, CSI_P1


	; Ausgabebyte = 1 -> Zeile einfgen
	cpi		par01, 1
	brne	ICTMP_H3


	; INSERT_LINE
	call	INSERT_LINE
	call	PAGEFLIP

	call	INSERT_LINE
	call	PAGEFLIP


	rjmp	ICTMP_Y


ICTMP_H3:

	; Meldung ESC auf den Bildschirm
	push	par01
	rcall	Meldung_ESC

	ldi		par01, '['
	rcall	PRINTCHAR

	pop		par01
	ldi		temp1, '0'
	add		par01, temp1
	rcall	PRINTCHAR

	ldi		par01, 'L'
	rcall	PRINTCHAR

	rjmp	ICTMP_Z


ICTMP_I:

	; Zeile loeschen = ESC [ 1 M
	cpi		par01, 'M'
	brne	ICTMP_J

	; CSI_P1 in AusgabeByte laden
	lds		par01, CSI_P1


	; Ausgabebyte = 1 -> Zeile einfgen
	cpi		par01, 1
	brne	ICTMP_I3


	; Lschen von Zeilen, Hoch-Rollen
	call	DELETE_LINE
	call	PAGEFLIP

	call	DELETE_LINE
	call	PAGEFLIP


	rjmp	ICTMP_Y


ICTMP_I3:

	; Meldung ESC auf den Bildschirm
	push	par01
	rcall	Meldung_ESC

	ldi		par01, '['
	rcall	PRINTCHAR

	pop		par01
	ldi		temp1, '0'
	add		par01, temp1
	rcall	PRINTCHAR

	ldi		par01, 'M'
	rcall	PRINTCHAR

	rjmp	ICTMP_Z


ICTMP_J:

	; Invers ein / aus - ESC [ P1 m
	cpi		par01, 'm'
	breq	ICTMP_J0

	rjmp	ICTMP_K

ICTMP_J0:

	lds		temp1, CSI_P1

	; ESC [ 0 m = Alle Attribute AUS
	cpi		temp1, 0
	brne	ICTMP_J1

	ldi		temp1, cl_Gray
	mov		FCOLOR, temp1

	ldi		temp1, cl_White
	mov		BCOLOR, temp1

	rjmp	ICTMP_Y

ICTMP_J1:

	; ESC [ 7 m = Invertierte Darstellung
	cpi		temp1, 7
	brne	ICTMP_J2

	ldi		temp1, cl_Black 
	mov		FCOLOR, temp1

	ldi		temp1, cl_White
	mov		BCOLOR, temp1

	rjmp	ICTMP_Y

; *** Vordergrundfarben ***

ICTMP_J2:

	; ESC [ 30 m = Vordergrundfarbe - 30 Schwarz
	cpi		temp1, 30
	brne	ICTMP_J3

	ldi		temp1, cl_Black 
	mov		FCOLOR, temp1

	rjmp	ICTMP_Y


ICTMP_J3:

	; ESC [ 31 m = Vordergrundfarbe - 31 Rot
	cpi		temp1, 31
	brne	ICTMP_J4

	ldi		temp1, cl_Red 
	mov		FCOLOR, temp1

	rjmp	ICTMP_Y


ICTMP_J4:

	; ESC [ 32 m = Vordergrundfarbe - 32 Grn
	cpi		temp1, 32
	brne	ICTMP_J5

	ldi		temp1, cl_Green 
	mov		FCOLOR, temp1

	rjmp	ICTMP_Y


ICTMP_J5:

	; ESC [ 33 m = Vordergrundfarbe - 33 Gelb
	cpi		temp1, 33
	brne	ICTMP_J6

	ldi		temp1, cl_Yellow 
	mov		FCOLOR, temp1

	rjmp	ICTMP_Y


ICTMP_J6:

	; ESC [ 34 m = Vordergrundfarbe - 34 Blau
	cpi		temp1, 34
	brne	ICTMP_J7

	ldi		temp1, cl_Blue 
	mov		FCOLOR, temp1

	rjmp	ICTMP_Y


ICTMP_J7:

	; ESC [ 35 m = Vordergrundfarbe - 35 Karmesinrot (Purpur, lila)
	cpi		temp1, 35
	brne	ICTMP_J8

	ldi		temp1, cl_Purple 
	mov		FCOLOR, temp1

	rjmp	ICTMP_Y


ICTMP_J8:

	; ESC [ 36 m = Vordergrundfarbe - 36 Cyanblau
	cpi		temp1, 36
	brne	ICTMP_J9

	ldi		temp1, cl_Aqua 
	mov		FCOLOR, temp1

	rjmp	ICTMP_Y


ICTMP_J9:

	; ESC [ 37 m = Vordergrundfarbe - 37 Wei
	cpi		temp1, 37
	brne	ICTMP_J10

	ldi		temp1, cl_White
	mov		FCOLOR, temp1

	rjmp	ICTMP_Y



; *** Hintergrundfarben ***

ICTMP_J10:

	; ESC [ 40 m = Hintergrundfarbe - 40 Schwarz
	cpi		temp1, 40
	brne	ICTMP_J11

	ldi		temp1, cl_Black 
	mov		BCOLOR, temp1

	rjmp	ICTMP_Y


ICTMP_J11:

	; ESC [ 41 m = Hintergrundfarbe - 41 Rot
	cpi		temp1, 41
	brne	ICTMP_J12

	ldi		temp1, cl_Red 
	mov		BCOLOR, temp1

	rjmp	ICTMP_Y


ICTMP_J12:

	; ESC [ 42 m = Hintergrundfarbe - 42 Grn
	cpi		temp1, 42
	brne	ICTMP_J14

	ldi		temp1, cl_Green 
	mov		BCOLOR, temp1

	rjmp	ICTMP_Y


ICTMP_J14:

	; ESC [ 43 m = Hintergrundfarbe - 43 Gelb
	cpi		temp1, 43
	brne	ICTMP_J15

	ldi		temp1, cl_Yellow 
	mov		BCOLOR, temp1

	rjmp	ICTMP_Y


ICTMP_J15:

	; ESC [ 44 m = Hintergrundfarbe - 44 Blau
	cpi		temp1, 44
	brne	ICTMP_J16

	ldi		temp1, cl_Blue 
	mov		BCOLOR, temp1

	rjmp	ICTMP_Y


ICTMP_J16:

	; ESC [ 45 m = Hintergrundfarbe - 45 Karmesinrot (Purpur, lila)
	cpi		temp1, 45
	brne	ICTMP_J17

	ldi		temp1, cl_Purple 
	mov		BCOLOR, temp1

	rjmp	ICTMP_Y


ICTMP_J17:

	; ESC [ 46 m = Hintergrundfarbe - 46 Cyanblau
	cpi		temp1, 46
	brne	ICTMP_J18

	ldi		temp1, cl_Aqua 
	mov		BCOLOR, temp1

	rjmp	ICTMP_Y


ICTMP_J18:

	; ESC [ 47 m = Hintergrundfarbe - 47 Wei
	cpi		temp1, 47
	brne	ICTMP_J99

	ldi		temp1, cl_White
	mov		BCOLOR, temp1

	rjmp	ICTMP_Y






ICTMP_J99:

	; Meldung ESC auf den Bildschirm
	push	par01
	rcall	Meldung_ESC

	ldi		par01, '['
	rcall	PRINTCHAR

	pop		par01
	ldi		temp1, '0'
	add		par01, temp1
	rcall	PRINTCHAR

	ldi		par01, 'm'
	rcall	PRINTCHAR

	rjmp	ICTMP_Z






ICTMP_K:

	; Meldung ESC auf den Bildschirm
	push	par01
	rcall	Meldung_ESC

	ldi		par01, '['
	rcall	PRINTCHAR

	pop		par01
	rcall	PRINTCHAR

ICTMP_Z:

	ldi		par01, Fuellzeichen
	rcall	PRINTCHAR

ICTMP_Y:

	; CSI_P1 Loeschen
	clr		temp1
	sts		CSI_P1,temp1

; Setzen des Terminal-Mode
	ldi		temp1, Normal_TM
	sts		TermMode,temp1

ICTMP_X:

	; Ruecksprung
	ret
; .....




; ****************************************************************************************
; Unterprogramm: ICTMPZ
; Funktion: TermMode = IBM_CSI_TM_Par_Zeile 
;
; Uebergaberegister: par01 = Steuerzeichen
; ****************************************************************************************
;

ICTMPZ:


	; 30..39H -> Parameter 1
	cpi		par01, 0x30
	brlo	ICTMPZ_A

	cpi		par01, 0x3A
	brcc	ICTMPZ_A

	rcall	AB_to_CSI_P2

; Setzen des Terminal-Mode
	ldi		temp1, IBM_CSI_TM_Par_Zeile
	sts		TermMode,temp1

	rjmp	ICTMPZ_X

ICTMPZ_A:

	; "H" -- MOVE CURSOR = ESC [ Pn1 ; PN2 H
	cpi		par01, 'H'
	brne	ICTMPZ_B


ICTMPZ_A2:
	; MOVE CURSOR 
	; -----------

	; Laden von CSI_P2
	lds		temp1, CSI_P2
	ldi		temp2, Max_X_Zeichen2+1
	cp		temp2, temp1
	brcs	ICTMPZ_A1

	; Setzen von X_KUR_POS
	mov		X_KUR_POS, temp1 

	; Setzen von X_PIX_POS
	ldi		temp2, 6		
	mul		temp1 ,temp2		; Ergebnis in -> r0:r1
	mov		X_PIX_POS, r0

	; Laden von CSI_P1
	lds		temp1, CSI_P1
	ldi		temp2, Max_Y_Zeichen+1
	cp		temp2, temp1
	brcs	ICTMPZ_A1

	; Setzen von Y_KUR_POS
	mov		Y_KUR_POS, temp1 

	; Setzen von Y_PIX_POS
	ldi		temp2, 8		
	mul		temp1 ,temp2		; Ergebnis in -> r0:r1
	mov		Y_PIX_POS, r0

	rjmp	ICTMPZ_Y


ICTMPZ_A1:

	; Meldung ESC auf den Bildschirm
	rcall	Meldung_ESC

	ldi		par01, '['
	rcall	PRINTCHAR

	lds		par01, CSI_P1
	ldi		temp1, '0'
	add		par01, temp1
	rcall	PRINTCHAR

	ldi		par01, ';'
	rcall	PRINTCHAR

	lds		par01, CSI_P2
	ldi		temp1, '0'
	add		par01, temp1
	rcall	PRINTCHAR

	ldi		par01, 'H'
	rcall	PRINTCHAR


	rjmp	ICTMPZ_Y



ICTMPZ_B:
	; "f" -- MOVE CURSOR = ESC [ Pn1 ; PN2 f
	cpi		par01, 'f'
	breq	ICTMPZ_A2


ICTMPZ_C:
	; Xterm 256 colors - ESC[48;5;BCOLOR m
	; Xterm 256 colors - ESC[38;5;FCOLOR m
	cpi		par01, ';'
	brne	ICTMPZ_D

	; CSI_P1 = 38 oder 48
	lds		temp1, CSI_P1
	cpi		temp1, 38

	breq	ICTMPZ_C1

	cpi		temp1, 48

	brne	ICTMPZ_D

ICTMPZ_C1:
	; CSI_P2 = 5
	lds		temp1, CSI_P2
	cpi		temp1, 5

	brne	ICTMPZ_D

	; Setzen des Terminal-Mode
	ldi		temp1, IBM_CSI_TM_Par_Xterm
	sts		TermMode,temp1

	; CSI_P2 Loeschen
	clr		temp1
	sts		CSI_P2,temp1

	rjmp	ICTMPZ_X


ICTMPZ_D:
	; Meldung ESC auf den Bildschirm
	push	par01

	rcall	Meldung_ESC

	ldi		par01, '['
	rcall	PRINTCHAR

	lds		par01, CSI_P1
	ldi		temp1, '0'
	add		par01, temp1
	rcall	PRINTCHAR

	ldi		par01, ';'
	rcall	PRINTCHAR

	lds		par01, CSI_P2
	ldi		temp1, '0'
	add		par01, temp1
	rcall	PRINTCHAR

	pop		par01
	rcall	PRINTCHAR



ICTMPZ_Y:

	; CSI_P1 Loeschen
	clr		temp1
	sts		CSI_P1,temp1

	; CSI_P2 Loeschen
	sts		CSI_P2,temp1

; Setzen des Terminal-Mode
	ldi		temp1, Normal_TM
	sts		TermMode,temp1

ICTMPZ_X:
;
; Ruecksprung
;
	ret
	;

















; ****************************************************************************************
; Unterprogramm: ICTMPX
; Funktion: TermMode = IBM_CSI_TM_Par_Xterm
;
; Uebergaberegister: par01 = Steuerzeichen
; ****************************************************************************************
;

ICTMPX:

	; 30..39H -> Parameter 1
	cpi		par01, 0x30
	brlo	ICTMPX_A

	cpi		par01, 0x3A
	brcc	ICTMPX_A

	rcall	AB_to_CSI_P2

; Setzen des Terminal-Mode
	ldi		temp1, IBM_CSI_TM_Par_Xterm
	sts		TermMode,temp1

	rjmp	ICTMPX_X

ICTMPX_A:

	; Xterm 256 colors - ESC[48;5;BCOLOR m oder
	; Xterm 256 colors - ESC[38;5;FCOLOR m
	cpi		par01, 'm'
	brne	ICTMPX_D


	; CSI_P1 = 38 oder 48
	lds		temp1, CSI_P1
	cpi		temp1, 38

	breq	ICTMPX_B38

	cpi		temp1, 48

	breq	ICTMPX_B48

	rjmp	ICTMPX_D

; *** Vordergrundfarbe ***
ICTMPX_B38:
	; CSI_P2 
	lds		temp1, CSI_P2
	mov		FCOLOR, temp1

	rjmp	ICTMPX_Y


; *** Hintergrundfarbe ***
ICTMPX_B48:

	; CSI_P2 
	lds		temp1, CSI_P2
	mov		BCOLOR, temp1

	rjmp	ICTMPX_Y



ICTMPX_D:
	; Meldung ESC auf den Bildschirm
	push	par01

	rcall	Meldung_ESC

	ldi		par01, '['
	rcall	PRINTCHAR

	lds		par01, CSI_P1		; !!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
	ldi		temp1, '0'
	add		par01, temp1
	rcall	PRINTCHAR

	ldi		par01, ';'
	rcall	PRINTCHAR

	ldi		par01, '5'
	rcall	PRINTCHAR

	ldi		par01, ';'
	rcall	PRINTCHAR

	lds		par01, CSI_P2		; !!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
	ldi		temp1, '0'
	add		par01, temp1
	rcall	PRINTCHAR

	pop		par01
	rcall	PRINTCHAR


ICTMPX_Y:

	; CSI_P1 Loeschen
	clr		temp1
	sts		CSI_P1,temp1

	; CSI_P2 Loeschen
	sts		CSI_P2,temp1

; Setzen des Terminal-Mode
	ldi		temp1, Normal_TM
	sts		TermMode,temp1

ICTMPX_X:
;
; Ruecksprung
;
	ret
	;




; Meldung ESC auf den Bildschirm
Meldung_ESC:

	ldi		temp1, 'E'
	mov		par01, temp1
	rcall	PRINTCHAR

	ldi		temp1, 'S'
	mov		par01, temp1
	rcall	PRINTCHAR

	ldi		temp1, 'C'
	mov		par01, temp1
	rcall	PRINTCHAR

	; Ruecksprung
	ret
; .....





Meldung_TM:

	; Meldung [TM] auf den Bildschirm
	push	par01
	rcall	Meldung_ESC

	ldi		temp1, '['
	mov		par01, temp1
	rcall	PRINTCHAR

	ldi		temp1, 'T'
	mov		par01, temp1
	rcall	PRINTCHAR

	ldi		temp1, 'M'
	mov		par01, temp1
	rcall	PRINTCHAR

	pop		par01
	rcall	PRINTCHAR

	ldi		temp1, ']'
	mov		par01, temp1
	rcall	PRINTCHAR

	ldi		temp1, Fuellzeichen
	mov		par01, temp1
	rcall	PRINTCHAR


; Setzen des Terminal-Mode
	ldi		temp1, Normal_TM
	sts		TermMode,temp1


; Ruecksprung
;
	ret
	;



; AusgabeByte als Einer in CSI_P1 abspeichern 
; CSI_P1 = (CSI_P1*10) + AusgabeByte
AB_to_CSI_P1:

	; Umwandeln Zeichen in Byte
	subi	par01, 0x30

	; Laden des CSI_P1 in temp2
	lds		temp2, CSI_P1 

	; ist CSI_P1 = 0 ? 
	cpi		temp2, 0		

	breq	AB_10_to_CSI_P1_B	; wenn ja,  dann nur AusgabeByte abspeichern 


	; ansonsten CSI_P1*10 berechnen
	clr		temp1
	ldi		temp3, 10

; Schleife fuer temp2 * 10 -> temp1
AB_10_to_CSI_P1_A:

	; temp1 + 10
	add		temp1, temp3
		
	; Dekrementieren des Multiplikators
	dec		temp2

	; Wiederholen bis temp2=0
	brne	AB_10_to_CSI_P1_A

	; nun wird zum Zehnerwert noch der Einerwert addiert
	add		par01, temp1 

AB_10_to_CSI_P1_B:

	; Abspeichern des CSI_P1
	sts		CSI_P1, par01

	; Ruecksprung
	ret
; .....




; AusgabeByte als Einer in CSI_P2 abspeichern 
; CSI_P2 = (CSI_P2*10) + AusgabeByte
AB_to_CSI_P2:

	; Umwandeln Zeichen in Byte
	subi	par01, 0x30

	; Laden des CSI_P2 in temp2
	lds		temp2, CSI_P2 

	; ist CSI_P2 = 0 ? 
	cpi		temp2, 0		

	breq	AB_10_to_CSI_P2_B	; wenn ja,  dann nur AusgabeByte abspeichern 


	; ansonsten CSI_P2*10 berechnen
	clr		temp1
	ldi		temp3, 10

; Schleife fuer temp2 * 10 -> temp1
AB_10_to_CSI_P2_A:

	; temp1 + 10
	add		temp1, temp3
		
	; Dekrementieren des Multiplikators
	dec		temp2

	; Wiederholen bis temp2=0
	brne	AB_10_to_CSI_P2_A

	; nun wird zum Zehnerwert noch der Einerwert addiert
	add		par01, temp1 

AB_10_to_CSI_P2_B:

	; Abspeichern des CSI_P1
	sts		CSI_P2, par01

	; Ruecksprung
	ret
; .....




; ****************************************************************************************
; Unterprogramm: DELETE_EOL
; Funktion: Lschen bis Zeilenende
; Mit 'Clear to end of line' kann die aktuelle Zeile ab der Cursorposition gelscht werden.
; ****************************************************************************************
;
DELETE_EOL:


; Laden der Anfangswerte
	mov		YPOS, Y_PIX_POS 
	mov		XPOS, X_PIX_POS


; Laden der Endpunkte
	ldi		temp1,Max_X_Pixel	; XEND
	inc		temp1
	ldi		temp2,8				; YEND
	add		temp2, Y_PIX_POS  


; WAIT FOR MEMREADY
DEL_EOL_01:
	sbrc	memready,0
	rjmp	DEL_EOL_01


; SEND SCREEN DATA
	out		port_dat,BCOLOR
	out		port_adr_hig,YPOS

	cbi		port_control,WE	

	out		port_adr_low,XPOS
	inc		XPOS

	sbi		port_control,WE


; COMPLETE LOOP
	cp		XPOS,temp1	; XEND erreicht?
	brne	DEL_EOL_01

	mov		XPOS, X_PIX_POS

	inc		YPOS
	cp		YPOS,temp2	; YEND erreicht?
	brne	DEL_EOL_01


; Ruecksprung
;
	ret
	;




; ****************************************************************************************
; Unterprogramm: DELETE_LINE
; Funktion: Loeschen der Zeile auf Kursorposition
; ****************************************************************************************
;
DELETE_LINE:


; *** 1. Setzen der Y-Grundregister

; Setzen der Y-Quell und Y-Ziel-Register
	mov		YPOS, Y_PIX_POS			; YPOS -> Ziel
	ldi		temp3, 8				; YPOS -> Quelle
	add		temp3, Y_PIX_POS


	cpi		temp3, Max_Y_Pixel
	breq	DELETE_LINE_05


; Vergleichsregister fuer den Wert 0
	clr		temp1



; # Ein kompletter Durchlauf der X-Schleife
DELETE_LINE_01:


; lfd. XPOS auf 0 setzen
	clr		XPOS				
; Einteilung der Zeile in 64 Durchlaeufe
	ldi		temp2, 64



; WAIT FOR MEMREADY
DELETE_LINE_02:
	sbrc	memready,0
	rjmp	DELETE_LINE_02


; die X-Schleife
DELETE_LINE_03:


; Daten-Port auf Eingang programmieren
	clr		temp4
	out		ddr_dat,temp4

; Setzen der Y-Speicheradresse
	out		port_adr_hig,temp3		; YPOS -> Quelle


; Setzen der X-Speicheradresse
	out		port_adr_low,XPOS

; SRAM OUTPUT ENABLE
	cbi		port_control,OE

	nop
	nop

; READ DATA BYTE
	in		par01,pin_dat

; SRAM OUTPUT DISABLE
	sbi		port_control,OE


; Daten-Port auf Ausgang programmieren
	ser		temp4
	out		ddr_dat,temp4

; YPOS -> Ziel 
	out		port_adr_hig,YPOS		; YPOS -> Ziel

; Ausgabe des Farbbytes fuer den Pixel an den Datenport des SRAM
	out		port_dat,par01

	nop

; und Uebernahme des Wertes in den RAM durch aktivieren von WE
	cbi		port_control,WE	
	nop
	sbi		port_control,WE

	inc		XPOS

; XEND erreicht?
	cp		XPOS, temp2
	
; Wenn nicht, X-Schleife-Schreiben nochmal durchlaufen		
	brne	DELETE_LINE_03



; der naechste 64-Schleifen Abschnitt der X-Schleife
	ldi		temp4, 64
	add		temp2, temp4

	cp		XPOS, temp1
	brne	DELETE_LINE_02



; erhoehen auf die naechste Pixel-Zeile
	inc		YPOS			; YPOS -> Ziel
	inc		temp3			; YPOS -> Quelle

	cpi		temp3, Max_Y_Pixel

; Wenn nicht, X-Schleife auf naechster Y-Zeile (8er Raster)
	brne	DELETE_LINE_01



DELETE_LINE_05:



; *** Loeschen der letzten Zeile



; Setzen der X- und Y-Register
	ldi		temp4,Max_Y_Pixel-8
	mov		YPOS, temp4
	clr		XPOS

; Laden der Endpunkte
	ldi		temp1,Max_X_Pixel	; XEND
	inc		temp1
	ldi		temp2,Max_Y_Pixel	; YEND




; WAIT FOR MEMREADY
DELETE_LINE_06:
	sbrc	memready,0
	rjmp	DELETE_LINE_06

DELETE_LINE_07:

; SEND SCREEN DATA
	out		port_dat,BCOLOR
	out		port_adr_hig,YPOS
	cbi		port_control,WE	


	out		port_adr_low,XPOS
	inc		XPOS
	out		port_adr_low,XPOS
	inc		XPOS
	out		port_adr_low,XPOS
	inc		XPOS
	out		port_adr_low,XPOS
	inc		XPOS

	out		port_adr_low,XPOS
	inc		XPOS
	out		port_adr_low,XPOS
	inc		XPOS
	out		port_adr_low,XPOS
	inc		XPOS
	out		port_adr_low,XPOS
	inc		XPOS


	out		port_adr_low,XPOS
	inc		XPOS
	out		port_adr_low,XPOS
	inc		XPOS
	out		port_adr_low,XPOS
	inc		XPOS
	out		port_adr_low,XPOS
	inc		XPOS

	out		port_adr_low,XPOS
	inc		XPOS
	out		port_adr_low,XPOS
	inc		XPOS
	out		port_adr_low,XPOS
	inc		XPOS
	out		port_adr_low,XPOS
	inc		XPOS


	sbi		port_control,WE


; COMPLETE LOOP
	cp		XPOS,temp1	; XEND erreicht?
	brne	DELETE_LINE_07
	inc		YPOS
	cp		YPOS,temp2	; YEND erreicht?
	brne	DELETE_LINE_06



; Ruecksprung
;
	ret
	;




; ****************************************************************************************
; Unterprogramm: DELETE_BOL
; Funktion: Lschen von Zeilenanfang bis (eischliesslich) Kursorposition
; ****************************************************************************************
;
DELETE_BOL:

	clr		temp3

; Laden der Anfangswerte
	mov		YPOS, Y_PIX_POS 
	mov		XPOS, temp3


; Laden der Endpunkte
	ldi		temp1, 6			; XEND
	add		temp1, X_PIX_POS 

	ldi		temp2, 8			; YEND
	add		temp2, Y_PIX_POS  


; WAIT FOR MEMREADY
DEL_BOL_01:
	sbrc	memready,0
	rjmp	DEL_BOL_01


; SEND SCREEN DATA
	out		port_dat,BCOLOR
	out		port_adr_hig,YPOS

	cbi		port_control,WE	

	out		port_adr_low,XPOS
	inc		XPOS

	sbi		port_control,WE


; COMPLETE LOOP
	cp		XPOS, temp1	; XEND erreicht?
	brne	DEL_BOL_01

	mov		XPOS, temp3

	inc		YPOS
	cp		YPOS, temp2	; YEND erreicht?
	brne	DEL_BOL_01


; Ruecksprung
;
	ret
	;



; ****************************************************************************************
; Unterprogramm: INSERT_LINE
; Funktion: Einfgen von Zeilen, Runter-Rollen
; ****************************************************************************************
;
INSERT_LINE:

; *** 1. Setzen der Y-Grundregister

; Setzen der Y-Quell und Y-Ziel-Register
	ldi		temp3, Max_Y_Pixel
	mov		YPOS, temp3				; YPOS -> Ziel
	ldi		temp3, Max_Y_Pixel-8	; YPOS -> Quelle

	cp		temp3, Y_PIX_POS 	 		
	breq	INSERT_LINE_05


; Vergleichsregister fuer den Wert 0
	clr		temp1


; # Ein kompletter Durchlauf der X-Schleife
INSERT_LINE_01:

; erhoehen auf die naechste Pixel-Zeile
	dec		YPOS			; YPOS -> Ziel
	dec		temp3			; YPOS -> Quelle

; lfd. XPOS auf 0 setzen
	clr		XPOS				
; Einteilung der Zeile in 64 Durchlaeufe
	ldi		temp2, 64



; WAIT FOR MEMREADY
INSERT_LINE_02:
	sbrc	memready,0
	rjmp	INSERT_LINE_02


; die X-Schleife
INSERT_LINE_03:


; Daten-Port auf Eingang programmieren
	clr		temp4
	out		ddr_dat,temp4

; Setzen der Y-Speicheradresse
	out		port_adr_hig,temp3		; YPOS -> Quelle


; Setzen der X-Speicheradresse
	out		port_adr_low,XPOS

; SRAM OUTPUT ENABLE
	cbi		port_control,OE

	nop
	nop

; READ DATA BYTE
	in		par01,pin_dat

; SRAM OUTPUT DISABLE
	sbi		port_control,OE


; Daten-Port auf Ausgang programmieren
	ser		temp4
	out		ddr_dat,temp4

; YPOS -> Ziel 
	out		port_adr_hig,YPOS		; YPOS -> Ziel

; Ausgabe des Farbbytes fuer den Pixel an den Datenport des SRAM
	out		port_dat,par01

; und Uebernahme des Wertes in den RAM durch aktivieren von WE
	cbi		port_control,WE	
	nop
	sbi		port_control,WE

	inc		XPOS

; XEND erreicht?
	cp		XPOS, temp2
	
; Wenn nicht, X-Schleife-Schreiben nochmal durchlaufen		
	brne	INSERT_LINE_03



; der naechste 64-Schleifen Abschnitt der X-Schleife
	ldi		temp4, 64
	add		temp2, temp4

		
	cp		XPOS, temp1
	brne	INSERT_LINE_02


	cp		temp3, Y_PIX_POS

; Wenn nicht, X-Schleife auf naechster Y-Zeile
	brne	INSERT_LINE_01



INSERT_LINE_05:


; *** Loeschen der aktuellen Zeile



; Setzen der X- und Y-Register
	mov		YPOS, Y_PIX_POS
	clr		XPOS

; Laden der Endpunkte
	ldi		temp1,Max_X_Pixel	; XEND
	inc		temp1

	ldi		temp2, 8			; YEND
	add		temp2, Y_PIX_POS



; WAIT FOR MEMREADY
INSERT_LINE_06:
	sbrc	memready,0
	rjmp	INSERT_LINE_06

INSERT_LINE_07:

; SEND SCREEN DATA
	out		port_dat,BCOLOR
	out		port_adr_hig,YPOS
	cbi		port_control,WE	


	out		port_adr_low,XPOS
	inc		XPOS
	out		port_adr_low,XPOS
	inc		XPOS
	out		port_adr_low,XPOS
	inc		XPOS
	out		port_adr_low,XPOS
	inc		XPOS

	out		port_adr_low,XPOS
	inc		XPOS
	out		port_adr_low,XPOS
	inc		XPOS
	out		port_adr_low,XPOS
	inc		XPOS
	out		port_adr_low,XPOS
	inc		XPOS


	out		port_adr_low,XPOS
	inc		XPOS
	out		port_adr_low,XPOS
	inc		XPOS
	out		port_adr_low,XPOS
	inc		XPOS
	out		port_adr_low,XPOS
	inc		XPOS

	out		port_adr_low,XPOS
	inc		XPOS
	out		port_adr_low,XPOS
	inc		XPOS
	out		port_adr_low,XPOS
	inc		XPOS
	out		port_adr_low,XPOS
	inc		XPOS


	sbi		port_control,WE


; COMPLETE LOOP
	cp		XPOS,temp1	; XEND erreicht?
	brne	INSERT_LINE_07
	inc		YPOS
	cp		YPOS,temp2	; YEND erreicht?
	brne	INSERT_LINE_06



; Ruecksprung
;
	ret
	;



; ****************************************************************************************
; Unterprogramm: AKT_CUR_OFF
; Funktion: Abschalten des Kursors, sofern er aktiv ist
;
; Uebergaberegister: keine
; ****************************************************************************************
;
AKT_CUR_OFF:

	clr		temp2

; Ist das Kursorblinken ausgeschaltet (dann ist CUR_ON_OFF=00)?
	lds		temp4, CUR_ON_OFF
	cp		temp4, temp2
	breq	AKT_CUR_OFF_02		; wenn ja, dann Sprung zu ML_28

; Je nach Kursorstatus CUR_ST verzweigen
	lds		temp4, CUR_ST
	cp		temp4, temp2

	brne	AKT_CUR_OFF_01

; Zuruecksetzen des LO-Cursor-Counters	
	sts		CUR_CNT_LO, temp2

; Zuruecksetzen des HI-Cursor-Counters	
	ldi		temp4, CUR_CNT
	sts		CUR_CNT_HI, temp4

	rjmp	AKT_CUR_OFF_02

AKT_CUR_OFF_01:

; nur abarbeiten, wenn ein Zeichen im Puffer ist
	lds		temp1, CUR_PUF_INHALT
	cpi		temp1, 0

	breq	AKT_CUR_OFF_02

	push	temp1
	push	temp2
	push	temp3
	push	par01

	call	SET_CUR_OFF
	call	PAGEFLIP
	call	SET_CUR_OFF
	call	PAGEFLIP

	pop		par01
	pop		temp3
	pop		temp2
	pop		temp1

AKT_CUR_OFF_02:


	; Ruecksprung
	ret
; .....





; ****************************************************************************************
; Unterprogramm: SET_CUR_ON
; Funktion: Anzeige von '_' an der Kursorposition
; ****************************************************************************************
;
SET_CUR_ON:

; WAIT FOR MEMREADY
SC_ON_MR:
	sbrc	memready,0
	rjmp	SC_ON_MR


	; 1. Retten des Zeichens
; Laden der Adresse des CUR_PUF
	ldi		zl, low(CUR_PUF)	
	ldi		zh, high(CUR_PUF)

; Zwischenspeicher fuer die Y-Position jeder Zeile
	mov		temp2, Y_PIX_POS

; 8 Zeilen in Y-Richtung
	ldi		temp4, 8

; SET DATA PORT INPUT
	clr		temp1
	out		ddr_dat,temp1


SC_ON_02:


	mov		YPOS, temp2
	mov		XPOS, X_PIX_POS

; SET SCREEN ADDRESS
	out		port_adr_hig,YPOS

; 6 Spalten in X-Richtung
	ldi		temp3, 6


SC_ON_01:
; 
	out		port_adr_low, XPOS


; SRAM OUTPUT ENABLE
	cbi		port_control, OE


; TURNAROUND DELAY
	nop
	nop


; Lesen des Daten-Bytes vom SRAM
	in		par01,pin_dat
; Schreiben des Daten-Bytes in den Puffer
	st		z+,par01


; SRAM OUTPUT DISABLE
	sbi		port_control,OE

	inc		XPOS

	dec		temp3

	brne	SC_ON_01


	inc		temp2

	dec		temp4

	brne	SC_ON_02


; Daten-Port auf Ausgang programmieren
	ser		temp1
	out		ddr_dat,temp1


; ---------------------------------


; SET POINTER TO CHARACTER DATA
	ldi		zl,low(2*CHAR_)		; Z mit Tabellenanfang CHAR_ laden
	ldi		zh,high(2*CHAR_)



; LOOP
	mov		YPOS,Y_PIX_POS		; YPOS <- Y_PIX_POS (Y-Position in port_adr_hig)
	ldi		temp4,8				; temp4 mit Anzahl der Bytes in Y-Richtung des Zeichens laden


; WAIT FOR MEMREADY
SC_ON_MR2:
	sbrc	memready,0
	rjmp	SC_ON_MR2



; Schreiben der Y-Position
SC_ON_20:

; READ CHARACTER BYTE
	lpm		temp2,z+			; Laden des Zeichenbytes in temp2

	lsr		temp2
	lsr		temp2

; SET SCREEN ADDRESS
	mov		XPOS, X_PIX_POS		; XPOS <- X_PIX_POS (X-Position in port_adr_low)
	out		port_adr_hig, YPOS	; Ausgabe yh
	ldi		temp1,6				; temp1 mit Anzahl der Bytes in X-Richtung des Zeichens laden

; Schreiben der X-Position
SC_ON_23:
	out		port_adr_low, XPOS	; Ausgabe XPOS


; Ausschieben des Zeichenbits
	out		port_dat, FCOLOR	; Ausgabe Zeichenfarbe an den Datenport
	sbrc	temp2, 0			; ist Zeichenbyte an der Stelle 0 = 1 (Zeichenfarbe),
	rjmp	SC_ON_21			; dann Sprung zu DOSHIFT


; Ueberspringen, wenn Zeichenfarbe = Hintergrundfarbe (dann bleibt der bereits ausgegebene Wert)
	cp		BCOLOR, FCOLOR		; Zeichenfarbe = Hintergrundfarbe ?
	breq	SC_ON_22			; dann keine Aenderung
	sbrs	temp2, 0			; ist temp2 an der Stelle 0 gesetzt,
								; dann wird das Pixelbyte mit der Hintergrundfarbe gefuellt
	out		port_dat,BCOLOR		; Ausgabe der Hintergrundfarbe an den Datenport 

SC_ON_21:
	cbi		port_control,WE		; Einschreiben des neuen Pixelwertes in den SRAM
	nop
	nop
	sbi		port_control,WE

SC_ON_22:


; COMPLETE SHIFT LOOP
	lsr		temp2				; Logical Shift Right --> 0 --> |7|6|5|4|3|2|1|0|
								; um das nchste Bit des Pixelbytes auszuwerten 
	inc		XPOS				; X-Position um eine Stelle nach rechts
	dec		temp1				; Dekrement X-Positionszaehler
	brne	SC_ON_23			; wenn nicht 0, dann noch eine X-Pos schreiben
	inc		YPOS				; Y-Position um eine Stelle nach unten
	dec		temp4				; Dekrement Y-Positionszaehler
	brne	SC_ON_20			; wenn nicht 0, dann noch eine Y-Pos schreiben


; ---------------------------------


	ser		temp1
	sts		CUR_ST, temp1
; Einschalten CUR_PUF_INHALT
	sts		CUR_PUF_INHALT, temp1


	clr		temp1
	sts		CUR_CNT_LO, temp1

; Setzen Cursor-Zeit	
	ldi		temp1, CUR_CNT
	sts		CUR_CNT_HI, temp1

; Ruecksprung
;
	ret
	;




; ****************************************************************************************
; Unterprogramm: SET_CUR_OFF
; Funktion: Anzeige des Zeichens an der Kursorposition
; ****************************************************************************************
;
SET_CUR_OFF:

	; nur abarbeiten, wenn ein Zeichen im Puffer ist
	lds		temp1, CUR_PUF_INHALT
	cpi		temp1, 0

	breq	SC_OFF_END




; WAIT FOR MEMREADY
SC_OFF_MR:
	sbrc	memready,0
	rjmp	SC_OFF_MR


; Zurueckschreiben des Zeichens

	; Laden der Adresse des CUR_PUF
	ldi		zl, low(CUR_PUF)	
	ldi		zh, high(CUR_PUF)

	; Zwischenspeicher fuer die Y-Position jeder Zeile
	mov		temp2, Y_PIX_POS


	; 8 Zeilen in Y-Richtung
	ldi		temp4, 8


SC_OFF_02:

	mov		YPOS, temp2
	mov		XPOS, X_PIX_POS

; SET SCREEN ADDRESS
	out		port_adr_hig,YPOS

; 6 Spalten in X-Richtung
	ldi		temp3, 6


SC_OFF_01:
; 
	out		port_adr_low, XPOS




; Lesen des Datenbyte vom RX-Puffer in par01 
	ld		par01,z+


	out		port_dat,par01

	nop

; und Uebernahme des Wertes in den RAM durch aktivieren von WE
	cbi		port_control,WE	
	nop
	nop
	sbi		port_control,WE



	inc		XPOS

	dec		temp3

	brne	SC_OFF_01


	inc		temp2

	dec		temp4

	brne	SC_OFF_02


; ---------------------------------

SC_OFF_END:


	clr		temp1
	sts		CUR_ST, temp1
	sts		CUR_CNT_LO, temp1

; Setzen Cursor-Zeit	
	ldi		temp1, CUR_CNT
	sts		CUR_CNT_HI, temp1

; Ruecksprung
;
	ret
	;






; ****************************************************************************************
; Unterprogramm: SCREENROLL
; Funktion: Anzeige des Zeichen-Puffers in beiden PAGES

; HintergrundDatenRegister:		X_PIX_POS = X-Position des Zeichens
;								X_KUR_POS = X-Position des Kursors
;								X_PIX_POS = Y-Position des Zeichens
;								Y_KUR_POS = Y-Position des Kursors
; ****************************************************************************************
;
SCREENROLL:


; *** 1. Setzen der Y-Grundregister

; Setzen der Y-Quell und Y-Ziel-Register
	ldi		temp3, 8				; YPOS -> Quelle
	clr		YPOS					; YPOS -> Ziel


; Vergleichsregister fuer den Wert 0
	clr		temp1



; # Ein kompletter Durchlauf der X-Schleife
SR_03:


; lfd. XPOS auf 0 setzen
	clr		XPOS				
; Einteilung der Zeile in 4 x 64 Durchlaeufe
	ldi		temp2, 32



SR_05:


; WAIT FOR MEMREADY
SR_MR:
	sbrc	memready,0
	rjmp	SR_MR


; die X-Schleife
SR_04:


; Daten-Port auf Eingang programmieren
	clr		temp4
	out		ddr_dat,temp4

; Setzen der Y-Speicheradresse
	out		port_adr_hig,temp3		; YPOS -> Quelle


; Setzen der X-Speicheradresse
	out		port_adr_low,XPOS

; SRAM OUTPUT ENABLE
	cbi		port_control,OE

	nop
	nop

; READ DATA BYTE
	in		par01,pin_dat

; SRAM OUTPUT DISABLE
	sbi		port_control,OE


; Daten-Port auf Ausgang programmieren
	ser		temp4
	out		ddr_dat,temp4

; YPOS -> Ziel 
	out		port_adr_hig,YPOS		; YPOS -> Ziel

; Ausgabe des Farbbytes fuer den Pixel an den Datenport des SRAM
	out		port_dat,par01

	nop

; und Uebernahme des Wertes in den RAM durch aktivieren von WE
	cbi		port_control,WE	
	nop
	sbi		port_control,WE

	inc		XPOS

; XEND erreicht?
	cp		XPOS, temp2
	
; Wenn nicht, X-Schleife-Schreiben nochmal durchlaufen		
	brne	SR_04



; der naechste 64-Schleifen Abschnitt der X-Schleife
	ldi		temp4, 32
	add		temp2, temp4

	cp		XPOS, temp1
	brne	SR_05



; erhoehen auf die naechste Pixel-Zeile
	inc		YPOS			; YPOS -> Ziel
	inc		temp3			; YPOS -> Quelle

	cpi		temp3, Max_Y_Pixel

; Wenn nicht, X-Schleife auf naechster Y-Zeile (8er Raster)
	brne	SR_03




; *** 7. Loeschen der letzten Zeile



; Setzen der X- und Y-Register
	ldi		temp4,Max_Y_Pixel-8
	mov		YPOS, temp4
	clr		XPOS

; Laden der Endpunkte
	ldi		temp1,Max_X_Pixel	; XEND
	inc		temp1
	ldi		temp2,Max_Y_Pixel	; YEND


SR_10:

; WAIT FOR MEMREADY
SR_12:
	sbrc	memready,0
	rjmp	SR_12

SR_14:

; SEND SCREEN DATA
	out		port_dat,BCOLOR
	out		port_adr_hig,YPOS
	cbi		port_control,WE	


	out		port_adr_low,XPOS
	inc		XPOS
	out		port_adr_low,XPOS
	inc		XPOS
	out		port_adr_low,XPOS
	inc		XPOS
	out		port_adr_low,XPOS
	inc		XPOS

	out		port_adr_low,XPOS
	inc		XPOS
	out		port_adr_low,XPOS
	inc		XPOS
	out		port_adr_low,XPOS
	inc		XPOS
	out		port_adr_low,XPOS
	inc		XPOS


	out		port_adr_low,XPOS
	inc		XPOS
	out		port_adr_low,XPOS
	inc		XPOS
	out		port_adr_low,XPOS
	inc		XPOS
	out		port_adr_low,XPOS
	inc		XPOS

	out		port_adr_low,XPOS
	inc		XPOS
	out		port_adr_low,XPOS
	inc		XPOS
	out		port_adr_low,XPOS
	inc		XPOS
	out		port_adr_low,XPOS
	inc		XPOS


	sbi		port_control,WE


; COMPLETE LOOP
	cp		XPOS,temp1	; XEND erreicht?
	brne	SR_14
	inc		YPOS
	cp		YPOS,temp2	; YEND erreicht?
	brne	SR_10

	
; Ruecksprung
;
	ret





; ****************************************************************************************
; Unterprogramm: ANZCHR2PAGES
; Funktion: Anzeige des Zeichen-Puffers in beiden PAGES
; HintergrundDatenRegister:		X_PIX_POS = X-Position des Zeichens
;								X_KUR_POS = X-Position des Kursors
;								X_PIX_POS = Y-Position des Zeichens
;								Y_KUR_POS = Y-Position des Kursors
; ****************************************************************************************
;
ANZCHR2PAGES:


	rcall	AKT_CUR_OFF

; Einschalten CUR_PUF_INHALT
	ser		temp1
	sts		CUR_PUF_INHALT, temp1


; da die Zeichen auf beiden Pages gezeichnet werden muessen,
; werden die Hintergrundregister fuer das Zeichnen der zweiten Page
; gesichert
	push	X_PIX_POS
	push	X_KUR_POS

; Anzeige aller Zeichen im Zeichen-Puffer auf der ersten PAGE
	call	PRINTCHAR6x8z
; Umschalten der PAGE
	call	PAGEFLIP

; Ruecksicherung der gespeicherten Register

	pop		X_KUR_POS
	pop		X_PIX_POS

; Anzeige aller Zeichen im Zeichen-Puffer auf der zweiten PAGE
	call	PRINTCHAR6x8z
; Umschalten der PAGE
	call	PAGEFLIP

; Ruecksetzen des Puffer-Zeigers
	clr		temp3
	sts		POS_ZEP,temp3	

; Ruecksprung
;
	ret








; ****************************************************************************************
; Unterprogramm: PAGEFLIP
; Funktion: Umschalten der VIDEO PAGE
;
; ****************************************************************************************
;
PAGEFLIP:

; WAIT FOR VSYNC
PFVS:
	sbic	port_control,VER
	rjmp	PFVS

; SWAP BUFFERS
	in		temp1,port_control
	sbrc	temp1,RAMPAGE
	cbi		port_control,RAMPAGE
	sbrs	temp1,RAMPAGE
	sbi		port_control,RAMPAGE

; Ruecksprung
;
	ret


 

; ****************************************************************************************
; Unterprogramm: READPIXEL
; Funktion: Lesen eines Pixels aus dem BWS

; HintergrundDatenRegister:		X_PIX_POS = X-Position des Zeichens
;								Y_PIX_POS = Y-Position des Zeichens
; RueckgabeRegister:			par01=Farbe
; ****************************************************************************************
;
READPIXEL:


; WAIT FOR MEMREADY
RPMR:
	sbrc	memready,0
	rjmp	RPMR


; SET SCREEN ADDRESS
	out		port_adr_low,X_PIX_POS
	out		port_adr_hig,Y_PIX_POS


; SET DATA PORT INPUT
	clr		temp1
	out		ddr_dat,temp1


; SRAM OUTPUT ENABLE
	cbi		port_control,OE


; TURNAROUND DELAY
	nop
	nop


; READ DATA BYTE
	in		par01,pin_dat


; SRAM OUTPUT DISABLE
	sbi		port_control,OE


; SET DATA PORT OUTPUT
	ser		temp1
	out		ddr_dat,temp1


; Ruecksprung
;
	ret



 

; ****************************************************************************************
; Unterprogramm: DRAWSPRITE
; Funktion: kopiert ein Sprite oder ein Bild auf den Bildschirm 
; Hinweis: Pixelfarbe 00=Transparent
; Uebergaberegister: 	Z= Adresse des Bildes o. Sprites	
; HintergrundDatenRegister:		X_PIX_POS=X-Position des Zeichens
;								Y_PIX_POS=X-Position des Zeichens
; ****************************************************************************************
DRAWSPRITE:

; READ SPRITE SIZE DATA
	lpm		temp3,z+	; Anzahl der X-Pixel -> temp3
	lpm		temp2,z+	; Anzahl der Y-Pixel -> temp2


; SET SCREEN LOCATION
	add		temp3,X_PIX_POS		; XEND + X_PIX_POS = letzte X-Pixelposition
	add		temp2,Y_PIX_POS		; YEND + Y_PIX_POS = letzte Y-Pixelposition

	mov		YPOS,Y_PIX_POS


; COUNT Y BYTES
DSYL:
	mov		XPOS,X_PIX_POS


; WAIT FOR MEMREADY
DSMR:
	sbrc	memready,0
	rjmp	DSMR


; COUNT X BYTES
DSXL:
	lpm		temp1,z+
	cpi		temp1,0		; 00=Transparent!!!


; SKIP BLANK PIXELS
	breq	DSBL


; SEND SPRITE DATA
	out		port_dat,temp1
	out		port_adr_low,XPOS
	out		port_adr_hig,YPOS
	cbi		port_control,WE
	sbi		port_control,WE


; COMPLETE LOOP
DSBL:
	inc		XPOS
	cp		XPOS,temp3		; XEND
	brne	DSXL
	inc		YPOS
	cp		YPOS,temp2		; YEND
	brne	DSYL


; Ruecksprung
;
	ret



 

; ****************************************************************************************
; Unterprogramm: CLEARSCREEN
; Funktion: Loeschen einer RAMPAGE des grafischen Bildwiederholspeichers im SRAM

; HintergrundDatenRegister:		FCOLOR=Zeichenfarbe
; ****************************************************************************************
CLEARSCREEN:


; RESET SCREEN ADDRESS
	clr		YPOS
	clr		XPOS


; Laden der Endpunkte
	ldi		temp1,Max_X_Pixel	; XEND
	inc		temp1
	ldi		temp2,Max_Y_Pixel	; YEND


; WAIT FOR MEMREADY
CLEAR:
CSMR:
	sbrc	memready,0
	rjmp	CSMR


; SEND SCREEN DATA
	out		port_dat,BCOLOR
	out		port_adr_hig,YPOS
	cbi		port_control,WE	


	out		port_adr_low,XPOS
	inc		XPOS
	out		port_adr_low,XPOS
	inc		XPOS
	out		port_adr_low,XPOS
	inc		XPOS
	out		port_adr_low,XPOS
	inc		XPOS

	out		port_adr_low,XPOS
	inc		XPOS
	out		port_adr_low,XPOS
	inc		XPOS
	out		port_adr_low,XPOS
	inc		XPOS
	out		port_adr_low,XPOS
	inc		XPOS


	out		port_adr_low,XPOS
	inc		XPOS
	out		port_adr_low,XPOS
	inc		XPOS
	out		port_adr_low,XPOS
	inc		XPOS
	out		port_adr_low,XPOS
	inc		XPOS

	out		port_adr_low,XPOS
	inc		XPOS
	out		port_adr_low,XPOS
	inc		XPOS
	out		port_adr_low,XPOS
	inc		XPOS
	out		port_adr_low,XPOS
	inc		XPOS



	sbi		port_control,WE


; COMPLETE LOOP
	cp		XPOS,temp1	; XEND erreicht?
	brne	clear
	inc		YPOS
	cp		YPOS,temp2	; YEND erreicht?
	brne	clear


; ---------------------------
; Laden des Y-Anfangs und Y-Endpunktes mit 0

	ser		temp1

	mov		YPOS, temp1
	clr		XPOS

	out		port_adr_hig,YPOS
	out		port_adr_low,XPOS
	cbi		port_control,WE	
	nop
	sbi		port_control,WE

	mov		XPOS, temp1

	out		port_adr_hig,YPOS
	out		port_adr_low,XPOS
	cbi		port_control,WE	
	nop
	sbi		port_control,WE


; Ruecksprung
;
	ret


 







; **********************************************************************************************
; Unterprogramm: PRINTCHAR6x8z
; Funktion: Einschreiben eines Zeichens im 6x8 Format an der aktuellen Kursorposition in die
; aktuelle RAMPAGE des grafischen Bildwiederholspeicher des SRAM
; Uebergabe:                    Zeichen im ZEI_PUF
; HintergrundDatenRegister:		X_PIX_POS=X-Position des Zeichens
;								Y_PIX_POS=X-Position des Zeichens
;								FCOLOR=Zeichenfarbe
;								BCOLOR=Hintergrundfarbe 
; **********************************************************************************************
PRINTCHAR6x8z:


; Laden der Adresse des Zeichen-Puffers
	ldi		xl, low(ZEI_PUF)
	ldi		xh, high(ZEI_PUF)

; temp1 wird Zaehlregister fuer die angezeigten Zeichen
	clr		temp1
; in temp2 wir die Position des letzten, anzuzeigenden Zeichens im Zeichen-Puffer geladen
	lds		temp2, POS_ZEP



; WAIT FOR MEMREADY
PC68z_MR:
	sbrc	memready,0
	rjmp	PC68z_MR



PC68z_01:

; !!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
; Alle Ausgaben groesser Max_X_Zeichen werden ignoriert 
	mov		temp3, X_KUR_POS
	cpi		temp3, Max_X_Show
	breq	PC68z_60


	push	temp1
	push	temp2

	ld		par01,x+
	



; SET POINTER TO CHARACTER DATA
	ldi		zl,low(2*font6x8)		; Z mit Tabellenanfang laden
	ldi		zh,high(2*font6x8)


	ldi		temp2,8		
	mul		par01,temp2			; Zeichen (par01) * 8 = Position 1 des Zeichens, Ergebnis in -> r0:r1
	add		zl,r0
	adc		zh,r1				; nun zeigt Z auf das 1.Byte des Zeichens in der Zeichentabelle





; LOOP
	mov		YPOS,Y_PIX_POS		; YPOS <- Y_PIX_POS (Y-Position in port_adr_hig)
	ldi		temp4,8				; temp4 mit Anzahl der Bytes in Y-Richtung des Zeichens laden

; Schreiben der Y-Position
PC68z_LY:

; READ CHARACTER BYTE
	lpm		temp2,z+			; Laden des Zeichenbytes in temp2

	lsr		temp2
	lsr		temp2

; SET SCREEN ADDRESS
	mov		XPOS, X_PIX_POS		; XPOS <- X_PIX_POS (X-Position in port_adr_low)
	out		port_adr_hig, YPOS	; Ausgabe yh
	ldi		temp1,6				; temp1 mit Anzahl der Bytes in X-Richtung des Zeichens laden

; Schreiben der X-Position
PC68z_LX:

	out		port_adr_low, XPOS	; Ausgabe XPOS


; Ausschieben des Zeichenbits
	out		port_dat, FCOLOR	; Ausgabe Zeichenfarbe an den Datenport
	sbrc	temp2, 0			; ist Zeichenbyte an der Stelle 0 = 1 (Zeichenfarbe),
	rjmp	PC68z_WE			; dann Sprung zu DOSHIFT


; Ueberspringen, wenn Zeichenfarbe = Hintergrundfarbe (dann bleibt der bereits ausgegebene Wert)
	cp		BCOLOR, FCOLOR		; Zeichenfarbe = Hintergrundfarbe ?
	breq	PC68z_NO			; dann keine Aenderung
	sbrs	temp2, 0			; ist temp2 an der Stelle 0 gesetzt,
								; dann wird das Pixelbyte mit der Hintergrundfarbe gefuellt
	out		port_dat,BCOLOR		; Ausgabe der Hintergrundfarbe an den Datenport 

PC68z_WE:
	cbi		port_control,WE		; Einschreiben des neuen Pixelwertes in den SRAM
	sbi		port_control,WE

PC68z_NO:


; COMPLETE SHIFT LOOP
	lsr		temp2				; Logical Shift Right --> 0 --> |7|6|5|4|3|2|1|0|
								; um das nchste Bit des Pixelbytes auszuwerten 
	inc		XPOS				; X-Position um eine Stelle nach rechts
	dec		temp1				; Dekrement X-Positionszaehler
	brne	PC68z_LX			; wenn nicht 0, dann noch eine X-Pos schreiben
	inc		YPOS				; Y-Position um eine Stelle nach unten
	dec		temp4				; Dekrement Y-Positionszaehler
	brne	PC68z_LY			; wenn nicht 0, dann noch eine Y-Pos schreiben


	; X_PIX_POS + 6
	ldi		temp3,6
	add		X_PIX_POS,temp3


	; X_KUR_POS + 1
	inc		X_KUR_POS



	pop		temp2
	pop		temp1


PC68z_60:

	inc		temp1
	cp		temp1,temp2


	brne	PC68z_01



; Ruecksprung
;
	ret












; **********************************************************************************************
; Unterprogramm: PRINTCHAR
; Funktion: Einschreiben eines Zeichens im 6x8 Format an der aktuellen Kursorposition 
; UebergabeRegister:            par01=Zeichen
; HintergrundDatenRegister:		X_PIX_POS=X-Position des Zeichens
;								Y_PIX_POS=X-Position des Zeichens
;								FCOLOR=Zeichenfarbe
;								BCOLOR=Hintergrundfarbe 
; **********************************************************************************************
;
PRINTCHAR:

	rcall	AKT_CUR_OFF


; Einschalten CUR_PUF_INHALT
	ser		temp1
	sts		CUR_PUF_INHALT, temp1


; !!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
; Alle Ausgaben groesser Max_X_Zeichen werden ignoriert 
	mov		temp3, X_KUR_POS
	cpi		temp3, Max_X_Show
	breq	pc_99


	push	par01
	call	PRINTCHAR6x8
	call	PAGEFLIP
	pop		par01

	call	PRINTCHAR6x8
	call	PAGEFLIP


	; X_PIX_POS + 6
	ldi		temp1,6
	add		X_PIX_POS,temp1

	; X_KUR_POS + 1
	inc		X_KUR_POS


pc_99:

	ret





; **********************************************************************************************
; Unterprogramm: PRINTCHAR6x8
; Funktion: Einschreiben eines Zeichens im 6x8 Format an der aktuellen Kursorposition in die
; aktuelle RAMPAGE des grafischen Bildwiederholspeicher des SRAM
; UebergabeRegister:            par01=Zeichen (wird zerstoehrt)
; HintergrundDatenRegister:		X_PIX_POS=X-Position des Zeichens
;								Y_PIX_POS=X-Position des Zeichens
;								FCOLOR=Zeichenfarbe
;								BCOLOR=Hintergrundfarbe 
; **********************************************************************************************
PRINTCHAR6x8:


; SET POINTER TO CHARACTER DATA
	ldi		zl,low(2*font6x8)		; Z mit Tabellenanfang laden
	ldi		zh,high(2*font6x8)

	ldi		temp2,8		
	mul		par01,temp2			; Zeichen (par01) * 8 = Position 1 des Zeichens, Ergebnis in -> r0:r1
	add		zl,r0
	adc		zh,r1				; nun zeigt Z auf das 1.Byte des Zeichens in der Zeichentabelle


; WAIT FOR MEMREADY
PC68_MR:
	sbrc	memready,0
	rjmp	PC68_MR



PC68_01:


; LOOP
	mov		YPOS,Y_PIX_POS		; YPOS <- Y_PIX_POS (Y-Position in port_adr_hig)
	ldi		temp4,8				; temp4 mit Anzahl der Bytes in Y-Richtung des Zeichens laden

; Schreiben der Y-Position
PC68_LY:

; READ CHARACTER BYTE
	lpm		temp2,z+			; Laden des Zeichenbytes in temp2

	lsr		temp2
	lsr		temp2

; SET SCREEN ADDRESS
	mov		XPOS, X_PIX_POS		; XPOS <- X_PIX_POS (X-Position in port_adr_low)
	out		port_adr_hig, YPOS	; Ausgabe yh
	ldi		temp1,6				; temp1 mit Anzahl der Bytes in X-Richtung des Zeichens laden

; Schreiben der X-Position
PC68_LX:

	out		port_adr_low, XPOS	; Ausgabe XPOS


; Ausschieben des Zeichenbits
	out		port_dat, FCOLOR	; Ausgabe Zeichenfarbe an den Datenport
	sbrc	temp2, 0			; ist Zeichenbyte an der Stelle 0 = 1 (Zeichenfarbe),
	rjmp	PC68_WE				; dann Sprung zu DOSHIFT


; Ueberspringen, wenn Zeichenfarbe = Hintergrundfarbe (dann bleibt der bereits ausgegebene Wert)
	cp		BCOLOR, FCOLOR		; Zeichenfarbe = Hintergrundfarbe ?
	breq	PC68_NO				; dann keine Aenderung
	sbrs	temp2, 0			; ist temp2 an der Stelle 0 gesetzt,
								; dann wird das Pixelbyte mit der Hintergrundfarbe gefuellt
	out		port_dat,BCOLOR		; Ausgabe der Hintergrundfarbe an den Datenport 

PC68_WE:
	cbi		port_control,WE		; Einschreiben des neuen Pixelwertes in den SRAM
	sbi		port_control,WE

PC68_NO:


; COMPLETE SHIFT LOOP
	lsr		temp2				; Logical Shift Right --> 0 --> |7|6|5|4|3|2|1|0|
								; um das nchste Bit des Pixelbytes auszuwerten 
	inc		XPOS				; X-Position um eine Stelle nach rechts
	dec		temp1				; Dekrement X-Positionszaehler
	brne	PC68_LX				; wenn nicht 0, dann noch eine X-Pos schreiben
	inc		YPOS				; Y-Position um eine Stelle nach unten
	dec		temp4				; Dekrement Y-Positionszaehler
	brne	PC68_LY				; wenn nicht 0, dann noch eine Y-Pos schreiben


; Ruecksprung
;
	ret




; **********************************************************************************************
; Unterprogramm: PRINTCHAR8x8
; Funktion: Einschreiben eines Zeichens im 8x8 Format an der aktuellen Kursorposition in die
; aktuelle RAMPAGE des grafischen Bildwiederholspeicher des SRAM
; Uebergaberegister: par01=Zeichen das ausgegeben werden soll (im ASCII-Format)
; HintergrundDatenRegister:		X_PIX_POS=X-Position des Zeichens
;								Y_PIX_POS=X-Position des Zeichens
;								FCOLOR=Zeichenfarbe
;								BCOLOR=Hintergrundfarbe 
; **********************************************************************************************
PRINTCHAR8x8:



; SET POINTER TO CHARACTER DATA

	ldi		zl,low(2*font8x8)		; Z mit Tabellenanfang laden
	ldi		zh,high(2*font8x8)


	ldi		temp2,8		
	mul		par01,temp2			; Zeichen (par01) * 8 = Position 1 des Zeichens, Ergebnis in -> r0:r1
	add		zl,r0
	adc		zh,r1				; nun zeigt Z auf das 1.Byte des Zeichens in der Zeichentabelle


; WAIT FOR MEMREADY
PC88_MR:
	sbrc	memready,0
	rjmp	PC88_MR


; LOOP
	mov		YPOS,Y_PIX_POS		; YPOS <- Y_PIX_POS (Y-Position in port_adr_hig)
	ldi		temp4,8				; temp4 mit Anzahl der Bytes in Y-Richtung des Zeichens laden

; Schreiben der Y-Position
PC88_LY:

; READ CHARACTER BYTE
	lpm		temp2,z+			; Laden des Zeichenbytes in temp2


; SET SCREEN ADDRESS
	mov		XPOS,X_PIX_POS		; XPOS <- X_PIX_POS (X-Position in port_adr_low)
	out		port_adr_hig,YPOS	; Ausgabe yh
	ldi		temp1,8				; temp1 mit Anzahl der Bytes in X-Richtung des Zeichens laden

; Schreiben der X-Position
PC88_LX:

	out		port_adr_low,XPOS	; Ausgabe XPOS


; Ausschieben des Zeichenbits
	out		port_dat,FCOLOR		; Ausgabe Zeichenfarbe an den Datenport
	sbrc	temp2,0				; ist Zeichenbyte an der Stelle 0 = 1 (Zeichenfarbe),
	rjmp	PC88_WE				; dann Sprung zu DOSHIFT


; Ueberspringen, wenn Zeichenfarbe = Hintergrundfarbe (dann bleibt der bereits ausgegebene Wert)
	cp		BCOLOR,FCOLOR		; Zeichenfarbe = Hintergrundfarbe ?
	breq	PC88_NO				; dann keine Aenderung
	sbrs	temp2,0				; ist temp2 an der Stelle 0 gesetzt,
								; dann wird das Pixelbyte mit der Hintergrundfarbe gefuellt
	out		port_dat,BCOLOR		; Ausgabe der Hintergrundfarbe an den Datenport 

PC88_WE:
	cbi		port_control,WE		; Einschreiben des neuen Pixelwertes in den SRAM
	sbi		port_control,WE

PC88_NO:


; COMPLETE SHIFT LOOP
	lsr		temp2					; Logical Shift Right --> 0 --> |7|6|5|4|3|2|1|0|
								; um das nchste Bit des Pixelbytes auszuwerten 
	inc		XPOS				; X-Position um eine Stelle nach rechts
	dec		temp1				; Dekrement X-Positionszaehler
	brne	PC88_LX				; wenn nicht 0, dann noch eine X-Pos schreiben
	inc		YPOS				; Y-Position um eine Stelle nach unten
	dec		temp4					; Dekrement Y-Positionszaehler
	brne	PC88_LY				; wenn nicht 0, dann noch eine Y-Pos schreiben


; Ruecksprung
;
	ret












;****************************************************************************
;****************************************************************************
;	VIDEO - Interruptroutine
;****************************************************************************
;****************************************************************************



 

; ****************************************************************************************
; **** VIDEO RENDERING INTERRUPT / R2=SOUND FREQUENCY / RETURNS R25=MEMREADY
; ****************************************************************************************
VIDEO:


; bei 21,504 MHz
; HSP =  3800 nS / 46,5 nS =  81
; HBP =  1540 nS / 46,5 nS =  33
; HPX = 25600 nS / 46,5 nS = 550
; HFP =   640 nS / 46,5 nS =  14



; ****************************************************************************************
; **** HORIZONTAL AND VERTICAL SYNC = 76 CYCLES
; ****************************************************************************************




; (0)


; SAVE STATUS REGISTER
	in		temp1i,sreg		;1
	push	temp1i			;2

; (3)

; EQUALIZE INTERRUPT LATENCY
; Angleichen der Interrupt Verborgenheit

	lds		temp1i,tcnt1l	;2
	cpi		temp1i,10		;1
	brlo	LATFIX1			;1/2

LATFIX1:
	cpi		temp1i,11		;1
	brlo	LATFIX2			;1/2

LATFIX2:
	cpi		temp1i,12		;1
	brlo	LATFIX3			;1/2

LATFIX3:
	cpi		temp1i,13		;1
	brlo	LATFIX4			;1/2

LATFIX4:


; HORIZONTAL SYNC LOW
	cbi		port_control, HOR	;2


; ---------------------------------------------
;  HSP - HBP - HPX   -    HFP
;   76   31   512         12
;__     ____xxxxxxxxxxxx_____
; |     |
; -------
; 1     2   3           4   5
; |
; ---------------------------------------------
; ab hier 76 Zyklen HSP

; (0)

; PUSH REGISTERS TO STACK
	push	yl					;2
	push	yh					;2
	push	zl					;2
	push	zh					;2




; (8)

; SET POINTER TO VLINE TABLE
	ldi		zl,low(2*VLINE)		;1
	ldi		zh,high(2*VLINE)	;1


; LOAD VLINE COUNTER
	mov		yl,REG_YL			;1
	mov		yh,REG_YH			;1

; (12)


; SET VLINE TABLE POSITION
	add		zl,yl				;1
	adc		zh,yh				;1


; READ VLINE TABLE BYTE
	lpm		temp1i,z			;3


; (17)


; INCREMENT VERTICAL ADDRESS
	sbrs	temp1i,VL_INC_VA		;1/2
	inc		VER_ADR					;1


; RESET VERTICAL ADDRESS
	sbrc	temp1i,VL_RESET			;1/2	; ueberspringen von ser, wenn
	ser		VER_ADR					;1		; Bit0=0 in temp1i


; INCREMENT VLINE COUNTER
	adiw	yh:yl,1					;1


; (22)


; RESET VLINE COUNTER
	sbrc	temp1i,VL_RESET			;1/2
	clr		yl						;1
	sbrc	temp1i,VL_RESET			;1/2
	clr		yh						;1


; SAVE VLINE COUNTER
	mov		REG_YL,yl				;1
	mov		REG_YH,yh				;1

; (28)


; VERTICAL SYNC CONTROL
	sbrc	temp1i,VL_VER_IMP		;1/2
	cbi		port_control,VER		;2
	sbrs	temp1i,VL_VER_IMP		;1/2
	sbi		port_control,VER		;2

; (33)


; SET MEMREADY FLAG
	sbrc	temp1i,VL_MEMREADY		;1/2
	clr		memready				;1
	sbrs	temp1i,VL_MEMREADY		;1/2
	ser		memready				;1


; (37)


; HORIZONTAL SYNC HIGH
	sbi		port_control,HOR		;2

 



; (47)





; ---------------------------------------------
;  HSP - HBP - HPX   -    HFP
;   76   31   512         12
;__     ____xxxxxxxxxxxx_____
; |     |
; -------
; 1     2   3           4   5
;       |
; ---------------------------------------------
; ab hier 31 Zyklen HBP


; (0)



; ****************************************************************************************
; **** HORIZONTAL BACK PORCH = 36 CYCLES
; ****************************************************************************************

; SET VERTICAL ADDRESS
	sbrc	temp1i,VL_OUT_VA		;1/2
	out		port_adr_hig,VER_ADR	;1

; CONTROL ACTIVE LINES
	sbrs	temp1i,VL_OUT_VA		;1/2
	rjmp	NOVID					;2

; (4)

; SET SRAM TO IDLE
	sbi		port_control,WE			;2
	sbi		port_control,OE			;2

; RELEASE DATA BUS
	clr		temp1i					;1
	out		ddr_dat,temp1i			;1

; (10)

; SET SRAM TO READ
	cbi		port_control,OE			;2

; (12)
 

; ****************************************************************************************
; **** HORIZONTAL ACTIVE LINE = 512 CYCLES
; ****************************************************************************************

	; SET TO FRONT BUFFER
	in		temp1i,port_control		;1
	sbrc	temp1i,RAMPAGE			;1/2
	cbi		port_control,RAMPAGE	;2
	sbrs	temp1i,RAMPAGE			;1/2
	sbi		port_control,RAMPAGE	;2


; (18)


	; RESET HORIZONTAL ADDRESS
	clr		temp1i					;1
	out		port_adr_low,temp1i		;1

; (20)

	; TURN ON VIDEO DAC
	cbi		port_control,G			;2



	push	temp1i					;1



; (23)

; ----------------------------------------
; 18 Zyklen

; Einlesen von UCSR0A
	lds		temp1i,UCSR0A 		; 2 

; wurde ein Byte durch die UART empfangen, so ist das RXC0-Bit gesetzt
	sbrs	temp1i,RXC0 		; 1/2 
						
; ansonsten wir die Auswertung uebersprungen
	rjmp	VI_UA_02			; 2

; Laden des empfangenen Bytes von der UART in WERT_RXPi
	lds		WERT_RXP,UDR0		; 2

; Register Z=Pointer auf aktuellen BWS-Wert
	ldi		zl, low(RX_PUF)		; 1
	add		zl, POS_RXP_WR		; 1
	ldi		zh, high(RX_PUF)	; 1

; Abspeichern des Zeichens in den RX-Puffer 
	st		z,WERT_RXP			; 2
	
; Zeiger erhoehen
	inc		POS_RXP_WR			; 1

; Ist die maximale Pufferlaenge erreicht?
	ldi		temp1i,Laenge_RXP	; 1
	cp		POS_RXP_WR,temp1i		; 1

; Wenn nicht Sprung zu Ende
	brne	VI_UA_03			; 1/2

; Ist die maximale Pufferlaenge erreicht, so wird der Zeiger des RX-Puffers zurueck gesetzt
	clr		POS_RXP_WR			; 1

; Sprung zum Ende
	rjmp	VI_UA_99			; 2

VI_UA_02:
; Ausgleich der Routine mit NOP-Befehlen, damit die Routine immer gleich viele Zyklen hat
	nop
	nop
	nop
	nop
	nop
	nop
	nop
	nop
	nop
	nop
	nop

VI_UA_03:

	nop
	nop

VI_UA_99:

; ---------------------------------


; (41)

	pop		temp1i				;1


; (42)

; ---------------------------------------------
;  HSP - HBP - HPX   -    HFP
;   76   31   512         12
;__     ____xxxxxxxxxxxx_____
; |     |
; -------
; 1     2   3           4   5
;           |
; ---------------------------------------------
; ab hier 512 Zyklen HPX


	; PIXEL 1-10
	inc		temp1i				;1
	out		port_adr_low,temp1i	;1
	inc		temp1i				;1
	out		port_adr_low,temp1i	;1
	inc		temp1i				;1
	out		port_adr_low,temp1i	;1
	inc		temp1i				;1
	out		port_adr_low,temp1i	;1
	inc		temp1i				;1
	out		port_adr_low,temp1i	;1
	inc		temp1i				;1
	out		port_adr_low,temp1i	;1
	inc		temp1i				;1
	out		port_adr_low,temp1i	;1
	inc		temp1i				;1
	out		port_adr_low,temp1i	;1
	inc		temp1i				;1
	out		port_adr_low,temp1i	;1
	inc		temp1i				;1
	out		port_adr_low,temp1i	;1


	; PIXEL 11-20
	inc		temp1i				;1
	out		port_adr_low,temp1i	;1
	inc		temp1i				;1
	out		port_adr_low,temp1i	;1
	inc		temp1i				;1
	out		port_adr_low,temp1i	;1
	inc		temp1i				;1
	out		port_adr_low,temp1i	;1
	inc		temp1i				;1
	out		port_adr_low,temp1i	;1
	inc		temp1i				;1
	out		port_adr_low,temp1i	;1
	inc		temp1i				;1
	out		port_adr_low,temp1i	;1
	inc		temp1i				;1
	out		port_adr_low,temp1i	;1
	inc		temp1i				;1
	out		port_adr_low,temp1i	;1
	inc		temp1i				;1
	out		port_adr_low,temp1i	;1



	; PIXEL 21-30
	inc		temp1i				;1
	out		port_adr_low,temp1i	;1
	inc		temp1i				;1
	out		port_adr_low,temp1i	;1
	inc		temp1i				;1
	out		port_adr_low,temp1i	;1
	inc		temp1i				;1
	out		port_adr_low,temp1i	;1
	inc		temp1i				;1
	out		port_adr_low,temp1i	;1
	inc		temp1i				;1
	out		port_adr_low,temp1i	;1
	inc		temp1i				;1
	out		port_adr_low,temp1i	;1
	inc		temp1i				;1
	out		port_adr_low,temp1i	;1
	inc		temp1i				;1
	out		port_adr_low,temp1i	;1
	inc		temp1i				;1
	out		port_adr_low,temp1i	;1



	; PIXEL 31-40
	inc		temp1i				;1
	out		port_adr_low,temp1i	;1
	inc		temp1i				;1
	out		port_adr_low,temp1i	;1
	inc		temp1i				;1
	out		port_adr_low,temp1i	;1
	inc		temp1i				;1
	out		port_adr_low,temp1i	;1
	inc		temp1i				;1
	out		port_adr_low,temp1i	;1
	inc		temp1i				;1
	out		port_adr_low,temp1i	;1
	inc		temp1i				;1
	out		port_adr_low,temp1i	;1
	inc		temp1i				;1
	out		port_adr_low,temp1i	;1
	inc		temp1i				;1
	out		port_adr_low,temp1i	;1
	inc		temp1i				;1
	out		port_adr_low,temp1i	;1



	; PIXEL 41-50
	inc		temp1i				;1
	out		port_adr_low,temp1i	;1
	inc		temp1i				;1
	out		port_adr_low,temp1i	;1
	inc		temp1i				;1
	out		port_adr_low,temp1i	;1
	inc		temp1i				;1
	out		port_adr_low,temp1i	;1
	inc		temp1i				;1
	out		port_adr_low,temp1i	;1
	inc		temp1i				;1
	out		port_adr_low,temp1i	;1
	inc		temp1i				;1
	out		port_adr_low,temp1i	;1
	inc		temp1i				;1
	out		port_adr_low,temp1i	;1
	inc		temp1i				;1
	out		port_adr_low,temp1i	;1
	inc		temp1i				;1
	out		port_adr_low,temp1i	;1



	; PIXEL 51-60
	inc		temp1i				;1
	out		port_adr_low,temp1i	;1
	inc		temp1i				;1
	out		port_adr_low,temp1i	;1
	inc		temp1i				;1
	out		port_adr_low,temp1i	;1
	inc		temp1i				;1
	out		port_adr_low,temp1i	;1
	inc		temp1i				;1
	out		port_adr_low,temp1i	;1
	inc		temp1i				;1
	out		port_adr_low,temp1i	;1
	inc		temp1i				;1
	out		port_adr_low,temp1i	;1
	inc		temp1i				;1
	out		port_adr_low,temp1i	;1
	inc		temp1i				;1
	out		port_adr_low,temp1i	;1
	inc		temp1i				;1
	out		port_adr_low,temp1i	;1



	; PIXEL 61-70
	inc		temp1i				;1
	out		port_adr_low,temp1i	;1
	inc		temp1i				;1
	out		port_adr_low,temp1i	;1
	inc		temp1i				;1
	out		port_adr_low,temp1i	;1
	inc		temp1i				;1
	out		port_adr_low,temp1i	;1
	inc		temp1i				;1
	out		port_adr_low,temp1i	;1
	inc		temp1i				;1
	out		port_adr_low,temp1i	;1
	inc		temp1i				;1
	out		port_adr_low,temp1i	;1
	inc		temp1i				;1
	out		port_adr_low,temp1i	;1
	inc		temp1i				;1
	out		port_adr_low,temp1i	;1
	inc		temp1i				;1
	out		port_adr_low,temp1i	;1



	; PIXEL 71-80
	inc		temp1i				;1
	out		port_adr_low,temp1i	;1
	inc		temp1i				;1
	out		port_adr_low,temp1i	;1
	inc		temp1i				;1
	out		port_adr_low,temp1i	;1
	inc		temp1i				;1
	out		port_adr_low,temp1i	;1
	inc		temp1i				;1
	out		port_adr_low,temp1i	;1
	inc		temp1i				;1
	out		port_adr_low,temp1i	;1
	inc		temp1i				;1
	out		port_adr_low,temp1i	;1
	inc		temp1i				;1
	out		port_adr_low,temp1i	;1
	inc		temp1i				;1
	out		port_adr_low,temp1i	;1
	inc		temp1i				;1
	out		port_adr_low,temp1i	;1



	; PIXEL 81-90
	inc		temp1i				;1
	out		port_adr_low,temp1i	;1
	inc		temp1i				;1
	out		port_adr_low,temp1i	;1
	inc		temp1i				;1
	out		port_adr_low,temp1i	;1
	inc		temp1i				;1
	out		port_adr_low,temp1i	;1
	inc		temp1i				;1
	out		port_adr_low,temp1i	;1
	inc		temp1i				;1
	out		port_adr_low,temp1i	;1
	inc		temp1i				;1
	out		port_adr_low,temp1i	;1
	inc		temp1i				;1
	out		port_adr_low,temp1i	;1
	inc		temp1i				;1
	out		port_adr_low,temp1i	;1
	inc		temp1i				;1
	out		port_adr_low,temp1i	;1



	; PIXEL 91-100
	inc		temp1i				;1
	out		port_adr_low,temp1i	;1
	inc		temp1i				;1
	out		port_adr_low,temp1i	;1
	inc		temp1i				;1
	out		port_adr_low,temp1i	;1
	inc		temp1i				;1
	out		port_adr_low,temp1i	;1
	inc		temp1i				;1
	out		port_adr_low,temp1i	;1
	inc		temp1i				;1
	out		port_adr_low,temp1i	;1
	inc		temp1i				;1
	out		port_adr_low,temp1i	;1
	inc		temp1i				;1
	out		port_adr_low,temp1i	;1
	inc		temp1i				;1
	out		port_adr_low,temp1i	;1
	inc		temp1i				;1
	out		port_adr_low,temp1i	;1



	; PIXEL 101-110
	inc		temp1i				;1
	out		port_adr_low,temp1i	;1
	inc		temp1i				;1
	out		port_adr_low,temp1i	;1
	inc		temp1i				;1
	out		port_adr_low,temp1i	;1
	inc		temp1i				;1
	out		port_adr_low,temp1i	;1
	inc		temp1i				;1
	out		port_adr_low,temp1i	;1
	inc		temp1i				;1
	out		port_adr_low,temp1i	;1
	inc		temp1i				;1
	out		port_adr_low,temp1i	;1
	inc		temp1i				;1
	out		port_adr_low,temp1i	;1
	inc		temp1i				;1
	out		port_adr_low,temp1i	;1
	inc		temp1i				;1
	out		port_adr_low,temp1i	;1



	; PIXEL 111-120
	inc		temp1i				;1
	out		port_adr_low,temp1i	;1
	inc		temp1i				;1
	out		port_adr_low,temp1i	;1
	inc		temp1i				;1
	out		port_adr_low,temp1i	;1
	inc		temp1i				;1
	out		port_adr_low,temp1i	;1
	inc		temp1i				;1
	out		port_adr_low,temp1i	;1
	inc		temp1i				;1
	out		port_adr_low,temp1i	;1
	inc		temp1i				;1
	out		port_adr_low,temp1i	;1
	inc		temp1i				;1
	out		port_adr_low,temp1i	;1
	inc		temp1i				;1
	out		port_adr_low,temp1i	;1
	inc		temp1i				;1
	out		port_adr_low,temp1i	;1



	; PIXEL 121-130
	inc		temp1i				;1
	out		port_adr_low,temp1i	;1
	inc		temp1i				;1
	out		port_adr_low,temp1i	;1
	inc		temp1i				;1
	out		port_adr_low,temp1i	;1
	inc		temp1i				;1
	out		port_adr_low,temp1i	;1
	inc		temp1i				;1
	out		port_adr_low,temp1i	;1
	inc		temp1i				;1
	out		port_adr_low,temp1i	;1
	inc		temp1i				;1
	out		port_adr_low,temp1i	;1
	inc		temp1i				;1
	out		port_adr_low,temp1i	;1
	inc		temp1i				;1
	out		port_adr_low,temp1i	;1
	inc		temp1i				;1
	out		port_adr_low,temp1i	;1



	; PIXEL 131-140
	inc		temp1i				;1
	out		port_adr_low,temp1i	;1
	inc		temp1i				;1
	out		port_adr_low,temp1i	;1
	inc		temp1i				;1
	out		port_adr_low,temp1i	;1
	inc		temp1i				;1
	out		port_adr_low,temp1i	;1
	inc		temp1i				;1
	out		port_adr_low,temp1i	;1
	inc		temp1i				;1
	out		port_adr_low,temp1i	;1
	inc		temp1i				;1
	out		port_adr_low,temp1i	;1
	inc		temp1i				;1
	out		port_adr_low,temp1i	;1
	inc		temp1i				;1
	out		port_adr_low,temp1i	;1
	inc		temp1i				;1
	out		port_adr_low,temp1i	;1



	; PIXEL 141-150
	inc		temp1i				;1
	out		port_adr_low,temp1i	;1
	inc		temp1i				;1
	out		port_adr_low,temp1i	;1
	inc		temp1i				;1
	out		port_adr_low,temp1i	;1
	inc		temp1i				;1
	out		port_adr_low,temp1i	;1
	inc		temp1i				;1
	out		port_adr_low,temp1i	;1
	inc		temp1i				;1
	out		port_adr_low,temp1i	;1
	inc		temp1i				;1
	out		port_adr_low,temp1i	;1
	inc		temp1i				;1
	out		port_adr_low,temp1i	;1
	inc		temp1i				;1
	out		port_adr_low,temp1i	;1
	inc		temp1i				;1
	out		port_adr_low,temp1i	;1


	; PIXEL 151-160
	inc		temp1i				;1
	out		port_adr_low,temp1i	;1
	inc		temp1i				;1
	out		port_adr_low,temp1i	;1
	inc		temp1i				;1
	out		port_adr_low,temp1i	;1
	inc		temp1i				;1
	out		port_adr_low,temp1i	;1
	inc		temp1i				;1
	out		port_adr_low,temp1i	;1
	inc		temp1i				;1
	out		port_adr_low,temp1i	;1
	inc		temp1i				;1
	out		port_adr_low,temp1i	;1
	inc		temp1i				;1
	out		port_adr_low,temp1i	;1
	inc		temp1i				;1
	out		port_adr_low,temp1i	;1
	inc		temp1i				;1
	out		port_adr_low,temp1i	;1



	; PIXEL 161-170
	inc		temp1i				;1
	out		port_adr_low,temp1i	;1
	inc		temp1i				;1
	out		port_adr_low,temp1i	;1
	inc		temp1i				;1
	out		port_adr_low,temp1i	;1
	inc		temp1i				;1
	out		port_adr_low,temp1i	;1
	inc		temp1i				;1
	out		port_adr_low,temp1i	;1
	inc		temp1i				;1
	out		port_adr_low,temp1i	;1
	inc		temp1i				;1
	out		port_adr_low,temp1i	;1
	inc		temp1i				;1
	out		port_adr_low,temp1i	;1
	inc		temp1i				;1
	out		port_adr_low,temp1i	;1
	inc		temp1i				;1
	out		port_adr_low,temp1i	;1



	; PIXEL 171-180
	inc		temp1i				;1
	out		port_adr_low,temp1i	;1
	inc		temp1i				;1
	out		port_adr_low,temp1i	;1
	inc		temp1i				;1
	out		port_adr_low,temp1i	;1
	inc		temp1i				;1
	out		port_adr_low,temp1i	;1
	inc		temp1i				;1
	out		port_adr_low,temp1i	;1
	inc		temp1i				;1
	out		port_adr_low,temp1i	;1
	inc		temp1i				;1
	out		port_adr_low,temp1i	;1
	inc		temp1i				;1
	out		port_adr_low,temp1i	;1
	inc		temp1i				;1
	out		port_adr_low,temp1i	;1
	inc		temp1i				;1
	out		port_adr_low,temp1i	;1



	; PIXEL 181-190
	inc		temp1i				;1
	out		port_adr_low,temp1i	;1
	inc		temp1i				;1
	out		port_adr_low,temp1i	;1
	inc		temp1i				;1
	out		port_adr_low,temp1i	;1
	inc		temp1i				;1
	out		port_adr_low,temp1i	;1
	inc		temp1i				;1
	out		port_adr_low,temp1i	;1
	inc		temp1i				;1
	out		port_adr_low,temp1i	;1
	inc		temp1i				;1
	out		port_adr_low,temp1i	;1
	inc		temp1i				;1
	out		port_adr_low,temp1i	;1
	inc		temp1i				;1
	out		port_adr_low,temp1i	;1
	inc		temp1i				;1
	out		port_adr_low,temp1i	;1



	; PIXEL 191-200
	inc		temp1i				;1
	out		port_adr_low,temp1i	;1
	inc		temp1i				;1
	out		port_adr_low,temp1i	;1
	inc		temp1i				;1
	out		port_adr_low,temp1i	;1
	inc		temp1i				;1
	out		port_adr_low,temp1i	;1
	inc		temp1i				;1
	out		port_adr_low,temp1i	;1
	inc		temp1i				;1
	out		port_adr_low,temp1i	;1
	inc		temp1i				;1
	out		port_adr_low,temp1i	;1
	inc		temp1i				;1
	out		port_adr_low,temp1i	;1
	inc		temp1i				;1
	out		port_adr_low,temp1i	;1
	inc		temp1i				;1
	out		port_adr_low,temp1i	;1


	; PIXEL 201-210
	inc		temp1i				;1
	out		port_adr_low,temp1i	;1
	inc		temp1i				;1
	out		port_adr_low,temp1i	;1
	inc		temp1i				;1
	out		port_adr_low,temp1i	;1
	inc		temp1i				;1
	out		port_adr_low,temp1i	;1
	inc		temp1i				;1
	out		port_adr_low,temp1i	;1
	inc		temp1i				;1
	out		port_adr_low,temp1i	;1
	inc		temp1i				;1
	out		port_adr_low,temp1i	;1
	inc		temp1i				;1
	out		port_adr_low,temp1i	;1
	inc		temp1i				;1
	out		port_adr_low,temp1i	;1
	inc		temp1i				;1
	out		port_adr_low,temp1i	;1



	; PIXEL 211-220
	inc		temp1i				;1
	out		port_adr_low,temp1i	;1
	inc		temp1i				;1
	out		port_adr_low,temp1i	;1
	inc		temp1i				;1
	out		port_adr_low,temp1i	;1
	inc		temp1i				;1
	out		port_adr_low,temp1i	;1
	inc		temp1i				;1
	out		port_adr_low,temp1i	;1
	inc		temp1i				;1
	out		port_adr_low,temp1i	;1
	inc		temp1i				;1
	out		port_adr_low,temp1i	;1
	inc		temp1i				;1
	out		port_adr_low,temp1i	;1
	inc		temp1i				;1
	out		port_adr_low,temp1i	;1
	inc		temp1i				;1
	out		port_adr_low,temp1i	;1



	; PIXEL 221-230
	inc		temp1i				;1
	out		port_adr_low,temp1i	;1
	inc		temp1i				;1
	out		port_adr_low,temp1i	;1
	inc		temp1i				;1
	out		port_adr_low,temp1i	;1
	inc		temp1i				;1
	out		port_adr_low,temp1i	;1
	inc		temp1i				;1
	out		port_adr_low,temp1i	;1
	inc		temp1i				;1
	out		port_adr_low,temp1i	;1
	inc		temp1i				;1
	out		port_adr_low,temp1i	;1
	inc		temp1i				;1
	out		port_adr_low,temp1i	;1
	inc		temp1i				;1
	out		port_adr_low,temp1i	;1
	inc		temp1i				;1
	out		port_adr_low,temp1i	;1


	; PIXEL 231-240
	inc		temp1i				;1
	out		port_adr_low,temp1i	;1
	inc		temp1i				;1
	out		port_adr_low,temp1i	;1
	inc		temp1i				;1
	out		port_adr_low,temp1i	;1
	inc		temp1i				;1
	out		port_adr_low,temp1i	;1
	inc		temp1i				;1
	out		port_adr_low,temp1i	;1
	inc		temp1i				;1
	out		port_adr_low,temp1i	;1
	inc		temp1i				;1
	out		port_adr_low,temp1i	;1
	inc		temp1i				;1
	out		port_adr_low,temp1i	;1
	inc		temp1i				;1
	out		port_adr_low,temp1i	;1
	inc		temp1i				;1
	out		port_adr_low,temp1i	;1


	; PIXEL 241-250
	inc		temp1i				;1
	out		port_adr_low,temp1i	;1
	inc		temp1i				;1
	out		port_adr_low,temp1i	;1
	inc		temp1i				;1
	out		port_adr_low,temp1i	;1
	inc		temp1i				;1
	out		port_adr_low,temp1i	;1
	inc		temp1i				;1
	out		port_adr_low,temp1i	;1
	inc		temp1i				;1
	out		port_adr_low,temp1i	;1
	inc		temp1i				;1
	out		port_adr_low,temp1i	;1
	inc		temp1i				;1
	out		port_adr_low,temp1i	;1
	inc		temp1i				;1
	out		port_adr_low,temp1i	;1
	inc		temp1i				;1
	out		port_adr_low,temp1i	;1


	; PIXEL 251-256
	inc		temp1i				;1
	out		port_adr_low,temp1i	;1
	inc		temp1i				;1
	out		port_adr_low,temp1i	;1
	inc		temp1i				;1
	out		port_adr_low,temp1i	;1
	inc		temp1i				;1
	out		port_adr_low,temp1i	;1
	inc		temp1i				;1
	out		port_adr_low,temp1i	;1


; 256*2=512



; (0)

; TURN OFF VIDEO DAC
	sbi		port_control,G			;2


; (2)

; SET TO BACK BUFFER
	in		temp1i,port_control		;1
	sbrc	temp1i,RAMPAGE			;1/2
	cbi		port_control,RAMPAGE	;2
	sbrs	temp1i,RAMPAGE			;1/2
	sbi		port_control,RAMPAGE	;2

; (8)


; SET SRAM TO IDLE
	sbi		port_control,OE			;2


; ENABLE DATA BUS
	ser		temp1i					;1
	out		ddr_dat,temp1i			;1

; (12) 





; ------------------------------------------------------

; maximal 20 Zyklen !!!
; minimal 4 (wenn RX_STOP = FF)
; normal  13 (wenn POS_RXP_WR >= POS_RXP_RD)

VI_XOFF:
	; 1. Vergleich, ob XOFF bereits gesendet wurde und 
	;    RX_STOP = FF ist
	ser		temp1i					; 1
	cp		temp1i, RX_STOP			; 1
	breq	VI_XOFF_80 				; 1/2

	lds		WERT_RXP, POS_RXP_RD	; 2   WERT_RXP <- POS_RXP_RD
	cp		POS_RXP_WR, WERT_RXP	; 1
	brcc	VI_XOFF_01				; 1/2 Sprung, wenn: Normalfall: POS_RXP_WR >= POS_RXP_RD 

	; POS_RXP_WR ist kleiner als POS_RXP_RD und muss korregiert werden
	ldi		temp1i, Laenge_RXP		; 1
	sub		temp1i, WERT_RXP		; 1   temp1i = Laenge_RXP - POS_RXP_RD
	add		temp1i, POS_RXP_WR		; 1   temp1i = (Laenge_RXP + POS_RXP_WR) - POS_RXP_RD
	rjmp	VI_XOFF_02				; 2

	; Normalfall: POS_RXP_WR >= POS_RXP_RD
VI_XOFF_01:
	; ... dann temp1i = POS_RXP_WR - POS_RXP_RD
	mov		temp1i, POS_RXP_WR		; 1
	sub		temp1i, WERT_RXP		; 1

	; jetzt Vergleich, ob im Puffer mehr als Laenge_RX_STOP Zeichen gespeichert wurden
VI_XOFF_02:
	cpi		temp1i, Laenge_RX_STOP	; 1
	brcs	VI_XOFF_80				; 1/2 Sprung, wenn Zeichenzahl < Laenge_RX_STOP


	ldi		temp1i, XOFF			; 1
	sts		UDR0, temp1i			; 2

	ser		temp1i					; 1
	mov		RX_STOP, temp1i			; 1


VI_XOFF_80:

; ------------------------------------------------------



; (30)



; ****************************************************************************************
; **** HORIZONTAL FRONT PORCH = 12 CYCLES
; ****************************************************************************************



; BLANKING LINES EXIT POINT
NOVID:


; POP REGISTERS FROM STACK
	pop		zh						;2
	pop		zl						;2
	pop		yh						;2
	pop		yl						;2


; RESTORE STATUS REGISTER
	pop		temp1i			 ;2
	out		sreg,temp1i		 ;1


; RETURN FROM INTERRUPT
	reti					 ;4


; (35)


; ---------------------------------------------
;  HSP - HBP - HPX   -    HFP
;   76   31   512         12
;__     ____xxxxxxxxxxxx_____
; |     |
; -------
; 1     2   3           4   5
; |
; ---------------------------------------------


;19+ 47 + 42 +  512  +    35 = 655

; bei 22 MHz (45,45 ns) = 30 us (max. 31,75 us)








; ****************************************************************************************
; **** VERTICAL LINE LOOKUP TABLE
; ****************************************************************************************

.equ 	BY_OT		= (1<<VL_OUT_VA)
.equ 	BY_IN_OT	= (1<<VL_INC_VA | 1<<VL_OUT_VA)
.equ 	BY_MR		= (1<<VL_MEMREADY)
.equ 	BY_MR_VI	= (1<<VL_MEMREADY | 1<<VL_VER_IMP)
.equ 	BY_ZERO		= 0
.equ 	BY_RT		= (1<<VL_RESET)

VLINE:

.db BY_OT,BY_IN_OT,BY_OT,BY_IN_OT,BY_OT,BY_IN_OT,BY_OT,BY_IN_OT,BY_OT,BY_IN_OT,BY_OT,BY_IN_OT 
.db BY_OT,BY_IN_OT,BY_OT,BY_IN_OT,BY_OT,BY_IN_OT,BY_OT,BY_IN_OT,BY_OT,BY_IN_OT,BY_OT,BY_IN_OT
.db BY_OT,BY_IN_OT,BY_OT,BY_IN_OT,BY_OT,BY_IN_OT,BY_OT,BY_IN_OT,BY_OT,BY_IN_OT,BY_OT,BY_IN_OT 
.db BY_OT,BY_IN_OT,BY_OT,BY_IN_OT,BY_OT,BY_IN_OT,BY_OT,BY_IN_OT,BY_OT,BY_IN_OT,BY_OT,BY_IN_OT
.db BY_OT,BY_IN_OT,BY_OT,BY_IN_OT,BY_OT,BY_IN_OT,BY_OT,BY_IN_OT,BY_OT,BY_IN_OT,BY_OT,BY_IN_OT 
.db BY_OT,BY_IN_OT,BY_OT,BY_IN_OT,BY_OT,BY_IN_OT,BY_OT,BY_IN_OT,BY_OT,BY_IN_OT,BY_OT,BY_IN_OT
.db BY_OT,BY_IN_OT,BY_OT,BY_IN_OT,BY_OT,BY_IN_OT,BY_OT,BY_IN_OT,BY_OT,BY_IN_OT,BY_OT,BY_IN_OT 
.db BY_OT,BY_IN_OT,BY_OT,BY_IN_OT,BY_OT,BY_IN_OT,BY_OT,BY_IN_OT,BY_OT,BY_IN_OT,BY_OT,BY_IN_OT
.db BY_OT,BY_IN_OT,BY_OT,BY_IN_OT,BY_OT,BY_IN_OT,BY_OT,BY_IN_OT,BY_OT,BY_IN_OT,BY_OT,BY_IN_OT 
.db BY_OT,BY_IN_OT,BY_OT,BY_IN_OT,BY_OT,BY_IN_OT,BY_OT,BY_IN_OT,BY_OT,BY_IN_OT,BY_OT,BY_IN_OT
.db BY_OT,BY_IN_OT,BY_OT,BY_IN_OT,BY_OT,BY_IN_OT,BY_OT,BY_IN_OT,BY_OT,BY_IN_OT,BY_OT,BY_IN_OT 
.db BY_OT,BY_IN_OT,BY_OT,BY_IN_OT,BY_OT,BY_IN_OT,BY_OT,BY_IN_OT,BY_OT,BY_IN_OT,BY_OT,BY_IN_OT
.db BY_OT,BY_IN_OT,BY_OT,BY_IN_OT,BY_OT,BY_IN_OT,BY_OT,BY_IN_OT,BY_OT,BY_IN_OT,BY_OT,BY_IN_OT 
.db BY_OT,BY_IN_OT,BY_OT,BY_IN_OT,BY_OT,BY_IN_OT,BY_OT,BY_IN_OT,BY_OT,BY_IN_OT,BY_OT,BY_IN_OT
.db BY_OT,BY_IN_OT,BY_OT,BY_IN_OT,BY_OT,BY_IN_OT,BY_OT,BY_IN_OT,BY_OT,BY_IN_OT,BY_OT,BY_IN_OT 
.db BY_OT,BY_IN_OT,BY_OT,BY_IN_OT,BY_OT,BY_IN_OT,BY_OT,BY_IN_OT,BY_OT,BY_IN_OT,BY_OT,BY_IN_OT
.db BY_OT,BY_IN_OT,BY_OT,BY_IN_OT,BY_OT,BY_IN_OT,BY_OT,BY_IN_OT,BY_OT,BY_IN_OT,BY_OT,BY_IN_OT 
.db BY_OT,BY_IN_OT,BY_OT,BY_IN_OT,BY_OT,BY_IN_OT,BY_OT,BY_IN_OT,BY_OT,BY_IN_OT,BY_OT,BY_IN_OT
.db BY_OT,BY_IN_OT,BY_OT,BY_IN_OT,BY_OT,BY_IN_OT,BY_OT,BY_IN_OT,BY_OT,BY_IN_OT,BY_OT,BY_IN_OT 
.db BY_OT,BY_IN_OT,BY_OT,BY_IN_OT,BY_OT,BY_IN_OT,BY_OT,BY_IN_OT,BY_OT,BY_IN_OT,BY_OT,BY_IN_OT
.db BY_OT,BY_IN_OT,BY_OT,BY_IN_OT,BY_OT,BY_IN_OT,BY_OT,BY_IN_OT,BY_OT,BY_IN_OT,BY_OT,BY_IN_OT 
.db BY_OT,BY_IN_OT,BY_OT,BY_IN_OT,BY_OT,BY_IN_OT,BY_OT,BY_IN_OT,BY_OT,BY_IN_OT,BY_OT,BY_IN_OT
.db BY_OT,BY_IN_OT,BY_OT,BY_IN_OT,BY_OT,BY_IN_OT,BY_OT,BY_IN_OT,BY_OT,BY_IN_OT,BY_OT,BY_IN_OT 
.db BY_OT,BY_IN_OT,BY_OT,BY_IN_OT,BY_OT,BY_IN_OT,BY_OT,BY_IN_OT,BY_OT,BY_IN_OT,BY_OT,BY_IN_OT
.db BY_OT,BY_IN_OT,BY_OT,BY_IN_OT,BY_OT,BY_IN_OT,BY_OT,BY_IN_OT,BY_OT,BY_IN_OT,BY_OT,BY_IN_OT 
.db BY_OT,BY_IN_OT,BY_OT,BY_IN_OT,BY_OT,BY_IN_OT,BY_OT,BY_IN_OT,BY_OT,BY_IN_OT,BY_OT,BY_IN_OT
.db BY_OT,BY_IN_OT,BY_OT,BY_IN_OT,BY_OT,BY_IN_OT,BY_OT,BY_IN_OT,BY_OT,BY_IN_OT,BY_OT,BY_IN_OT 
.db BY_OT,BY_IN_OT,BY_OT,BY_IN_OT,BY_OT,BY_IN_OT,BY_OT,BY_IN_OT,BY_OT,BY_IN_OT,BY_OT,BY_IN_OT
.db BY_OT,BY_IN_OT,BY_OT,BY_IN_OT,BY_OT,BY_IN_OT,BY_OT,BY_IN_OT,BY_OT,BY_IN_OT,BY_OT,BY_IN_OT 
.db BY_OT,BY_IN_OT,BY_OT,BY_IN_OT,BY_OT,BY_IN_OT,BY_OT,BY_IN_OT,BY_OT,BY_IN_OT,BY_OT,BY_IN_OT
.db BY_OT,BY_IN_OT,BY_OT,BY_IN_OT,BY_OT,BY_IN_OT,BY_OT,BY_IN_OT,BY_OT,BY_IN_OT,BY_OT,BY_IN_OT 
.db BY_OT,BY_IN_OT,BY_OT,BY_IN_OT,BY_OT,BY_IN_OT,BY_OT,BY_IN_OT,BY_OT,BY_IN_OT,BY_OT,BY_IN_OT
.db BY_OT,BY_IN_OT,BY_OT,BY_IN_OT,BY_OT,BY_IN_OT,BY_OT,BY_IN_OT,BY_OT,BY_IN_OT,BY_OT,BY_IN_OT 
.db BY_OT,BY_IN_OT,BY_OT,BY_IN_OT,BY_OT,BY_IN_OT,BY_OT,BY_IN_OT,BY_OT,BY_IN_OT,BY_OT,BY_IN_OT
.db BY_OT,BY_IN_OT,BY_OT,BY_IN_OT,BY_OT,BY_IN_OT,BY_OT,BY_IN_OT,BY_OT,BY_IN_OT,BY_OT,BY_IN_OT 
.db BY_OT,BY_IN_OT,BY_OT,BY_IN_OT,BY_OT,BY_IN_OT,BY_OT,BY_IN_OT,BY_OT,BY_IN_OT,BY_OT,BY_IN_OT
.db BY_OT,BY_IN_OT,BY_OT,BY_IN_OT,BY_OT,BY_IN_OT,BY_OT,BY_IN_OT,BY_OT,BY_IN_OT,BY_OT,BY_IN_OT 
.db BY_OT,BY_IN_OT,BY_OT,BY_IN_OT,BY_OT,BY_IN_OT,BY_OT,BY_IN_OT,BY_OT,BY_IN_OT,BY_OT,BY_IN_OT
.db BY_OT,BY_IN_OT,BY_OT,BY_IN_OT,BY_OT,BY_IN_OT,BY_OT,BY_IN_OT,BY_OT,BY_IN_OT,BY_OT,BY_IN_OT 
.db BY_OT,BY_IN_OT,BY_OT,BY_IN_OT,BY_OT,BY_IN_OT,BY_OT,BY_IN_OT,BY_OT,BY_IN_OT,BY_OT,BY_IN_OT

.db BY_MR,BY_MR,BY_MR,BY_MR,BY_MR,BY_MR,BY_MR,BY_MR,BY_MR,BY_MR,BY_MR,BY_MR_VI,BY_MR_VI,BY_MR
.db BY_MR,BY_MR,BY_MR,BY_MR,BY_MR,BY_MR,BY_MR,BY_MR,BY_MR,BY_MR,BY_MR,BY_MR,BY_MR,BY_MR
.db BY_MR,BY_MR,BY_MR,BY_MR,BY_MR,BY_MR,BY_MR,BY_MR,BY_MR,BY_MR,BY_MR,BY_MR,BY_MR,BY_MR

.db BY_MR,BY_ZERO,BY_RT,BY_ZERO



 

; **********************************************************************************************
; **** ALPHANUMERIC CHARACTER SET
; **********************************************************************************************

  
font6x8:
.db 0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00	; 0x00
.db 0x70,0x88,0xD8,0x88,0xA8,0x88,0x70,0x00	; 0x01
.db 0x70,0xF8,0xA8,0xF8,0x88,0xF8,0x70,0x00	; 0x02
.db 0x00,0x50,0xF8,0xF8,0xF8,0x70,0x20,0x00	; 0x03
.db 0x00,0x20,0x70,0xF8,0xF8,0x70,0x20,0x00	; 0x04
.db 0x20,0x70,0x70,0x20,0xF8,0xF8,0x20,0x00	; 0x05
.db 0x00,0x20,0x70,0xF8,0xF8,0x20,0x70,0x00	; 0x06
.db 0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00	; 0x07
.db 0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00	; 0x08
.db 0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00	; 0x09
.db 0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00	; 0x0A
.db 0x00,0xE0,0xC0,0xB0,0x48,0x48,0x30,0x00	; 0x0B
.db 0x70,0x88,0x88,0x70,0x20,0x70,0x20,0x00	; 0x0C
.db 0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00	; 0x0D
.db 0xC0,0xB0,0xD0,0xB0,0xD0,0xD8,0x18,0x00	; 0x0E
.db 0x00,0xA8,0x70,0xD8,0x70,0xA8,0x00,0x00	; 0x0F
.db 0x10,0x30,0x70,0xF0,0x70,0x30,0x10,0x00	; 0x10
.db 0x40,0x60,0x70,0x78,0x70,0x60,0x40,0x00	; 0x11
.db 0x20,0x70,0xF8,0x20,0xF8,0x70,0x20,0x00	; 0x12
.db 0x50,0x50,0x50,0x50,0x50,0x00,0x50,0x00	; 0x13
.db 0xF0,0xA8,0xA8,0xB0,0xA0,0xA0,0xA0,0x00	; 0x14
.db 0x70,0x88,0x30,0x50,0x60,0x88,0x70,0x00	; 0x15
.db 0x00,0x00,0x00,0x00,0x00,0x78,0x78,0x00	; 0x16
.db 0x20,0x70,0xF8,0x20,0xF8,0x70,0x20,0x70	; 0x17
.db 0x20,0x70,0xF8,0x20,0x20,0x20,0x20,0x00	; 0x18
.db 0x20,0x20,0x20,0x20,0xF8,0x70,0x20,0x00	; 0x19
.db 0x00,0x20,0x60,0xF8,0x60,0x20,0x00,0x00	; 0x1A
.db 0x00,0x20,0x30,0xF8,0x30,0x20,0x00,0x00	; 0x1B
.db 0x00,0x00,0x00,0x08,0x08,0x08,0xF8,0x00	; 0x1C
.db 0x00,0x50,0x50,0xF8,0x50,0x50,0x00,0x00	; 0x1D
.db 0x20,0x20,0x70,0x70,0xF8,0xF8,0x00,0x00	; 0x1E
.db 0xF8,0xF8,0x70,0x70,0x20,0x20,0x00,0x00	; 0x1F
.db 0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00	; 0x20
.db 0x20,0x70,0x70,0x20,0x20,0x00,0x20,0x00	; 0x21
.db 0xD8,0xD8,0x48,0x00,0x00,0x00,0x00,0x00	; 0x22
.db 0x00,0x50,0xF8,0x50,0x50,0xF8,0x50,0x00	; 0x23
.db 0x10,0x70,0x08,0x30,0x40,0x38,0x20,0x00	; 0x24
.db 0x98,0x98,0x40,0x20,0x10,0xC8,0xC8,0x00	; 0x25
.db 0x10,0x28,0x28,0x10,0xA8,0x48,0xB0,0x00	; 0x26
.db 0x30,0x30,0x10,0x00,0x00,0x00,0x00,0x00	; 0x27
.db 0x20,0x10,0x10,0x10,0x10,0x10,0x20,0x00	; 0x28
.db 0x10,0x20,0x20,0x20,0x20,0x20,0x10,0x00	; 0x29
.db 0x00,0x50,0x70,0xF8,0x70,0x50,0x00,0x00	; 0x2A
.db 0x00,0x20,0x20,0xF8,0x20,0x20,0x00,0x00	; 0x2B
.db 0x00,0x00,0x00,0x00,0x00,0x30,0x30,0x10	; 0x2C
.db 0x00,0x00,0x00,0xF8,0x00,0x00,0x00,0x00	; 0x2D
.db 0x00,0x00,0x00,0x00,0x00,0x30,0x30,0x00	; 0x2E
.db 0x00,0x80,0x40,0x20,0x10,0x08,0x00,0x00	; 0x2F
.db 0x70,0x88,0xC8,0xA8,0x98,0x88,0x70,0x00	; 0x30
.db 0x20,0x30,0x20,0x20,0x20,0x20,0x70,0x00	; 0x31
.db 0x70,0x88,0x80,0x60,0x10,0x08,0xF8,0x00	; 0x32
.db 0x70,0x88,0x80,0x70,0x80,0x88,0x70,0x00	; 0x33
.db 0x40,0x60,0x50,0x48,0xF8,0x40,0x40,0x00	; 0x34
.db 0xF8,0x08,0x08,0x78,0x80,0x88,0x70,0x00	; 0x35
.db 0x60,0x10,0x08,0x78,0x88,0x88,0x70,0x00	; 0x36
.db 0xF8,0x80,0x40,0x20,0x10,0x10,0x10,0x00	; 0x37
.db 0x70,0x88,0x88,0x70,0x88,0x88,0x70,0x00	; 0x38
.db 0x70,0x88,0x88,0xF0,0x80,0x40,0x30,0x00	; 0x39
.db 0x00,0x00,0x30,0x30,0x00,0x30,0x30,0x00	; 0x3A
.db 0x00,0x00,0x30,0x30,0x00,0x30,0x30,0x10	; 0x3B
.db 0x40,0x20,0x10,0x08,0x10,0x20,0x40,0x00	; 0x3C
.db 0x00,0x00,0xF8,0x00,0x00,0xF8,0x00,0x00	; 0x3D
.db 0x10,0x20,0x40,0x80,0x40,0x20,0x10,0x00	; 0x3E
.db 0x70,0x88,0x80,0x60,0x20,0x00,0x20,0x00	; 0x3F
.db 0x70,0x88,0xE8,0xA8,0xE8,0x08,0x70,0x00	; 0x40
.db 0x70,0x88,0x88,0x88,0xF8,0x88,0x88,0x00	; 0x41
.db 0x78,0x88,0x88,0x78,0x88,0x88,0x78,0x00	; 0x42
.db 0x70,0x88,0x08,0x08,0x08,0x88,0x70,0x00	; 0x43
.db 0x78,0x88,0x88,0x88,0x88,0x88,0x78,0x00	; 0x44
.db 0xF8,0x08,0x08,0x78,0x08,0x08,0xF8,0x00	; 0x45
.db 0xF8,0x08,0x08,0x78,0x08,0x08,0x08,0x00	; 0x46
.db 0x70,0x88,0x08,0xE8,0x88,0x88,0xF0,0x00	; 0x47
.db 0x88,0x88,0x88,0xF8,0x88,0x88,0x88,0x00	; 0x48
.db 0x70,0x20,0x20,0x20,0x20,0x20,0x70,0x00	; 0x49
.db 0x80,0x80,0x80,0x80,0x88,0x88,0x70,0x00	; 0x4A
.db 0x88,0x48,0x28,0x18,0x28,0x48,0x88,0x00	; 0x4B
.db 0x08,0x08,0x08,0x08,0x08,0x08,0xF8,0x00	; 0x4C
.db 0x88,0xD8,0xA8,0x88,0x88,0x88,0x88,0x00	; 0x4D
.db 0x88,0x98,0xA8,0xC8,0x88,0x88,0x88,0x00	; 0x4E
.db 0x70,0x88,0x88,0x88,0x88,0x88,0x70,0x00	; 0x4F
.db 0x78,0x88,0x88,0x78,0x08,0x08,0x08,0x00	; 0x50
.db 0x70,0x88,0x88,0x88,0xA8,0x48,0xB0,0x00	; 0x51
.db 0x78,0x88,0x88,0x78,0x48,0x88,0x88,0x00	; 0x52
.db 0x70,0x88,0x08,0x70,0x80,0x88,0x70,0x00	; 0x53
.db 0xF8,0x20,0x20,0x20,0x20,0x20,0x20,0x00	; 0x54
.db 0x88,0x88,0x88,0x88,0x88,0x88,0x70,0x00	; 0x55
.db 0x88,0x88,0x88,0x88,0x88,0x50,0x20,0x00	; 0x56
.db 0x88,0x88,0xA8,0xA8,0xA8,0xA8,0x50,0x00	; 0x57
.db 0x88,0x88,0x50,0x20,0x50,0x88,0x88,0x00	; 0x58
.db 0x88,0x88,0x88,0x50,0x20,0x20,0x20,0x00	; 0x59
.db 0x78,0x40,0x20,0x10,0x08,0x08,0x78,0x00	; 0x5A
.db 0x70,0x10,0x10,0x10,0x10,0x10,0x70,0x00	; 0x5B
.db 0x00,0x08,0x10,0x20,0x40,0x80,0x00,0x00	; 0x5C
.db 0x70,0x40,0x40,0x40,0x40,0x40,0x70,0x00	; 0x5D
.db 0x20,0x50,0x88,0x00,0x00,0x00,0x00,0x00	; 0x5E
CHAR_:
.db 0x00,0x00,0x00,0x00,0x00,0x00,0x00,0xF8	; 0x5F
.db 0x30,0x30,0x20,0x00,0x00,0x00,0x00,0x00	; 0x60
.db 0x00,0x00,0x70,0x80,0xF0,0x88,0xF0,0x00	; 0x61
.db 0x08,0x08,0x78,0x88,0x88,0x88,0x78,0x00	; 0x62
.db 0x00,0x00,0x70,0x88,0x08,0x88,0x70,0x00	; 0x63
.db 0x80,0x80,0xF0,0x88,0x88,0x88,0xF0,0x00	; 0x64
.db 0x00,0x00,0x70,0x88,0x78,0x08,0x70,0x00	; 0x65
.db 0x60,0x10,0x10,0x78,0x10,0x10,0x10,0x00	; 0x66
.db 0x00,0x00,0xF0,0x88,0x88,0xF0,0x80,0x70	; 0x67
.db 0x08,0x08,0x38,0x48,0x48,0x48,0x48,0x00	; 0x68
.db 0x20,0x00,0x20,0x20,0x20,0x20,0x60,0x00	; 0x69
.db 0x40,0x00,0x60,0x40,0x40,0x40,0x48,0x30	; 0x6A
.db 0x08,0x08,0x48,0x28,0x18,0x28,0x48,0x00	; 0x6B
.db 0x20,0x20,0x20,0x20,0x20,0x20,0x60,0x00	; 0x6C
.db 0x00,0x00,0x58,0xA8,0xA8,0x88,0x88,0x00	; 0x6D
.db 0x00,0x00,0x38,0x48,0x48,0x48,0x48,0x00	; 0x6E
.db 0x00,0x00,0x70,0x88,0x88,0x88,0x70,0x00	; 0x6F
.db 0x00,0x00,0x78,0x88,0x88,0x88,0x78,0x08	; 0x70
.db 0x00,0x00,0xF0,0x88,0x88,0x88,0xF0,0x80	; 0x71
.db 0x00,0x00,0x68,0x90,0x10,0x10,0x38,0x00	; 0x72
.db 0x00,0x00,0x70,0x08,0x70,0x80,0x70,0x00	; 0x73
.db 0x00,0x10,0x78,0x10,0x10,0x50,0x20,0x00	; 0x74
.db 0x00,0x00,0x48,0x48,0x48,0x68,0x50,0x00	; 0x75
.db 0x00,0x00,0x88,0x88,0x88,0x50,0x20,0x00	; 0x76
.db 0x00,0x00,0x88,0x88,0xA8,0xF8,0x50,0x00	; 0x77
.db 0x00,0x00,0x48,0x48,0x30,0x48,0x48,0x00	; 0x78
.db 0x00,0x00,0x48,0x48,0x48,0x70,0x20,0x18	; 0x79
.db 0x00,0x00,0x78,0x40,0x30,0x08,0x78,0x00	; 0x7A
.db 0x60,0x10,0x10,0x18,0x10,0x10,0x60,0x00	; 0x7B
.db 0x20,0x20,0x20,0x00,0x20,0x20,0x20,0x00	; 0x7C
.db 0x30,0x40,0x40,0xC0,0x40,0x40,0x30,0x00	; 0x7D
.db 0x50,0x28,0x00,0x00,0x00,0x00,0x00,0x00	; 0x7E
.db 0x20,0x70,0xD8,0x88,0x88,0xF8,0x00,0x00	; 0x7F
.db 0x70,0x88,0x08,0x08,0x88,0x70,0x20,0x30	; 0x80
.db 0x48,0x00,0x48,0x48,0x48,0x68,0x50,0x00	; 0x81
.db 0xC0,0x00,0x70,0x88,0x78,0x08,0x70,0x00	; 0x82
.db 0x70,0x00,0x70,0x80,0xF0,0x88,0xF0,0x00	; 0x83
.db 0x50,0x00,0x70,0x80,0xF0,0x88,0xF0,0x00	; 0x84
.db 0x30,0x00,0x70,0x80,0xF0,0x88,0xF0,0x00	; 0x85
.db 0x70,0x50,0x70,0x80,0xF0,0x88,0xF0,0x00	; 0x86
.db 0x00,0x70,0x88,0x08,0x88,0x70,0x20,0x30	; 0x87
.db 0x70,0x00,0x70,0x88,0x78,0x08,0x70,0x00	; 0x88
.db 0x50,0x00,0x70,0x88,0x78,0x08,0x70,0x00	; 0x89
.db 0x30,0x00,0x70,0x88,0x78,0x08,0x70,0x00	; 0x8A
.db 0x50,0x00,0x20,0x20,0x20,0x20,0x60,0x00	; 0x8B
.db 0x70,0x00,0x20,0x20,0x20,0x20,0x60,0x00	; 0x8C
.db 0x10,0x00,0x20,0x20,0x20,0x20,0x60,0x00	; 0x8D
.db 0x50,0x00,0x20,0x50,0x88,0xF8,0x88,0x00	; 0x8E
.db 0x70,0x50,0x70,0xD8,0x88,0xF8,0x88,0x00	; 0x8F
.db 0xC0,0x00,0xF8,0x08,0x78,0x08,0xF8,0x00	; 0x90
.db 0x00,0x00,0x78,0xA0,0xF8,0x28,0xF0,0x00	; 0x91
.db 0xF0,0x28,0x28,0xF8,0x28,0x28,0xE8,0x00	; 0x92
.db 0x70,0x00,0x30,0x48,0x48,0x48,0x30,0x00	; 0x93
.db 0x50,0x00,0x30,0x48,0x48,0x48,0x30,0x00	; 0x94
.db 0x18,0x00,0x30,0x48,0x48,0x48,0x30,0x00	; 0x95
.db 0x70,0x00,0x48,0x48,0x48,0x68,0x50,0x00	; 0x96
.db 0x18,0x00,0x48,0x48,0x48,0x68,0x50,0x00	; 0x97
.db 0x50,0x00,0x48,0x48,0x48,0x70,0x20,0x18	; 0x98
.db 0x48,0x30,0x48,0x48,0x48,0x48,0x30,0x00	; 0x99
.db 0x50,0x00,0x48,0x48,0x48,0x48,0x30,0x00	; 0x9A
.db 0x00,0x00,0x80,0x70,0x68,0x58,0x38,0x04	; 0x9B
.db 0x60,0x90,0x10,0x78,0x10,0x90,0xE8,0x00	; 0x9C
.db 0xF0,0xC8,0xA8,0xA8,0xA8,0x98,0x78,0x00	; 0x9D
.db 0x00,0x88,0x50,0x20,0x50,0x88,0x00,0x00	; 0x9E
.db 0x40,0xA0,0x20,0x70,0x20,0x20,0x28,0x10	; 0x9F
.db 0x60,0x00,0x70,0x80,0xF0,0x88,0xF0,0x00	; 0xA0
.db 0x60,0x00,0x20,0x20,0x20,0x20,0x60,0x00	; 0xA1
.db 0x60,0x00,0x30,0x48,0x48,0x48,0x30,0x00	; 0xA2
.db 0x60,0x00,0x48,0x48,0x48,0x68,0x50,0x00	; 0xA3
.db 0x50,0x28,0x00,0x38,0x48,0x48,0x48,0x00	; 0xA4
.db 0x50,0x28,0x00,0x48,0x58,0x68,0x48,0x00	; 0xA5
.db 0x70,0x80,0xF0,0x88,0xF0,0x00,0xF0,0x00	; 0xA6
.db 0x30,0x48,0x48,0x48,0x30,0x00,0x78,0x00	; 0xA7
.db 0x20,0x00,0x20,0x30,0x08,0x88,0x70,0x00	; 0xA8
.db 0x78,0xA4,0xD4,0xB4,0xD4,0x84,0x78,0x00	; 0xA9
.db 0x00,0x00,0xFC,0x80,0x80,0x00,0x00,0x00	; 0xAA
.db 0x08,0x48,0x28,0x70,0x88,0x40,0xE0,0x00	; 0xAB
.db 0x08,0x48,0x28,0xD0,0xA8,0xE0,0x80,0x00	; 0xAC
.db 0x20,0x00,0x20,0x20,0x70,0x70,0x20,0x00	; 0xAD
.db 0x00,0x00,0x90,0x48,0x90,0x00,0x00,0x00	; 0xAE
.db 0x00,0x00,0x48,0x90,0x48,0x00,0x00,0x00	; 0xAF
.db 0xA8,0x00,0x54,0x00,0xA8,0x00,0x54,0x00	; 0xB0
.db 0xA8,0x54,0xA8,0x54,0xA8,0x54,0xA8,0x54	; 0xB1
.db 0x54,0xFC,0xA8,0xFC,0x54,0xFC,0xA8,0xFC	; 0xB2
.db 0x20,0x20,0x20,0x20,0x20,0x20,0x20,0x20	; 0xB3
.db 0x20,0x20,0x20,0x3C,0x20,0x20,0x20,0x20	; 0xB4
.db 0x60,0x00,0x20,0x50,0x88,0xF8,0x88,0x00	; 0xB5
.db 0x70,0x00,0x20,0x50,0x88,0xF8,0x88,0x00	; 0xB6
.db 0x30,0x00,0x20,0x50,0x88,0xF8,0x88,0x00	; 0xB7
.db 0x78,0x84,0xB4,0x94,0xB4,0x84,0x78,0x00	; 0xB8
.db 0x28,0x2C,0x20,0x2C,0x28,0x28,0x28,0x28	; 0xB9
.db 0x28,0x28,0x28,0x28,0x28,0x28,0x28,0x28	; 0xBA
.db 0x00,0x3C,0x20,0x2C,0x28,0x28,0x28,0x28	; 0xBB
.db 0x28,0x2C,0x20,0x3C,0x00,0x00,0x00,0x00	; 0xBC
.db 0x00,0x20,0x70,0x08,0x08,0x70,0x20,0x00	; 0xBD
.db 0x88,0x50,0x20,0xF8,0x20,0xF8,0x20,0x00	; 0xBE
.db 0x00,0x00,0x00,0x3C,0x20,0x20,0x20,0x20	; 0xBF
.db 0x20,0x20,0x20,0xE0,0x00,0x00,0x00,0x00	; 0xC0
.db 0x20,0x20,0x20,0xFC,0x00,0x00,0x00,0x00	; 0xC1
.db 0x00,0x00,0x00,0xFC,0x20,0x20,0x20,0x20	; 0xC2
.db 0x20,0x20,0x20,0xE0,0x20,0x20,0x20,0x20	; 0xC3
.db 0x00,0x00,0x00,0xFC,0x00,0x00,0x00,0x00	; 0xC4
.db 0x20,0x20,0x20,0xFC,0x20,0x20,0x20,0x20	; 0xC5
.db 0xA0,0x50,0x70,0x80,0xF0,0x88,0xF0,0x00	; 0xC6
.db 0xA0,0x50,0x20,0x50,0x88,0xF8,0x88,0x00	; 0xC7
.db 0x28,0xE8,0x08,0xF8,0x00,0x00,0x00,0x00	; 0xC8
.db 0x00,0xF8,0x08,0xE8,0x28,0x28,0x28,0x28	; 0xC9
.db 0x28,0xEC,0x00,0xFC,0x00,0x00,0x00,0x00	; 0xCA
.db 0x00,0xFC,0x00,0xEC,0x28,0x28,0x28,0x28	; 0xCB
.db 0x28,0xE8,0x08,0xE8,0x28,0x28,0x28,0x28	; 0xCC
.db 0x00,0xFC,0x00,0xFC,0x00,0x00,0x00,0x00	; 0xCD
.db 0x28,0xEC,0x00,0xEC,0x28,0x28,0x28,0x28	; 0xCE
.db 0x88,0x70,0x88,0x88,0x88,0x70,0x88,0x00	; 0xCF
.db 0x30,0x08,0x10,0x20,0x70,0x48,0x30,0x00	; 0xD0
.db 0x70,0x90,0x90,0xB8,0x90,0x90,0x70,0x00	; 0xD1
.db 0x70,0x00,0xF8,0x08,0x78,0x08,0xF8,0x00	; 0xD2
.db 0x50,0x00,0xF8,0x08,0x78,0x08,0xF8,0x00	; 0xD3
.db 0x30,0x00,0xF8,0x08,0x78,0x08,0xF8,0x00	; 0xD4
.db 0x20,0x20,0x20,0x00,0x00,0x00,0x00,0x00	; 0xD5
.db 0x60,0x00,0x70,0x20,0x20,0x20,0x70,0x00	; 0xD6
.db 0x70,0x00,0x70,0x20,0x20,0x20,0x70,0x00	; 0xD7
.db 0x50,0x00,0x70,0x20,0x20,0x20,0x70,0x00	; 0xD8
.db 0x20,0x20,0x20,0x3C,0x00,0x00,0x00,0x00	; 0xD9
.db 0x00,0x00,0x00,0xE0,0x20,0x20,0x20,0x20	; 0xDA
.db 0xFC,0xFC,0xFC,0xFC,0xFC,0xFC,0xFC,0xFC	; 0xDB
.db 0x00,0x00,0x00,0x00,0xFC,0xFC,0xFC,0xFC	; 0xDC
.db 0x20,0x20,0x20,0x00,0x20,0x20,0x20,0x00	; 0xDD
.db 0x30,0x00,0x70,0x20,0x20,0x20,0x70,0x00	; 0xDE
.db 0xFC,0xFC,0xFC,0xFC,0x00,0x00,0x00,0x00	; 0xDF
.db 0x60,0x30,0x48,0x48,0x48,0x48,0x30,0x00	; 0xE0
.db 0x00,0x38,0x48,0x38,0x48,0x48,0x38,0x08	; 0xE1
.db 0x70,0x30,0x48,0x48,0x48,0x48,0x30,0x00	; 0xE2
.db 0x18,0x30,0x48,0x48,0x48,0x48,0x30,0x00	; 0xE3
.db 0x50,0x28,0x00,0x30,0x48,0x48,0x30,0x00	; 0xE4
.db 0x50,0x28,0x30,0x48,0x48,0x48,0x30,0x00	; 0xE5
.db 0x00,0x00,0x48,0x48,0x48,0x38,0x08,0x08	; 0xE6
.db 0x00,0x18,0x08,0x38,0x48,0x38,0x08,0x18	; 0xE7
.db 0x18,0x08,0x38,0x48,0x48,0x38,0x08,0x18	; 0xE8
.db 0x60,0x00,0x48,0x48,0x48,0x48,0x30,0x00	; 0xE9
.db 0x70,0x00,0x48,0x48,0x48,0x48,0x30,0x00	; 0xEA
.db 0x18,0x00,0x48,0x48,0x48,0x48,0x30,0x00	; 0xEB
.db 0x60,0x00,0x48,0x48,0x48,0x70,0x20,0x18	; 0xEC
.db 0x60,0x00,0x88,0x50,0x20,0x20,0x20,0x00	; 0xED
.db 0x00,0x70,0x00,0x00,0x00,0x00,0x00,0x00	; 0xEE
.db 0x30,0x30,0x10,0x00,0x00,0x00,0x00,0x00	; 0xEF
.db 0x00,0x00,0x00,0x70,0x00,0x00,0x00,0x00	; 0xF0
.db 0x00,0x20,0x70,0x20,0x00,0x70,0x00,0x00	; 0xF1
.db 0x00,0x00,0xF8,0x00,0x00,0xF8,0x00,0x00	; 0xF2
.db 0x0C,0x58,0x2C,0xD0,0xA8,0xE0,0x80,0x00	; 0xF3
.db 0xF0,0xA8,0xA8,0xB0,0xA0,0xA0,0xA0,0x00	; 0xF4
.db 0x70,0x88,0x30,0x50,0x60,0x88,0x70,0x00	; 0xF5
.db 0x00,0x20,0x00,0xF8,0x00,0x20,0x00,0x00	; 0xF6
.db 0x00,0x00,0x00,0x70,0x60,0x00,0x00,0x00	; 0xF7
.db 0x30,0x48,0x48,0x30,0x00,0x00,0x00,0x00	; 0xF8
.db 0x00,0x00,0x00,0x50,0x00,0x00,0x00,0x00	; 0xF9
.db 0x00,0x00,0x00,0x10,0x00,0x00,0x00,0x00	; 0xFA
.db 0x10,0x18,0x10,0x10,0x00,0x00,0x00,0x00	; 0xFB
.db 0x38,0x10,0x30,0x18,0x00,0x00,0x00,0x00	; 0xFC
.db 0x18,0x20,0x10,0x38,0x00,0x00,0x00,0x00	; 0xFD
.db 0x00,0x00,0x78,0x78,0x78,0x78,0x00,0x00	; 0xFE
.db 0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00	; 0xFF 



font8x8:
.db 0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00	; 0x00
.db 0x7E,0x81,0xA5,0x81,0xBD,0x99,0x81,0x7E	; 0x01
.db 0x7E,0xFF,0xDB,0xFF,0xC3,0xE7,0xFF,0x7E	; 0x02
.db 0x36,0x7F,0x7F,0x7F,0x3E,0x1C,0x08,0x00	; 0x03
.db 0x08,0x1C,0x3E,0x7F,0x3E,0x1C,0x08,0x00	; 0x04
.db 0x1C,0x3E,0x1C,0x7F,0x7F,0x6B,0x08,0x1C	; 0x05
.db 0x08,0x08,0x1C,0x3E,0x7F,0x3E,0x08,0x1C	; 0x06
.db 0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00	; 0x07
.db 0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00	; 0x08
.db 0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00	; 0x09
.db 0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00	; 0x0A
.db 0xF0,0xE0,0xF0,0xBE,0x33,0x33,0x33,0x1E	; 0x0B
.db 0x3C,0x66,0x66,0x66,0x3C,0x18,0x7E,0x18	; 0x0C
.db 0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00	; 0x0D
.db 0xFE,0xC6,0xFE,0xC6,0xC6,0xE6,0x67,0x03	; 0x0E
.db 0x99,0x5A,0x3C,0xE7,0xE7,0x3C,0x5A,0x99	; 0x0F
.db 0x01,0x07,0x1F,0x7F,0x1F,0x07,0x01,0x00	; 0x10
.db 0x40,0x70,0x7C,0x7F,0x7C,0x70,0x40,0x00	; 0x11
.db 0x18,0x3C,0x7E,0x18,0x18,0x7E,0x3C,0x18	; 0x12
.db 0x66,0x66,0x66,0x66,0x66,0x00,0x66,0x00	; 0x13
.db 0xFE,0xDB,0xDB,0xDE,0xD8,0xD8,0xD8,0x00	; 0x14
.db 0x7E,0xC3,0x1E,0x33,0x33,0x1E,0x31,0x1F	; 0x15
.db 0x00,0x00,0x00,0x00,0x7E,0x7E,0x7E,0x00	; 0x16
.db 0x18,0x3C,0x7E,0x18,0x7E,0x3C,0x18,0xFF	; 0x17
.db 0x18,0x3C,0x7E,0x18,0x18,0x18,0x18,0x00	; 0x18
.db 0x18,0x18,0x18,0x18,0x7E,0x3C,0x18,0x00	; 0x19
.db 0x00,0x18,0x30,0x7F,0x30,0x18,0x00,0x00	; 0x1A
.db 0x00,0x0C,0x06,0x7F,0x06,0x0C,0x00,0x00	; 0x1B
.db 0x00,0x00,0x03,0x03,0x03,0x7F,0x00,0x00	; 0x1C
.db 0x00,0x24,0x66,0xFF,0x66,0x24,0x00,0x00	; 0x1D
.db 0x00,0x18,0x3C,0x7E,0xFF,0xFF,0x00,0x00	; 0x1E
.db 0x00,0xFF,0xFF,0x7E,0x3C,0x18,0x00,0x00	; 0x1F
.db 0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00	; 0x20
.db 0x0C,0x1E,0x1E,0x0C,0x0C,0x00,0x0C,0x00	; 0x21
.db 0x36,0x36,0x36,0x00,0x00,0x00,0x00,0x00	; 0x22
.db 0x36,0x36,0x7F,0x36,0x7F,0x36,0x36,0x00	; 0x23
.db 0x0C,0x3E,0x03,0x1E,0x30,0x1F,0x0C,0x00	; 0x24
.db 0x00,0x63,0x33,0x18,0x0C,0x66,0x63,0x00	; 0x25
.db 0x1C,0x36,0x1C,0x6E,0x3B,0x33,0x6E,0x00	; 0x26
.db 0x06,0x06,0x03,0x00,0x00,0x00,0x00,0x00	; 0x27
.db 0x18,0x0C,0x06,0x06,0x06,0x0C,0x18,0x00	; 0x28
.db 0x06,0x0C,0x18,0x18,0x18,0x0C,0x06,0x00	; 0x29
.db 0x00,0x66,0x3C,0xFF,0x3C,0x66,0x00,0x00	; 0x2A
.db 0x00,0x0C,0x0C,0x3F,0x0C,0x0C,0x00,0x00	; 0x2B
.db 0x00,0x00,0x00,0x00,0x00,0x0E,0x0C,0x06	; 0x2C
.db 0x00,0x00,0x00,0x3F,0x00,0x00,0x00,0x00	; 0x2D
.db 0x00,0x00,0x00,0x00,0x00,0x0C,0x0C,0x00	; 0x2E
.db 0x60,0x30,0x18,0x0C,0x06,0x03,0x01,0x00	; 0x2F
.db 0x1E,0x33,0x3B,0x3F,0x37,0x33,0x1E,0x00	; 0x30
.db 0x0C,0x0F,0x0C,0x0C,0x0C,0x0C,0x3F,0x00	; 0x31
.db 0x1E,0x33,0x30,0x1C,0x06,0x33,0x3F,0x00	; 0x32
.db 0x1E,0x33,0x30,0x1C,0x30,0x33,0x1E,0x00	; 0x33
.db 0x38,0x3C,0x36,0x33,0x7F,0x30,0x30,0x00	; 0x34
.db 0x3F,0x03,0x1F,0x30,0x30,0x33,0x1E,0x00	; 0x35
.db 0x1C,0x06,0x03,0x1F,0x33,0x33,0x1E,0x00	; 0x36
.db 0x3F,0x33,0x30,0x18,0x0C,0x06,0x06,0x00	; 0x37
.db 0x1E,0x33,0x33,0x1E,0x33,0x33,0x1E,0x00	; 0x38
.db 0x1E,0x33,0x33,0x3E,0x30,0x18,0x0E,0x00	; 0x39
.db 0x00,0x00,0x0C,0x0C,0x00,0x0C,0x0C,0x00	; 0x3A
.db 0x00,0x00,0x0C,0x0C,0x00,0x0E,0x0C,0x06	; 0x3B
.db 0x18,0x0C,0x06,0x03,0x06,0x0C,0x18,0x00	; 0x3C
.db 0x00,0x00,0x3F,0x00,0x3F,0x00,0x00,0x00	; 0x3D
.db 0x06,0x0C,0x18,0x30,0x18,0x0C,0x06,0x00	; 0x3E
.db 0x1E,0x33,0x30,0x18,0x0C,0x00,0x0C,0x00	; 0x3F
.db 0x3E,0x63,0x7B,0x7B,0x7B,0x03,0x1E,0x00	; 0x40
.db 0x0C,0x1E,0x33,0x33,0x3F,0x33,0x33,0x00	; 0x41
.db 0x3F,0x66,0x66,0x3E,0x66,0x66,0x3F,0x00	; 0x42
.db 0x3C,0x66,0x03,0x03,0x03,0x66,0x3C,0x00	; 0x43
.db 0x3F,0x36,0x66,0x66,0x66,0x36,0x3F,0x00	; 0x44
.db 0x7F,0x46,0x16,0x1E,0x16,0x46,0x7F,0x00	; 0x45
.db 0x7F,0x46,0x16,0x1E,0x16,0x06,0x0F,0x00	; 0x46
.db 0x3C,0x66,0x03,0x03,0x73,0x66,0x7C,0x00	; 0x47
.db 0x33,0x33,0x33,0x3F,0x33,0x33,0x33,0x00	; 0x48
.db 0x1E,0x0C,0x0C,0x0C,0x0C,0x0C,0x1E,0x00	; 0x49
.db 0x78,0x30,0x30,0x30,0x33,0x33,0x1E,0x00	; 0x4A
.db 0x67,0x66,0x36,0x1E,0x36,0x66,0x67,0x00	; 0x4B
.db 0x0F,0x06,0x06,0x06,0x46,0x66,0x7F,0x00	; 0x4C
.db 0x63,0x77,0x7F,0x6B,0x63,0x63,0x63,0x00	; 0x4D
.db 0x63,0x67,0x6F,0x7B,0x73,0x63,0x63,0x00	; 0x4E
.db 0x1C,0x36,0x63,0x63,0x63,0x36,0x1C,0x00	; 0x4F
.db 0x3F,0x66,0x66,0x3E,0x06,0x06,0x0F,0x00	; 0x50
.db 0x1E,0x33,0x33,0x33,0x3B,0x1E,0x38,0x00	; 0x51
.db 0x3F,0x66,0x66,0x3E,0x1E,0x36,0x67,0x00	; 0x52
.db 0x1E,0x33,0x07,0x1C,0x38,0x33,0x1E,0x00	; 0x53
.db 0x3F,0x2D,0x0C,0x0C,0x0C,0x0C,0x1E,0x00	; 0x54
.db 0x33,0x33,0x33,0x33,0x33,0x33,0x3F,0x00	; 0x55
.db 0x33,0x33,0x33,0x33,0x33,0x1E,0x0C,0x00	; 0x56
.db 0x63,0x63,0x63,0x6B,0x7F,0x77,0x63,0x00	; 0x57
.db 0x63,0x63,0x36,0x1C,0x36,0x63,0x63,0x00	; 0x58
.db 0x33,0x33,0x33,0x1E,0x0C,0x0C,0x1E,0x00	; 0x59
.db 0x7F,0x33,0x19,0x0C,0x46,0x63,0x7F,0x00	; 0x5A
.db 0x1E,0x06,0x06,0x06,0x06,0x06,0x1E,0x00	; 0x5B
.db 0x03,0x06,0x0C,0x18,0x30,0x60,0x40,0x00	; 0x5C
.db 0x1E,0x18,0x18,0x18,0x18,0x18,0x1E,0x00	; 0x5D
.db 0x08,0x1C,0x36,0x63,0x00,0x00,0x00,0x00	; 0x5E
.db 0x00,0x00,0x00,0x00,0x00,0x00,0x00,0xFF	; 0x5F
.db 0x0C,0x0C,0x18,0x00,0x00,0x00,0x00,0x00	; 0x60
.db 0x00,0x00,0x1E,0x30,0x3E,0x33,0x6E,0x00	; 0x61
.db 0x07,0x06,0x3E,0x66,0x66,0x66,0x3D,0x00	; 0x62
.db 0x00,0x00,0x1E,0x33,0x03,0x33,0x1E,0x00	; 0x63
.db 0x38,0x30,0x30,0x3E,0x33,0x33,0x6E,0x00	; 0x64
.db 0x00,0x00,0x1E,0x33,0x3F,0x03,0x1E,0x00	; 0x65
.db 0x1C,0x36,0x06,0x0F,0x06,0x06,0x0F,0x00	; 0x66
.db 0x00,0x00,0x6E,0x33,0x33,0x3E,0x30,0x1F	; 0x67
.db 0x07,0x06,0x36,0x6E,0x66,0x66,0x67,0x00	; 0x68
.db 0x0C,0x00,0x0E,0x0C,0x0C,0x0C,0x1E,0x00	; 0x69
.db 0x18,0x00,0x1E,0x18,0x18,0x18,0x1B,0x0E	; 0x6A
.db 0x07,0x06,0x66,0x36,0x1E,0x36,0x67,0x00	; 0x6B
.db 0x0E,0x0C,0x0C,0x0C,0x0C,0x0C,0x1E,0x00	; 0x6C
.db 0x00,0x00,0x37,0x7F,0x6B,0x63,0x63,0x00	; 0x6D
.db 0x00,0x00,0x1F,0x33,0x33,0x33,0x33,0x00	; 0x6E
.db 0x00,0x00,0x1E,0x33,0x33,0x33,0x1E,0x00	; 0x6F
.db 0x00,0x00,0x3B,0x66,0x66,0x3E,0x06,0x0F	; 0x70
.db 0x00,0x00,0x6E,0x33,0x33,0x3E,0x30,0x78	; 0x71
.db 0x00,0x00,0x1B,0x36,0x36,0x06,0x0F,0x00	; 0x72
.db 0x00,0x00,0x3E,0x03,0x1E,0x30,0x1F,0x00	; 0x73
.db 0x08,0x0C,0x3E,0x0C,0x0C,0x2C,0x18,0x00	; 0x74
.db 0x00,0x00,0x33,0x33,0x33,0x33,0x6E,0x00	; 0x75
.db 0x00,0x00,0x33,0x33,0x33,0x1E,0x0C,0x00	; 0x76
.db 0x00,0x00,0x63,0x63,0x6B,0x7F,0x36,0x00	; 0x77
.db 0x00,0x00,0x63,0x36,0x1C,0x36,0x63,0x00	; 0x78
.db 0x00,0x00,0x33,0x33,0x33,0x3E,0x30,0x1F	; 0x79
.db 0x00,0x00,0x3F,0x19,0x0C,0x26,0x3F,0x00	; 0x7A
.db 0x38,0x0C,0x0C,0x07,0x0C,0x0C,0x38,0x00	; 0x7B
.db 0x18,0x18,0x18,0x00,0x18,0x18,0x18,0x00	; 0x7C
.db 0x07,0x0C,0x0C,0x38,0x0C,0x0C,0x07,0x00	; 0x7D
.db 0x6E,0x3B,0x00,0x00,0x00,0x00,0x00,0x00	; 0x7E
.db 0x08,0x1C,0x36,0x63,0x63,0x63,0x7F,0x00	; 0x7F
.db 0x1E,0x33,0x03,0x03,0x33,0x1E,0x0C,0x06	; 0x80
.db 0x00,0x33,0x00,0x33,0x33,0x33,0x7E,0x00	; 0x81
.db 0x18,0x0C,0x1E,0x33,0x3F,0x03,0x1E,0x00	; 0x82
.db 0x7E,0xC3,0x3C,0x60,0x7C,0x66,0xFC,0x00	; 0x83
.db 0x33,0x00,0x1E,0x30,0x3E,0x33,0x7E,0x00	; 0x84
.db 0x06,0x0C,0x1E,0x30,0x3E,0x33,0x7E,0x00	; 0x85
.db 0x3C,0x66,0x3C,0x60,0x7C,0x66,0xFC,0x00	; 0x86
.db 0x00,0x1E,0x33,0x03,0x33,0x1E,0x0C,0x06	; 0x87
.db 0x7E,0xC3,0x3C,0x66,0x7E,0x06,0x3C,0x00	; 0x88
.db 0x33,0x00,0x1E,0x33,0x3F,0x03,0x1E,0x00	; 0x89
.db 0x06,0x0C,0x1E,0x33,0x3F,0x03,0x1E,0x00	; 0x8A
.db 0x33,0x00,0x0E,0x0C,0x0C,0x0C,0x1E,0x00	; 0x8B
.db 0x3E,0x63,0x1C,0x18,0x18,0x18,0x3C,0x00	; 0x8C
.db 0x06,0x0C,0x0E,0x0C,0x0C,0x0C,0x1E,0x00	; 0x8D
.db 0x33,0x0C,0x1E,0x33,0x33,0x3F,0x33,0x00	; 0x8E
.db 0x0C,0x12,0x0C,0x1E,0x33,0x3F,0x33,0x00	; 0x8F
.db 0x18,0x0C,0x3F,0x06,0x1E,0x06,0x3F,0x00	; 0x90
.db 0x00,0x00,0xFE,0x30,0xFE,0x33,0xFE,0x00	; 0x91
.db 0x7C,0x36,0x33,0x7F,0x33,0x33,0x73,0x00	; 0x92
.db 0x1E,0x33,0x00,0x1E,0x33,0x33,0x1E,0x00	; 0x93
.db 0x00,0x33,0x00,0x1E,0x33,0x33,0x1E,0x00	; 0x94
.db 0x06,0x0C,0x00,0x1E,0x33,0x33,0x1E,0x00	; 0x95
.db 0x1E,0x33,0x00,0x33,0x33,0x33,0x7E,0x00	; 0x96
.db 0x06,0x0C,0x00,0x33,0x33,0x33,0x7E,0x00	; 0x97
.db 0x00,0x33,0x00,0x33,0x33,0x3F,0x30,0x1F	; 0x98
.db 0x63,0x00,0x3E,0x63,0x63,0x63,0x3E,0x00	; 0x99
.db 0x33,0x00,0x33,0x33,0x33,0x33,0x1E,0x00	; 0x9A
.db 0x00,0x00,0x3E,0x73,0x6B,0x67,0x3E,0x00	; 0x9B
.db 0x1C,0x36,0x26,0x0F,0x06,0x67,0x3F,0x00	; 0x9C
.db 0x5C,0x36,0x73,0x6B,0x67,0x36,0x1D,0x00	; 0x9D
.db 0x00,0x00,0x33,0x1E,0x0C,0x1E,0x33,0x00	; 0x9E
.db 0x70,0xD8,0x18,0x7E,0x18,0x18,0x1B,0x0E	; 0x9F
.db 0x18,0x0C,0x1E,0x30,0x3E,0x33,0x7E,0x00	; 0xA0
.db 0x18,0x0C,0x0E,0x0C,0x0C,0x0C,0x1E,0x00	; 0xA1
.db 0x30,0x18,0x00,0x1E,0x33,0x33,0x1E,0x00	; 0xA2
.db 0x30,0x18,0x00,0x33,0x33,0x33,0x7E,0x00	; 0xA3
.db 0x6E,0x3B,0x00,0x1F,0x33,0x33,0x33,0x00	; 0xA4
.db 0x6E,0x3B,0x00,0x37,0x3F,0x3B,0x33,0x00	; 0xA5
.db 0x3C,0x36,0x36,0x7C,0x00,0x7E,0x00,0x00	; 0xA6
.db 0x3C,0x66,0x66,0x3C,0x00,0x7E,0x00,0x00	; 0xA7
.db 0x0C,0x00,0x0C,0x06,0x03,0x33,0x1E,0x00	; 0xA8
.db 0x3C,0x5A,0xA5,0x9D,0x95,0x66,0x3C,0x00	; 0xA9
.db 0x00,0x00,0x00,0x3F,0x30,0x30,0x00,0x00	; 0xAA
.db 0x67,0x36,0x1E,0x7E,0xC6,0x73,0x19,0xF8	; 0xAB
.db 0x67,0x36,0x1E,0xCE,0xE6,0xB3,0xF9,0xC0	; 0xAC
.db 0x00,0x18,0x00,0x18,0x18,0x3C,0x3C,0x18	; 0xAD
.db 0x00,0xCC,0x66,0x33,0x66,0xCC,0x00,0x00	; 0xAE
.db 0x00,0x33,0x66,0xCC,0x66,0x33,0x00,0x00	; 0xAF
.db 0x44,0x11,0x44,0x11,0x44,0x11,0x44,0x11	; 0xB0
.db 0xAA,0x55,0xAA,0x55,0xAA,0x55,0xAA,0x55	; 0xB1
.db 0xBB,0xEE,0xBB,0xEE,0xBB,0xEE,0xBB,0xEE	; 0xB2
.db 0x18,0x18,0x18,0x18,0x18,0x18,0x18,0x18	; 0xB3
.db 0x18,0x18,0x18,0x18,0x1F,0x18,0x18,0x18	; 0xB4
.db 0x30,0x18,0x0C,0x1E,0x33,0x3F,0x33,0x00	; 0xB5
.db 0x1E,0x21,0x0C,0x1E,0x33,0x3F,0x33,0x00	; 0xB6
.db 0x03,0x06,0x0C,0x1E,0x33,0x3F,0x33,0x00	; 0xB7
.db 0x3C,0x42,0x9D,0x85,0x9D,0x42,0x3C,0x00	; 0xB8
.db 0x6C,0x6C,0x6F,0x60,0x6F,0x6C,0x6C,0x6C	; 0xB9
.db 0x6C,0x6C,0x6C,0x6C,0x6C,0x6C,0x6C,0x6C	; 0xBA
.db 0x00,0x00,0x7F,0x60,0x6F,0x6C,0x6C,0x6C	; 0xBB
.db 0x6C,0x6C,0x6F,0x60,0x7F,0x00,0x00,0x00	; 0xBC
.db 0x18,0x18,0x7E,0x03,0x03,0x7E,0x18,0x18	; 0xBD
.db 0x33,0x33,0x1E,0x3F,0x0C,0x3F,0x0C,0x0C	; 0xBE
.db 0x00,0x00,0x00,0x00,0x1F,0x18,0x18,0x18	; 0xBF
.db 0x18,0x18,0x18,0x18,0xF8,0x00,0x00,0x00	; 0xC0
.db 0x18,0x18,0x18,0x18,0xFF,0x00,0x00,0x00	; 0xC1
.db 0x00,0x00,0x00,0x00,0xFF,0x18,0x18,0x18	; 0xC2
.db 0x18,0x18,0x18,0x18,0xF8,0x18,0x18,0x18	; 0xC3
.db 0x00,0x00,0x00,0x00,0xFF,0x00,0x00,0x00	; 0xC4
.db 0x18,0x18,0x18,0x18,0xFF,0x18,0x18,0x18	; 0xC5
.db 0x6E,0x3B,0x1E,0x30,0x3E,0x33,0x7E,0x00	; 0xC6
.db 0x6E,0x3B,0x0C,0x1E,0x33,0x3F,0x33,0x00	; 0xC7
.db 0x6C,0x6C,0xEC,0x0C,0xFC,0x00,0x00,0x00	; 0xC8
.db 0x00,0x00,0xFC,0x0C,0xEC,0x6C,0x6C,0x6C	; 0xC9
.db 0x6C,0x6C,0xEF,0x00,0xFF,0x00,0x00,0x00	; 0xCA
.db 0x00,0x00,0xFF,0x00,0xEF,0x6C,0x6C,0x6C	; 0xCB
.db 0x6C,0x6C,0xEC,0x0C,0xEC,0x6C,0x6C,0x6C	; 0xCC
.db 0x00,0x00,0xFF,0x00,0xFF,0x00,0x00,0x00	; 0xCD
.db 0x6C,0x6C,0xEF,0x00,0xEF,0x6C,0x6C,0x6C	; 0xCE
.db 0x00,0x41,0x7F,0x36,0x36,0x7F,0x41,0x00	; 0xCF
.db 0x1B,0x0E,0x1B,0x30,0x3C,0x36,0x1C,0x00	; 0xD0
.db 0x3F,0x36,0x66,0x6F,0x66,0x36,0x3F,0x00	; 0xD1
.db 0x1E,0x21,0x3F,0x06,0x1E,0x06,0x3F,0x00	; 0xD2
.db 0x33,0x00,0x3F,0x06,0x1E,0x06,0x3F,0x00	; 0xD3
.db 0x06,0x0C,0x3F,0x06,0x1E,0x06,0x3F,0x00	; 0xD4
.db 0x00,0x03,0x02,0x07,0x00,0x00,0x00,0x00	; 0xD5
.db 0x18,0x0C,0x1E,0x0C,0x0C,0x0C,0x1E,0x00	; 0xD6
.db 0x1E,0x21,0x1E,0x0C,0x0C,0x0C,0x1E,0x00	; 0xD7
.db 0x33,0x00,0x1E,0x0C,0x0C,0x0C,0x1E,0x00	; 0xD8
.db 0x18,0x18,0x18,0x18,0x1F,0x00,0x00,0x00	; 0xD9
.db 0x00,0x00,0x00,0x00,0xF8,0x18,0x18,0x18	; 0xDA
.db 0xFF,0xFF,0xFF,0xFF,0xFF,0xFF,0xFF,0xFF	; 0xDB
.db 0x00,0x00,0x00,0x00,0xFF,0xFF,0xFF,0xFF	; 0xDC
.db 0x18,0x18,0x18,0x00,0x18,0x18,0x18,0x00	; 0xDD
.db 0x06,0x0C,0x1E,0x0C,0x0C,0x0C,0x1E,0x00	; 0xDE
.db 0xFF,0xFF,0xFF,0xFF,0x00,0x00,0x00,0x00	; 0xDF
.db 0x18,0x0C,0x3E,0x63,0x63,0x63,0x3E,0x00	; 0xE0
.db 0x00,0x1E,0x33,0x1F,0x33,0x1F,0x03,0x03	; 0xE1
.db 0x3E,0x41,0x3E,0x63,0x63,0x63,0x3E,0x00	; 0xE2
.db 0x0C,0x18,0x3E,0x63,0x63,0x63,0x3E,0x00	; 0xE3
.db 0x6E,0x3B,0x00,0x1E,0x33,0x33,0x1E,0x00	; 0xE4
.db 0x6E,0x3B,0x3E,0x63,0x63,0x63,0x3E,0x00	; 0xE5
.db 0x00,0x66,0x66,0x66,0x66,0x3E,0x06,0x03	; 0xE6
.db 0x00,0x07,0x1E,0x36,0x1E,0x06,0x0F,0x00	; 0xE7
.db 0x0F,0x06,0x3E,0x66,0x3E,0x06,0x0F,0x00	; 0xE8
.db 0x18,0x0C,0x33,0x33,0x33,0x33,0x1E,0x00	; 0xE9
.db 0x1E,0x21,0x00,0x33,0x33,0x33,0x1E,0x00	; 0xEA
.db 0x06,0x0C,0x33,0x33,0x33,0x33,0x1E,0x00	; 0xEB
.db 0x18,0x0C,0x00,0x33,0x33,0x3F,0x30,0x1F	; 0xEC
.db 0x18,0x0C,0x33,0x33,0x1E,0x0C,0x1E,0x00	; 0xED
.db 0x00,0x3F,0x00,0x00,0x00,0x00,0x00,0x00	; 0xEE
.db 0x18,0x0C,0x00,0x00,0x00,0x00,0x00,0x00	; 0xEF
.db 0x00,0x00,0x00,0x00,0x3F,0x00,0x00,0x00	; 0xF0
.db 0x0C,0x0C,0x3F,0x0C,0x0C,0x00,0x3F,0x00	; 0xF1
.db 0x00,0x00,0x00,0x3F,0x00,0x3F,0x00,0x00	; 0xF2
.db 0x67,0x34,0x1E,0xCC,0xE7,0xB3,0xF9,0xC0	; 0xF3
.db 0xFE,0xDB,0xDB,0xDE,0xD8,0xD8,0xD8,0x00	; 0xF4
.db 0x7E,0xC3,0x1E,0x33,0x33,0x1E,0x31,0x1F	; 0xF5
.db 0x0C,0x0C,0x00,0x3F,0x00,0x0C,0x0C,0x00	; 0xF6
.db 0x00,0x00,0x00,0x00,0x00,0x00,0x0C,0x06	; 0xF7
.db 0x1C,0x36,0x36,0x1C,0x00,0x00,0x00,0x00	; 0xF8
.db 0x00,0x33,0x00,0x00,0x00,0x00,0x00,0x00	; 0xF9
.db 0x00,0x00,0x00,0x00,0x18,0x00,0x00,0x00	; 0xFA
.db 0x1C,0x1E,0x18,0x18,0x7E,0x00,0x00,0x00	; 0xFB
.db 0x3E,0x70,0x3C,0x70,0x3E,0x00,0x00,0x00	; 0xFC
.db 0x1E,0x30,0x1C,0x06,0x3E,0x00,0x00,0x00	; 0xFD
.db 0x00,0x00,0x3C,0x3C,0x3C,0x3C,0x00,0x00	; 0xFE
.db 0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00	; 0xFF 


PALETTE:
.db 0x40,0x40
.db 0x00,0x00,0x00,0x00,0x01,0x01,0x01,0x01,0x02,0x02,0x02,0x02,0x03,0x03,0x03,0x03,0x04,0x04,0x04,0x04,0x05,0x05,0x05,0x05,0x06,0x06,0x06,0x06,0x07,0x07,0x07,0x07,0x08,0x08,0x08,0x08,0x09,0x09,0x09,0x09,0x0A,0x0A,0x0A,0x0A,0x0B,0x0B,0x0B,0x0B,0x0C,0x0C,0x0C,0x0C,0x0D,0x0D,0x0D,0x0D,0x0E,0x0E,0x0E,0x0E,0x0F,0x0F,0x0F,0x0F
.db 0x00,0x00,0x00,0x00,0x01,0x01,0x01,0x01,0x02,0x02,0x02,0x02,0x03,0x03,0x03,0x03,0x04,0x04,0x04,0x04,0x05,0x05,0x05,0x05,0x06,0x06,0x06,0x06,0x07,0x07,0x07,0x07,0x08,0x08,0x08,0x08,0x09,0x09,0x09,0x09,0x0A,0x0A,0x0A,0x0A,0x0B,0x0B,0x0B,0x0B,0x0C,0x0C,0x0C,0x0C,0x0D,0x0D,0x0D,0x0D,0x0E,0x0E,0x0E,0x0E,0x0F,0x0F,0x0F,0x0F
.db 0x00,0x00,0x00,0x00,0x01,0x01,0x01,0x01,0x02,0x02,0x02,0x02,0x03,0x03,0x03,0x03,0x04,0x04,0x04,0x04,0x05,0x05,0x05,0x05,0x06,0x06,0x06,0x06,0x07,0x07,0x07,0x07,0x08,0x08,0x08,0x08,0x09,0x09,0x09,0x09,0x0A,0x0A,0x0A,0x0A,0x0B,0x0B,0x0B,0x0B,0x0C,0x0C,0x0C,0x0C,0x0D,0x0D,0x0D,0x0D,0x0E,0x0E,0x0E,0x0E,0x0F,0x0F,0x0F,0x0F
.db 0x00,0x00,0x00,0x00,0x01,0x01,0x01,0x01,0x02,0x02,0x02,0x02,0x03,0x03,0x03,0x03,0x04,0x04,0x04,0x04,0x05,0x05,0x05,0x05,0x06,0x06,0x06,0x06,0x07,0x07,0x07,0x07,0x08,0x08,0x08,0x08,0x09,0x09,0x09,0x09,0x0A,0x0A,0x0A,0x0A,0x0B,0x0B,0x0B,0x0B,0x0C,0x0C,0x0C,0x0C,0x0D,0x0D,0x0D,0x0D,0x0E,0x0E,0x0E,0x0E,0x0F,0x0F,0x0F,0x0F
.db 0x10,0x10,0x10,0x10,0x11,0x11,0x11,0x11,0x12,0x12,0x12,0x12,0x13,0x13,0x13,0x13,0x14,0x14,0x14,0x14,0x15,0x15,0x15,0x15,0x16,0x16,0x16,0x16,0x17,0x17,0x17,0x17,0x18,0x18,0x18,0x18,0x19,0x19,0x19,0x19,0x1A,0x1A,0x1A,0x1A,0x1B,0x1B,0x1B,0x1B,0x1C,0x1C,0x1C,0x1C,0x1D,0x1D,0x1D,0x1D,0x1E,0x1E,0x1E,0x1E,0x1F,0x1F,0x1F,0x1F
.db 0x10,0x10,0x10,0x10,0x11,0x11,0x11,0x11,0x12,0x12,0x12,0x12,0x13,0x13,0x13,0x13,0x14,0x14,0x14,0x14,0x15,0x15,0x15,0x15,0x16,0x16,0x16,0x16,0x17,0x17,0x17,0x17,0x18,0x18,0x18,0x18,0x19,0x19,0x19,0x19,0x1A,0x1A,0x1A,0x1A,0x1B,0x1B,0x1B,0x1B,0x1C,0x1C,0x1C,0x1C,0x1D,0x1D,0x1D,0x1D,0x1E,0x1E,0x1E,0x1E,0x1F,0x1F,0x1F,0x1F
.db 0x10,0x10,0x10,0x10,0x11,0x11,0x11,0x11,0x12,0x12,0x12,0x12,0x13,0x13,0x13,0x13,0x14,0x14,0x14,0x14,0x15,0x15,0x15,0x15,0x16,0x16,0x16,0x16,0x17,0x17,0x17,0x17,0x18,0x18,0x18,0x18,0x19,0x19,0x19,0x19,0x1A,0x1A,0x1A,0x1A,0x1B,0x1B,0x1B,0x1B,0x1C,0x1C,0x1C,0x1C,0x1D,0x1D,0x1D,0x1D,0x1E,0x1E,0x1E,0x1E,0x1F,0x1F,0x1F,0x1F
.db 0x10,0x10,0x10,0x10,0x11,0x11,0x11,0x11,0x12,0x12,0x12,0x12,0x13,0x13,0x13,0x13,0x14,0x14,0x14,0x14,0x15,0x15,0x15,0x15,0x16,0x16,0x16,0x16,0x17,0x17,0x17,0x17,0x18,0x18,0x18,0x18,0x19,0x19,0x19,0x19,0x1A,0x1A,0x1A,0x1A,0x1B,0x1B,0x1B,0x1B,0x1C,0x1C,0x1C,0x1C,0x1D,0x1D,0x1D,0x1D,0x1E,0x1E,0x1E,0x1E,0x1F,0x1F,0x1F,0x1F
.db 0x20,0x20,0x20,0x20,0x21,0x21,0x21,0x21,0x22,0x22,0x22,0x22,0x23,0x23,0x23,0x23,0x24,0x24,0x24,0x24,0x25,0x25,0x25,0x25,0x26,0x26,0x26,0x26,0x27,0x27,0x27,0x27,0x28,0x28,0x28,0x28,0x29,0x29,0x29,0x29,0x2A,0x2A,0x2A,0x2A,0x2B,0x2B,0x2B,0x2B,0x2C,0x2C,0x2C,0x2C,0x2D,0x2D,0x2D,0x2D,0x2E,0x2E,0x2E,0x2E,0x2F,0x2F,0x2F,0x2F
.db 0x20,0x20,0x20,0x20,0x21,0x21,0x21,0x21,0x22,0x22,0x22,0x22,0x23,0x23,0x23,0x23,0x24,0x24,0x24,0x24,0x25,0x25,0x25,0x25,0x26,0x26,0x26,0x26,0x27,0x27,0x27,0x27,0x28,0x28,0x28,0x28,0x29,0x29,0x29,0x29,0x2A,0x2A,0x2A,0x2A,0x2B,0x2B,0x2B,0x2B,0x2C,0x2C,0x2C,0x2C,0x2D,0x2D,0x2D,0x2D,0x2E,0x2E,0x2E,0x2E,0x2F,0x2F,0x2F,0x2F
.db 0x20,0x20,0x20,0x20,0x21,0x21,0x21,0x21,0x22,0x22,0x22,0x22,0x23,0x23,0x23,0x23,0x24,0x24,0x24,0x24,0x25,0x25,0x25,0x25,0x26,0x26,0x26,0x26,0x27,0x27,0x27,0x27,0x28,0x28,0x28,0x28,0x29,0x29,0x29,0x29,0x2A,0x2A,0x2A,0x2A,0x2B,0x2B,0x2B,0x2B,0x2C,0x2C,0x2C,0x2C,0x2D,0x2D,0x2D,0x2D,0x2E,0x2E,0x2E,0x2E,0x2F,0x2F,0x2F,0x2F
.db 0x20,0x20,0x20,0x20,0x21,0x21,0x21,0x21,0x22,0x22,0x22,0x22,0x23,0x23,0x23,0x23,0x24,0x24,0x24,0x24,0x25,0x25,0x25,0x25,0x26,0x26,0x26,0x26,0x27,0x27,0x27,0x27,0x28,0x28,0x28,0x28,0x29,0x29,0x29,0x29,0x2A,0x2A,0x2A,0x2A,0x2B,0x2B,0x2B,0x2B,0x2C,0x2C,0x2C,0x2C,0x2D,0x2D,0x2D,0x2D,0x2E,0x2E,0x2E,0x2E,0x2F,0x2F,0x2F,0x2F
.db 0x30,0x30,0x30,0x30,0x31,0x31,0x31,0x31,0x32,0x32,0x32,0x32,0x33,0x33,0x33,0x33,0x34,0x34,0x34,0x34,0x35,0x35,0x35,0x35,0x36,0x36,0x36,0x36,0x37,0x37,0x37,0x37,0x38,0x38,0x38,0x38,0x39,0x39,0x39,0x39,0x3A,0x3A,0x3A,0x3A,0x3B,0x3B,0x3B,0x3B,0x3C,0x3C,0x3C,0x3C,0x3D,0x3D,0x3D,0x3D,0x3E,0x3E,0x3E,0x3E,0x3F,0x3F,0x3F,0x3F
.db 0x30,0x30,0x30,0x30,0x31,0x31,0x31,0x31,0x32,0x32,0x32,0x32,0x33,0x33,0x33,0x33,0x34,0x34,0x34,0x34,0x35,0x35,0x35,0x35,0x36,0x36,0x36,0x36,0x37,0x37,0x37,0x37,0x38,0x38,0x38,0x38,0x39,0x39,0x39,0x39,0x3A,0x3A,0x3A,0x3A,0x3B,0x3B,0x3B,0x3B,0x3C,0x3C,0x3C,0x3C,0x3D,0x3D,0x3D,0x3D,0x3E,0x3E,0x3E,0x3E,0x3F,0x3F,0x3F,0x3F
.db 0x30,0x30,0x30,0x30,0x31,0x31,0x31,0x31,0x32,0x32,0x32,0x32,0x33,0x33,0x33,0x33,0x34,0x34,0x34,0x34,0x35,0x35,0x35,0x35,0x36,0x36,0x36,0x36,0x37,0x37,0x37,0x37,0x38,0x38,0x38,0x38,0x39,0x39,0x39,0x39,0x3A,0x3A,0x3A,0x3A,0x3B,0x3B,0x3B,0x3B,0x3C,0x3C,0x3C,0x3C,0x3D,0x3D,0x3D,0x3D,0x3E,0x3E,0x3E,0x3E,0x3F,0x3F,0x3F,0x3F
.db 0x30,0x30,0x30,0x30,0x31,0x31,0x31,0x31,0x32,0x32,0x32,0x32,0x33,0x33,0x33,0x33,0x34,0x34,0x34,0x34,0x35,0x35,0x35,0x35,0x36,0x36,0x36,0x36,0x37,0x37,0x37,0x37,0x38,0x38,0x38,0x38,0x39,0x39,0x39,0x39,0x3A,0x3A,0x3A,0x3A,0x3B,0x3B,0x3B,0x3B,0x3C,0x3C,0x3C,0x3C,0x3D,0x3D,0x3D,0x3D,0x3E,0x3E,0x3E,0x3E,0x3F,0x3F,0x3F,0x3F
.db 0x40,0x40,0x40,0x40,0x41,0x41,0x41,0x41,0x42,0x42,0x42,0x42,0x43,0x43,0x43,0x43,0x44,0x44,0x44,0x44,0x45,0x45,0x45,0x45,0x46,0x46,0x46,0x46,0x47,0x47,0x47,0x47,0x48,0x48,0x48,0x48,0x49,0x49,0x49,0x49,0x4A,0x4A,0x4A,0x4A,0x4B,0x4B,0x4B,0x4B,0x4C,0x4C,0x4C,0x4C,0x4D,0x4D,0x4D,0x4D,0x4E,0x4E,0x4E,0x4E,0x4F,0x4F,0x4F,0x4F
.db 0x40,0x40,0x40,0x40,0x41,0x41,0x41,0x41,0x42,0x42,0x42,0x42,0x43,0x43,0x43,0x43,0x44,0x44,0x44,0x44,0x45,0x45,0x45,0x45,0x46,0x46,0x46,0x46,0x47,0x47,0x47,0x47,0x48,0x48,0x48,0x48,0x49,0x49,0x49,0x49,0x4A,0x4A,0x4A,0x4A,0x4B,0x4B,0x4B,0x4B,0x4C,0x4C,0x4C,0x4C,0x4D,0x4D,0x4D,0x4D,0x4E,0x4E,0x4E,0x4E,0x4F,0x4F,0x4F,0x4F
.db 0x40,0x40,0x40,0x40,0x41,0x41,0x41,0x41,0x42,0x42,0x42,0x42,0x43,0x43,0x43,0x43,0x44,0x44,0x44,0x44,0x45,0x45,0x45,0x45,0x46,0x46,0x46,0x46,0x47,0x47,0x47,0x47,0x48,0x48,0x48,0x48,0x49,0x49,0x49,0x49,0x4A,0x4A,0x4A,0x4A,0x4B,0x4B,0x4B,0x4B,0x4C,0x4C,0x4C,0x4C,0x4D,0x4D,0x4D,0x4D,0x4E,0x4E,0x4E,0x4E,0x4F,0x4F,0x4F,0x4F
.db 0x40,0x40,0x40,0x40,0x41,0x41,0x41,0x41,0x42,0x42,0x42,0x42,0x43,0x43,0x43,0x43,0x44,0x44,0x44,0x44,0x45,0x45,0x45,0x45,0x46,0x46,0x46,0x46,0x47,0x47,0x47,0x47,0x48,0x48,0x48,0x48,0x49,0x49,0x49,0x49,0x4A,0x4A,0x4A,0x4A,0x4B,0x4B,0x4B,0x4B,0x4C,0x4C,0x4C,0x4C,0x4D,0x4D,0x4D,0x4D,0x4E,0x4E,0x4E,0x4E,0x4F,0x4F,0x4F,0x4F
.db 0x50,0x50,0x50,0x50,0x51,0x51,0x51,0x51,0x52,0x52,0x52,0x52,0x53,0x53,0x53,0x53,0x54,0x54,0x54,0x54,0x55,0x55,0x55,0x55,0x56,0x56,0x56,0x56,0x57,0x57,0x57,0x57,0x58,0x58,0x58,0x58,0x59,0x59,0x59,0x59,0x5A,0x5A,0x5A,0x5A,0x5B,0x5B,0x5B,0x5B,0x5C,0x5C,0x5C,0x5C,0x5D,0x5D,0x5D,0x5D,0x5E,0x5E,0x5E,0x5E,0x5F,0x5F,0x5F,0x5F
.db 0x50,0x50,0x50,0x50,0x51,0x51,0x51,0x51,0x52,0x52,0x52,0x52,0x53,0x53,0x53,0x53,0x54,0x54,0x54,0x54,0x55,0x55,0x55,0x55,0x56,0x56,0x56,0x56,0x57,0x57,0x57,0x57,0x58,0x58,0x58,0x58,0x59,0x59,0x59,0x59,0x5A,0x5A,0x5A,0x5A,0x5B,0x5B,0x5B,0x5B,0x5C,0x5C,0x5C,0x5C,0x5D,0x5D,0x5D,0x5D,0x5E,0x5E,0x5E,0x5E,0x5F,0x5F,0x5F,0x5F
.db 0x50,0x50,0x50,0x50,0x51,0x51,0x51,0x51,0x52,0x52,0x52,0x52,0x53,0x53,0x53,0x53,0x54,0x54,0x54,0x54,0x55,0x55,0x55,0x55,0x56,0x56,0x56,0x56,0x57,0x57,0x57,0x57,0x58,0x58,0x58,0x58,0x59,0x59,0x59,0x59,0x5A,0x5A,0x5A,0x5A,0x5B,0x5B,0x5B,0x5B,0x5C,0x5C,0x5C,0x5C,0x5D,0x5D,0x5D,0x5D,0x5E,0x5E,0x5E,0x5E,0x5F,0x5F,0x5F,0x5F
.db 0x50,0x50,0x50,0x50,0x51,0x51,0x51,0x51,0x52,0x52,0x52,0x52,0x53,0x53,0x53,0x53,0x54,0x54,0x54,0x54,0x55,0x55,0x55,0x55,0x56,0x56,0x56,0x56,0x57,0x57,0x57,0x57,0x58,0x58,0x58,0x58,0x59,0x59,0x59,0x59,0x5A,0x5A,0x5A,0x5A,0x5B,0x5B,0x5B,0x5B,0x5C,0x5C,0x5C,0x5C,0x5D,0x5D,0x5D,0x5D,0x5E,0x5E,0x5E,0x5E,0x5F,0x5F,0x5F,0x5F
.db 0x60,0x60,0x60,0x60,0x61,0x61,0x61,0x61,0x62,0x62,0x62,0x62,0x63,0x63,0x63,0x63,0x64,0x64,0x64,0x64,0x65,0x65,0x65,0x65,0x66,0x66,0x66,0x66,0x67,0x67,0x67,0x67,0x68,0x68,0x68,0x68,0x69,0x69,0x69,0x69,0x6A,0x6A,0x6A,0x6A,0x6B,0x6B,0x6B,0x6B,0x6C,0x6C,0x6C,0x6C,0x6D,0x6D,0x6D,0x6D,0x6E,0x6E,0x6E,0x6E,0x6F,0x6F,0x6F,0x6F
.db 0x60,0x60,0x60,0x60,0x61,0x61,0x61,0x61,0x62,0x62,0x62,0x62,0x63,0x63,0x63,0x63,0x64,0x64,0x64,0x64,0x65,0x65,0x65,0x65,0x66,0x66,0x66,0x66,0x67,0x67,0x67,0x67,0x68,0x68,0x68,0x68,0x69,0x69,0x69,0x69,0x6A,0x6A,0x6A,0x6A,0x6B,0x6B,0x6B,0x6B,0x6C,0x6C,0x6C,0x6C,0x6D,0x6D,0x6D,0x6D,0x6E,0x6E,0x6E,0x6E,0x6F,0x6F,0x6F,0x6F
.db 0x60,0x60,0x60,0x60,0x61,0x61,0x61,0x61,0x62,0x62,0x62,0x62,0x63,0x63,0x63,0x63,0x64,0x64,0x64,0x64,0x65,0x65,0x65,0x65,0x66,0x66,0x66,0x66,0x67,0x67,0x67,0x67,0x68,0x68,0x68,0x68,0x69,0x69,0x69,0x69,0x6A,0x6A,0x6A,0x6A,0x6B,0x6B,0x6B,0x6B,0x6C,0x6C,0x6C,0x6C,0x6D,0x6D,0x6D,0x6D,0x6E,0x6E,0x6E,0x6E,0x6F,0x6F,0x6F,0x6F
.db 0x60,0x60,0x60,0x60,0x61,0x61,0x61,0x61,0x62,0x62,0x62,0x62,0x63,0x63,0x63,0x63,0x64,0x64,0x64,0x64,0x65,0x65,0x65,0x65,0x66,0x66,0x66,0x66,0x67,0x67,0x67,0x67,0x68,0x68,0x68,0x68,0x69,0x69,0x69,0x69,0x6A,0x6A,0x6A,0x6A,0x6B,0x6B,0x6B,0x6B,0x6C,0x6C,0x6C,0x6C,0x6D,0x6D,0x6D,0x6D,0x6E,0x6E,0x6E,0x6E,0x6F,0x6F,0x6F,0x6F
.db 0x70,0x70,0x70,0x70,0x71,0x71,0x71,0x71,0x72,0x72,0x72,0x72,0x73,0x73,0x73,0x73,0x74,0x74,0x74,0x74,0x75,0x75,0x75,0x75,0x76,0x76,0x76,0x76,0x77,0x77,0x77,0x77,0x78,0x78,0x78,0x78,0x79,0x79,0x79,0x79,0x7A,0x7A,0x7A,0x7A,0x7B,0x7B,0x7B,0x7B,0x7C,0x7C,0x7C,0x7C,0x7D,0x7D,0x7D,0x7D,0x7E,0x7E,0x7E,0x7E,0x7F,0x7F,0x7F,0x7F
.db 0x70,0x70,0x70,0x70,0x71,0x71,0x71,0x71,0x72,0x72,0x72,0x72,0x73,0x73,0x73,0x73,0x74,0x74,0x74,0x74,0x75,0x75,0x75,0x75,0x76,0x76,0x76,0x76,0x77,0x77,0x77,0x77,0x78,0x78,0x78,0x78,0x79,0x79,0x79,0x79,0x7A,0x7A,0x7A,0x7A,0x7B,0x7B,0x7B,0x7B,0x7C,0x7C,0x7C,0x7C,0x7D,0x7D,0x7D,0x7D,0x7E,0x7E,0x7E,0x7E,0x7F,0x7F,0x7F,0x7F
.db 0x70,0x70,0x70,0x70,0x71,0x71,0x71,0x71,0x72,0x72,0x72,0x72,0x73,0x73,0x73,0x73,0x74,0x74,0x74,0x74,0x75,0x75,0x75,0x75,0x76,0x76,0x76,0x76,0x77,0x77,0x77,0x77,0x78,0x78,0x78,0x78,0x79,0x79,0x79,0x79,0x7A,0x7A,0x7A,0x7A,0x7B,0x7B,0x7B,0x7B,0x7C,0x7C,0x7C,0x7C,0x7D,0x7D,0x7D,0x7D,0x7E,0x7E,0x7E,0x7E,0x7F,0x7F,0x7F,0x7F
.db 0x70,0x70,0x70,0x70,0x71,0x71,0x71,0x71,0x72,0x72,0x72,0x72,0x73,0x73,0x73,0x73,0x74,0x74,0x74,0x74,0x75,0x75,0x75,0x75,0x76,0x76,0x76,0x76,0x77,0x77,0x77,0x77,0x78,0x78,0x78,0x78,0x79,0x79,0x79,0x79,0x7A,0x7A,0x7A,0x7A,0x7B,0x7B,0x7B,0x7B,0x7C,0x7C,0x7C,0x7C,0x7D,0x7D,0x7D,0x7D,0x7E,0x7E,0x7E,0x7E,0x7F,0x7F,0x7F,0x7F
.db 0x80,0x80,0x80,0x80,0x81,0x81,0x81,0x81,0x82,0x82,0x82,0x82,0x83,0x83,0x83,0x83,0x84,0x84,0x84,0x84,0x85,0x85,0x85,0x85,0x86,0x86,0x86,0x86,0x87,0x87,0x87,0x87,0x88,0x88,0x88,0x88,0x89,0x89,0x89,0x89,0x8A,0x8A,0x8A,0x8A,0x8B,0x8B,0x8B,0x8B,0x8C,0x8C,0x8C,0x8C,0x8D,0x8D,0x8D,0x8D,0x8E,0x8E,0x8E,0x8E,0x8F,0x8F,0x8F,0x8F
.db 0x80,0x80,0x80,0x80,0x81,0x81,0x81,0x81,0x82,0x82,0x82,0x82,0x83,0x83,0x83,0x83,0x84,0x84,0x84,0x84,0x85,0x85,0x85,0x85,0x86,0x86,0x86,0x86,0x87,0x87,0x87,0x87,0x88,0x88,0x88,0x88,0x89,0x89,0x89,0x89,0x8A,0x8A,0x8A,0x8A,0x8B,0x8B,0x8B,0x8B,0x8C,0x8C,0x8C,0x8C,0x8D,0x8D,0x8D,0x8D,0x8E,0x8E,0x8E,0x8E,0x8F,0x8F,0x8F,0x8F
.db 0x80,0x80,0x80,0x80,0x81,0x81,0x81,0x81,0x82,0x82,0x82,0x82,0x83,0x83,0x83,0x83,0x84,0x84,0x84,0x84,0x85,0x85,0x85,0x85,0x86,0x86,0x86,0x86,0x87,0x87,0x87,0x87,0x88,0x88,0x88,0x88,0x89,0x89,0x89,0x89,0x8A,0x8A,0x8A,0x8A,0x8B,0x8B,0x8B,0x8B,0x8C,0x8C,0x8C,0x8C,0x8D,0x8D,0x8D,0x8D,0x8E,0x8E,0x8E,0x8E,0x8F,0x8F,0x8F,0x8F
.db 0x80,0x80,0x80,0x80,0x81,0x81,0x81,0x81,0x82,0x82,0x82,0x82,0x83,0x83,0x83,0x83,0x84,0x84,0x84,0x84,0x85,0x85,0x85,0x85,0x86,0x86,0x86,0x86,0x87,0x87,0x87,0x87,0x88,0x88,0x88,0x88,0x89,0x89,0x89,0x89,0x8A,0x8A,0x8A,0x8A,0x8B,0x8B,0x8B,0x8B,0x8C,0x8C,0x8C,0x8C,0x8D,0x8D,0x8D,0x8D,0x8E,0x8E,0x8E,0x8E,0x8F,0x8F,0x8F,0x8F
.db 0x90,0x90,0x90,0x90,0x91,0x91,0x91,0x91,0x92,0x92,0x92,0x92,0x93,0x93,0x93,0x93,0x94,0x94,0x94,0x94,0x95,0x95,0x95,0x95,0x96,0x96,0x96,0x96,0x97,0x97,0x97,0x97,0x98,0x98,0x98,0x98,0x99,0x99,0x99,0x99,0x9A,0x9A,0x9A,0x9A,0x9B,0x9B,0x9B,0x9B,0x9C,0x9C,0x9C,0x9C,0x9D,0x9D,0x9D,0x9D,0x9E,0x9E,0x9E,0x9E,0x9F,0x9F,0x9F,0x9F
.db 0x90,0x90,0x90,0x90,0x91,0x91,0x91,0x91,0x92,0x92,0x92,0x92,0x93,0x93,0x93,0x93,0x94,0x94,0x94,0x94,0x95,0x95,0x95,0x95,0x96,0x96,0x96,0x96,0x97,0x97,0x97,0x97,0x98,0x98,0x98,0x98,0x99,0x99,0x99,0x99,0x9A,0x9A,0x9A,0x9A,0x9B,0x9B,0x9B,0x9B,0x9C,0x9C,0x9C,0x9C,0x9D,0x9D,0x9D,0x9D,0x9E,0x9E,0x9E,0x9E,0x9F,0x9F,0x9F,0x9F
.db 0x90,0x90,0x90,0x90,0x91,0x91,0x91,0x91,0x92,0x92,0x92,0x92,0x93,0x93,0x93,0x93,0x94,0x94,0x94,0x94,0x95,0x95,0x95,0x95,0x96,0x96,0x96,0x96,0x97,0x97,0x97,0x97,0x98,0x98,0x98,0x98,0x99,0x99,0x99,0x99,0x9A,0x9A,0x9A,0x9A,0x9B,0x9B,0x9B,0x9B,0x9C,0x9C,0x9C,0x9C,0x9D,0x9D,0x9D,0x9D,0x9E,0x9E,0x9E,0x9E,0x9F,0x9F,0x9F,0x9F
.db 0x90,0x90,0x90,0x90,0x91,0x91,0x91,0x91,0x92,0x92,0x92,0x92,0x93,0x93,0x93,0x93,0x94,0x94,0x94,0x94,0x95,0x95,0x95,0x95,0x96,0x96,0x96,0x96,0x97,0x97,0x97,0x97,0x98,0x98,0x98,0x98,0x99,0x99,0x99,0x99,0x9A,0x9A,0x9A,0x9A,0x9B,0x9B,0x9B,0x9B,0x9C,0x9C,0x9C,0x9C,0x9D,0x9D,0x9D,0x9D,0x9E,0x9E,0x9E,0x9E,0x9F,0x9F,0x9F,0x9F
.db 0xA0,0xA0,0xA0,0xA0,0xA1,0xA1,0xA1,0xA1,0xA2,0xA2,0xA2,0xA2,0xA3,0xA3,0xA3,0xA3,0xA4,0xA4,0xA4,0xA4,0xA5,0xA5,0xA5,0xA5,0xA6,0xA6,0xA6,0xA6,0xA7,0xA7,0xA7,0xA7,0xA8,0xA8,0xA8,0xA8,0xA9,0xA9,0xA9,0xA9,0xAA,0xAA,0xAA,0xAA,0xAB,0xAB,0xAB,0xAB,0xAC,0xAC,0xAC,0xAC,0xAD,0xAD,0xAD,0xAD,0xAE,0xAE,0xAE,0xAE,0xAF,0xAF,0xAF,0xAF
.db 0xA0,0xA0,0xA0,0xA0,0xA1,0xA1,0xA1,0xA1,0xA2,0xA2,0xA2,0xA2,0xA3,0xA3,0xA3,0xA3,0xA4,0xA4,0xA4,0xA4,0xA5,0xA5,0xA5,0xA5,0xA6,0xA6,0xA6,0xA6,0xA7,0xA7,0xA7,0xA7,0xA8,0xA8,0xA8,0xA8,0xA9,0xA9,0xA9,0xA9,0xAA,0xAA,0xAA,0xAA,0xAB,0xAB,0xAB,0xAB,0xAC,0xAC,0xAC,0xAC,0xAD,0xAD,0xAD,0xAD,0xAE,0xAE,0xAE,0xAE,0xAF,0xAF,0xAF,0xAF
.db 0xA0,0xA0,0xA0,0xA0,0xA1,0xA1,0xA1,0xA1,0xA2,0xA2,0xA2,0xA2,0xA3,0xA3,0xA3,0xA3,0xA4,0xA4,0xA4,0xA4,0xA5,0xA5,0xA5,0xA5,0xA6,0xA6,0xA6,0xA6,0xA7,0xA7,0xA7,0xA7,0xA8,0xA8,0xA8,0xA8,0xA9,0xA9,0xA9,0xA9,0xAA,0xAA,0xAA,0xAA,0xAB,0xAB,0xAB,0xAB,0xAC,0xAC,0xAC,0xAC,0xAD,0xAD,0xAD,0xAD,0xAE,0xAE,0xAE,0xAE,0xAF,0xAF,0xAF,0xAF
.db 0xA0,0xA0,0xA0,0xA0,0xA1,0xA1,0xA1,0xA1,0xA2,0xA2,0xA2,0xA2,0xA3,0xA3,0xA3,0xA3,0xA4,0xA4,0xA4,0xA4,0xA5,0xA5,0xA5,0xA5,0xA6,0xA6,0xA6,0xA6,0xA7,0xA7,0xA7,0xA7,0xA8,0xA8,0xA8,0xA8,0xA9,0xA9,0xA9,0xA9,0xAA,0xAA,0xAA,0xAA,0xAB,0xAB,0xAB,0xAB,0xAC,0xAC,0xAC,0xAC,0xAD,0xAD,0xAD,0xAD,0xAE,0xAE,0xAE,0xAE,0xAF,0xAF,0xAF,0xAF
.db 0xB0,0xB0,0xB0,0xB0,0xB1,0xB1,0xB1,0xB1,0xB2,0xB2,0xB2,0xB2,0xB3,0xB3,0xB3,0xB3,0xB4,0xB4,0xB4,0xB4,0xB5,0xB5,0xB5,0xB5,0xB6,0xB6,0xB6,0xB6,0xB7,0xB7,0xB7,0xB7,0xB8,0xB8,0xB8,0xB8,0xB9,0xB9,0xB9,0xB9,0xBA,0xBA,0xBA,0xBA,0xBB,0xBB,0xBB,0xBB,0xBC,0xBC,0xBC,0xBC,0xBD,0xBD,0xBD,0xBD,0xBE,0xBE,0xBE,0xBE,0xBF,0xBF,0xBF,0xBF
.db 0xB0,0xB0,0xB0,0xB0,0xB1,0xB1,0xB1,0xB1,0xB2,0xB2,0xB2,0xB2,0xB3,0xB3,0xB3,0xB3,0xB4,0xB4,0xB4,0xB4,0xB5,0xB5,0xB5,0xB5,0xB6,0xB6,0xB6,0xB6,0xB7,0xB7,0xB7,0xB7,0xB8,0xB8,0xB8,0xB8,0xB9,0xB9,0xB9,0xB9,0xBA,0xBA,0xBA,0xBA,0xBB,0xBB,0xBB,0xBB,0xBC,0xBC,0xBC,0xBC,0xBD,0xBD,0xBD,0xBD,0xBE,0xBE,0xBE,0xBE,0xBF,0xBF,0xBF,0xBF
.db 0xB0,0xB0,0xB0,0xB0,0xB1,0xB1,0xB1,0xB1,0xB2,0xB2,0xB2,0xB2,0xB3,0xB3,0xB3,0xB3,0xB4,0xB4,0xB4,0xB4,0xB5,0xB5,0xB5,0xB5,0xB6,0xB6,0xB6,0xB6,0xB7,0xB7,0xB7,0xB7,0xB8,0xB8,0xB8,0xB8,0xB9,0xB9,0xB9,0xB9,0xBA,0xBA,0xBA,0xBA,0xBB,0xBB,0xBB,0xBB,0xBC,0xBC,0xBC,0xBC,0xBD,0xBD,0xBD,0xBD,0xBE,0xBE,0xBE,0xBE,0xBF,0xBF,0xBF,0xBF
.db 0xB0,0xB0,0xB0,0xB0,0xB1,0xB1,0xB1,0xB1,0xB2,0xB2,0xB2,0xB2,0xB3,0xB3,0xB3,0xB3,0xB4,0xB4,0xB4,0xB4,0xB5,0xB5,0xB5,0xB5,0xB6,0xB6,0xB6,0xB6,0xB7,0xB7,0xB7,0xB7,0xB8,0xB8,0xB8,0xB8,0xB9,0xB9,0xB9,0xB9,0xBA,0xBA,0xBA,0xBA,0xBB,0xBB,0xBB,0xBB,0xBC,0xBC,0xBC,0xBC,0xBD,0xBD,0xBD,0xBD,0xBE,0xBE,0xBE,0xBE,0xBF,0xBF,0xBF,0xBF
.db 0xC0,0xC0,0xC0,0xC0,0xC1,0xC1,0xC1,0xC1,0xC2,0xC2,0xC2,0xC2,0xC3,0xC3,0xC3,0xC3,0xC4,0xC4,0xC4,0xC4,0xC5,0xC5,0xC5,0xC5,0xC6,0xC6,0xC6,0xC6,0xC7,0xC7,0xC7,0xC7,0xC8,0xC8,0xC8,0xC8,0xC9,0xC9,0xC9,0xC9,0xCA,0xCA,0xCA,0xCA,0xCB,0xCB,0xCB,0xCB,0xCC,0xCC,0xCC,0xCC,0xCD,0xCD,0xCD,0xCD,0xCE,0xCE,0xCE,0xCE,0xCF,0xCF,0xCF,0xCF
.db 0xC0,0xC0,0xC0,0xC0,0xC1,0xC1,0xC1,0xC1,0xC2,0xC2,0xC2,0xC2,0xC3,0xC3,0xC3,0xC3,0xC4,0xC4,0xC4,0xC4,0xC5,0xC5,0xC5,0xC5,0xC6,0xC6,0xC6,0xC6,0xC7,0xC7,0xC7,0xC7,0xC8,0xC8,0xC8,0xC8,0xC9,0xC9,0xC9,0xC9,0xCA,0xCA,0xCA,0xCA,0xCB,0xCB,0xCB,0xCB,0xCC,0xCC,0xCC,0xCC,0xCD,0xCD,0xCD,0xCD,0xCE,0xCE,0xCE,0xCE,0xCF,0xCF,0xCF,0xCF
.db 0xC0,0xC0,0xC0,0xC0,0xC1,0xC1,0xC1,0xC1,0xC2,0xC2,0xC2,0xC2,0xC3,0xC3,0xC3,0xC3,0xC4,0xC4,0xC4,0xC4,0xC5,0xC5,0xC5,0xC5,0xC6,0xC6,0xC6,0xC6,0xC7,0xC7,0xC7,0xC7,0xC8,0xC8,0xC8,0xC8,0xC9,0xC9,0xC9,0xC9,0xCA,0xCA,0xCA,0xCA,0xCB,0xCB,0xCB,0xCB,0xCC,0xCC,0xCC,0xCC,0xCD,0xCD,0xCD,0xCD,0xCE,0xCE,0xCE,0xCE,0xCF,0xCF,0xCF,0xCF
.db 0xC0,0xC0,0xC0,0xC0,0xC1,0xC1,0xC1,0xC1,0xC2,0xC2,0xC2,0xC2,0xC3,0xC3,0xC3,0xC3,0xC4,0xC4,0xC4,0xC4,0xC5,0xC5,0xC5,0xC5,0xC6,0xC6,0xC6,0xC6,0xC7,0xC7,0xC7,0xC7,0xC8,0xC8,0xC8,0xC8,0xC9,0xC9,0xC9,0xC9,0xCA,0xCA,0xCA,0xCA,0xCB,0xCB,0xCB,0xCB,0xCC,0xCC,0xCC,0xCC,0xCD,0xCD,0xCD,0xCD,0xCE,0xCE,0xCE,0xCE,0xCF,0xCF,0xCF,0xCF
.db 0xD0,0xD0,0xD0,0xD0,0xD1,0xD1,0xD1,0xD1,0xD2,0xD2,0xD2,0xD2,0xD3,0xD3,0xD3,0xD3,0xD4,0xD4,0xD4,0xD4,0xD5,0xD5,0xD5,0xD5,0xD6,0xD6,0xD6,0xD6,0xD7,0xD7,0xD7,0xD7,0xD8,0xD8,0xD8,0xD8,0xD9,0xD9,0xD9,0xD9,0xDA,0xDA,0xDA,0xDA,0xDB,0xDB,0xDB,0xDB,0xDC,0xDC,0xDC,0xDC,0xDD,0xDD,0xDD,0xDD,0xDE,0xDE,0xDE,0xDE,0xDF,0xDF,0xDF,0xDF
.db 0xD0,0xD0,0xD0,0xD0,0xD1,0xD1,0xD1,0xD1,0xD2,0xD2,0xD2,0xD2,0xD3,0xD3,0xD3,0xD3,0xD4,0xD4,0xD4,0xD4,0xD5,0xD5,0xD5,0xD5,0xD6,0xD6,0xD6,0xD6,0xD7,0xD7,0xD7,0xD7,0xD8,0xD8,0xD8,0xD8,0xD9,0xD9,0xD9,0xD9,0xDA,0xDA,0xDA,0xDA,0xDB,0xDB,0xDB,0xDB,0xDC,0xDC,0xDC,0xDC,0xDD,0xDD,0xDD,0xDD,0xDE,0xDE,0xDE,0xDE,0xDF,0xDF,0xDF,0xDF
.db 0xD0,0xD0,0xD0,0xD0,0xD1,0xD1,0xD1,0xD1,0xD2,0xD2,0xD2,0xD2,0xD3,0xD3,0xD3,0xD3,0xD4,0xD4,0xD4,0xD4,0xD5,0xD5,0xD5,0xD5,0xD6,0xD6,0xD6,0xD6,0xD7,0xD7,0xD7,0xD7,0xD8,0xD8,0xD8,0xD8,0xD9,0xD9,0xD9,0xD9,0xDA,0xDA,0xDA,0xDA,0xDB,0xDB,0xDB,0xDB,0xDC,0xDC,0xDC,0xDC,0xDD,0xDD,0xDD,0xDD,0xDE,0xDE,0xDE,0xDE,0xDF,0xDF,0xDF,0xDF
.db 0xD0,0xD0,0xD0,0xD0,0xD1,0xD1,0xD1,0xD1,0xD2,0xD2,0xD2,0xD2,0xD3,0xD3,0xD3,0xD3,0xD4,0xD4,0xD4,0xD4,0xD5,0xD5,0xD5,0xD5,0xD6,0xD6,0xD6,0xD6,0xD7,0xD7,0xD7,0xD7,0xD8,0xD8,0xD8,0xD8,0xD9,0xD9,0xD9,0xD9,0xDA,0xDA,0xDA,0xDA,0xDB,0xDB,0xDB,0xDB,0xDC,0xDC,0xDC,0xDC,0xDD,0xDD,0xDD,0xDD,0xDE,0xDE,0xDE,0xDE,0xDF,0xDF,0xDF,0xDF
.db 0xE0,0xE0,0xE0,0xE0,0xE1,0xE1,0xE1,0xE1,0xE2,0xE2,0xE2,0xE2,0xE3,0xE3,0xE3,0xE3,0xE4,0xE4,0xE4,0xE4,0xE5,0xE5,0xE5,0xE5,0xE6,0xE6,0xE6,0xE6,0xE7,0xE7,0xE7,0xE7,0xE8,0xE8,0xE8,0xE8,0xE9,0xE9,0xE9,0xE9,0xEA,0xEA,0xEA,0xEA,0xEB,0xEB,0xEB,0xEB,0xEC,0xEC,0xEC,0xEC,0xED,0xED,0xED,0xED,0xEE,0xEE,0xEE,0xEE,0xEF,0xEF,0xEF,0xEF
.db 0xE0,0xE0,0xE0,0xE0,0xE1,0xE1,0xE1,0xE1,0xE2,0xE2,0xE2,0xE2,0xE3,0xE3,0xE3,0xE3,0xE4,0xE4,0xE4,0xE4,0xE5,0xE5,0xE5,0xE5,0xE6,0xE6,0xE6,0xE6,0xE7,0xE7,0xE7,0xE7,0xE8,0xE8,0xE8,0xE8,0xE9,0xE9,0xE9,0xE9,0xEA,0xEA,0xEA,0xEA,0xEB,0xEB,0xEB,0xEB,0xEC,0xEC,0xEC,0xEC,0xED,0xED,0xED,0xED,0xEE,0xEE,0xEE,0xEE,0xEF,0xEF,0xEF,0xEF
.db 0xE0,0xE0,0xE0,0xE0,0xE1,0xE1,0xE1,0xE1,0xE2,0xE2,0xE2,0xE2,0xE3,0xE3,0xE3,0xE3,0xE4,0xE4,0xE4,0xE4,0xE5,0xE5,0xE5,0xE5,0xE6,0xE6,0xE6,0xE6,0xE7,0xE7,0xE7,0xE7,0xE8,0xE8,0xE8,0xE8,0xE9,0xE9,0xE9,0xE9,0xEA,0xEA,0xEA,0xEA,0xEB,0xEB,0xEB,0xEB,0xEC,0xEC,0xEC,0xEC,0xED,0xED,0xED,0xED,0xEE,0xEE,0xEE,0xEE,0xEF,0xEF,0xEF,0xEF
.db 0xE0,0xE0,0xE0,0xE0,0xE1,0xE1,0xE1,0xE1,0xE2,0xE2,0xE2,0xE2,0xE3,0xE3,0xE3,0xE3,0xE4,0xE4,0xE4,0xE4,0xE5,0xE5,0xE5,0xE5,0xE6,0xE6,0xE6,0xE6,0xE7,0xE7,0xE7,0xE7,0xE8,0xE8,0xE8,0xE8,0xE9,0xE9,0xE9,0xE9,0xEA,0xEA,0xEA,0xEA,0xEB,0xEB,0xEB,0xEB,0xEC,0xEC,0xEC,0xEC,0xED,0xED,0xED,0xED,0xEE,0xEE,0xEE,0xEE,0xEF,0xEF,0xEF,0xEF
.db 0xF0,0xF0,0xF0,0xF0,0xF1,0xF1,0xF1,0xF1,0xF2,0xF2,0xF2,0xF2,0xF3,0xF3,0xF3,0xF3,0xF4,0xF4,0xF4,0xF4,0xF5,0xF5,0xF5,0xF5,0xF6,0xF6,0xF6,0xF6,0xF7,0xF7,0xF7,0xF7,0xF8,0xF8,0xF8,0xF8,0xF9,0xF9,0xF9,0xF9,0xFA,0xFA,0xFA,0xFA,0xFB,0xFB,0xFB,0xFB,0xFC,0xFC,0xFC,0xFC,0xFD,0xFD,0xFD,0xFD,0xFE,0xFE,0xFE,0xFE,0xFF,0xFF,0xFF,0xFF
.db 0xF0,0xF0,0xF0,0xF0,0xF1,0xF1,0xF1,0xF1,0xF2,0xF2,0xF2,0xF2,0xF3,0xF3,0xF3,0xF3,0xF4,0xF4,0xF4,0xF4,0xF5,0xF5,0xF5,0xF5,0xF6,0xF6,0xF6,0xF6,0xF7,0xF7,0xF7,0xF7,0xF8,0xF8,0xF8,0xF8,0xF9,0xF9,0xF9,0xF9,0xFA,0xFA,0xFA,0xFA,0xFB,0xFB,0xFB,0xFB,0xFC,0xFC,0xFC,0xFC,0xFD,0xFD,0xFD,0xFD,0xFE,0xFE,0xFE,0xFE,0xFF,0xFF,0xFF,0xFF
.db 0xF0,0xF0,0xF0,0xF0,0xF1,0xF1,0xF1,0xF1,0xF2,0xF2,0xF2,0xF2,0xF3,0xF3,0xF3,0xF3,0xF4,0xF4,0xF4,0xF4,0xF5,0xF5,0xF5,0xF5,0xF6,0xF6,0xF6,0xF6,0xF7,0xF7,0xF7,0xF7,0xF8,0xF8,0xF8,0xF8,0xF9,0xF9,0xF9,0xF9,0xFA,0xFA,0xFA,0xFA,0xFB,0xFB,0xFB,0xFB,0xFC,0xFC,0xFC,0xFC,0xFD,0xFD,0xFD,0xFD,0xFE,0xFE,0xFE,0xFE,0xFF,0xFF,0xFF,0xFF
.db 0xF0,0xF0,0xF0,0xF0,0xF1,0xF1,0xF1,0xF1,0xF2,0xF2,0xF2,0xF2,0xF3,0xF3,0xF3,0xF3,0xF4,0xF4,0xF4,0xF4,0xF5,0xF5,0xF5,0xF5,0xF6,0xF6,0xF6,0xF6,0xF7,0xF7,0xF7,0xF7,0xF8,0xF8,0xF8,0xF8,0xF9,0xF9,0xF9,0xF9,0xFA,0xFA,0xFA,0xFA,0xFB,0xFB,0xFB,0xFB,0xFC,0xFC,0xFC,0xFC,0xFD,0xFD,0xFD,0xFD,0xFE,0xFE,0xFE,0xFE,0xFF,0xFF,0xFF,0xFF











